Package: chk
Title: Check User-Supplied Function Arguments
Version: 0.10.0
Authors@R: c(
    person("Joe", "Thorley", , "joe@poissonconsulting.ca", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-7683-4592")),
    person("Kirill", "Müller", role = "aut",
           comment = c(ORCID = "0000-0002-1416-3412")),
    person("Ayla", "Pearson", role = "aut",
           comment = c(ORCID = "0000-0001-7388-1222")),
    person("Florencia", "D'Andrea", role = "ctb"),
    person("Nadine", "Hussein", , "nadine@poissonconsulting.ca", role = "ctb",
           comment = c(ORCID = "0000-0003-4470-8361")),
    person("Evan", "Amies-Galonski", , "evan@poissonconsulting.ca", role = "ctb",
           comment = c(ORCID = "0000-0003-1096-2089")),
    person("Poisson Consulting", role = c("cph", "fnd"))
  )
Description: For developers to check user-supplied function arguments.  It
    is designed to be simple, fast and customizable.  Error messages
    follow the tidyverse style guide.
License: MIT + file LICENSE
URL: https://github.com/poissonconsulting/chk,
        https://poissonconsulting.github.io/chk/
BugReports: https://github.com/poissonconsulting/chk/issues/
Depends: R (>= 3.6)
Imports: lifecycle, methods, rlang, tools
Suggests: covr, knitr, rmarkdown, testthat (>= 3.0.0), withr
VignetteBuilder: knitr
RdMacros: lifecycle
Config/Needs/website: poissonconsulting/poissontemplate
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.2.9000
NeedsCompilation: no
Packaged: 2025-01-24 19:31:20 UTC; joe
Author: Joe Thorley [aut, cre] (<https://orcid.org/0000-0002-7683-4592>),
  Kirill Müller [aut] (<https://orcid.org/0000-0002-1416-3412>),
  Ayla Pearson [aut] (<https://orcid.org/0000-0001-7388-1222>),
  Florencia D'Andrea [ctb],
  Nadine Hussein [ctb] (<https://orcid.org/0000-0003-4470-8361>),
  Evan Amies-Galonski [ctb] (<https://orcid.org/0000-0003-1096-2089>),
  Poisson Consulting [cph, fnd]
Maintainer: Joe Thorley <joe@poissonconsulting.ca>
Repository: CRAN
Date/Publication: 2025-01-24 21:20:05 UTC
Built: R 4.6.0; ; 2026-04-27 05:39:31 UTC; unix

# Generated by roxygen2: do not edit by hand

export(abort_chk)
export(backtick_chk)
export(cc)
export(check_data)
export(check_dim)
export(check_dirs)
export(check_files)
export(check_key)
export(check_length)
export(check_names)
export(check_values)
export(chk_all)
export(chk_all_equal)
export(chk_all_equivalent)
export(chk_all_identical)
export(chk_array)
export(chk_atomic)
export(chk_character)
export(chk_character_or_factor)
export(chk_chr)
export(chk_compatible_lengths)
export(chk_complex)
export(chk_complex_number)
export(chk_count)
export(chk_data)
export(chk_date)
export(chk_date_time)
export(chk_datetime)
export(chk_dbl)
export(chk_dir)
export(chk_dirs)
export(chk_double)
export(chk_environment)
export(chk_equal)
export(chk_equivalent)
export(chk_ext)
export(chk_factor)
export(chk_false)
export(chk_file)
export(chk_files)
export(chk_flag)
export(chk_function)
export(chk_gt)
export(chk_gte)
export(chk_has)
export(chk_identical)
export(chk_in)
export(chk_integer)
export(chk_is)
export(chk_join)
export(chk_length)
export(chk_lgl)
export(chk_list)
export(chk_logical)
export(chk_lt)
export(chk_lte)
export(chk_match)
export(chk_matrix)
export(chk_missing)
export(chk_named)
export(chk_no_missing)
export(chk_not_any_na)
export(chk_not_empty)
export(chk_not_missing)
export(chk_not_null)
export(chk_not_subset)
export(chk_null)
export(chk_null_or)
export(chk_number)
export(chk_numeric)
export(chk_off)
export(chk_on)
export(chk_orderset)
export(chk_proportion)
export(chk_range)
export(chk_raw)
export(chk_s3_class)
export(chk_s4_class)
export(chk_scalar)
export(chk_setequal)
export(chk_sorted)
export(chk_string)
export(chk_subset)
export(chk_superset)
export(chk_true)
export(chk_tz)
export(chk_unique)
export(chk_unused)
export(chk_used)
export(chk_valid_name)
export(chk_vector)
export(chk_whole_number)
export(chk_whole_numeric)
export(chk_wnum)
export(chkor)
export(chkor_vld)
export(deparse_backtick)
export(deparse_backtick_chk)
export(err)
export(expect_chk_error)
export(is_chk_on)
export(message_chk)
export(msg)
export(p)
export(p0)
export(unbacktick_chk)
export(vld_all)
export(vld_all_equal)
export(vld_all_equivalent)
export(vld_all_identical)
export(vld_array)
export(vld_atomic)
export(vld_character)
export(vld_character_or_factor)
export(vld_chr)
export(vld_compatible_lengths)
export(vld_complex)
export(vld_complex_number)
export(vld_count)
export(vld_data)
export(vld_date)
export(vld_date_time)
export(vld_datetime)
export(vld_dbl)
export(vld_dir)
export(vld_double)
export(vld_environment)
export(vld_equal)
export(vld_equivalent)
export(vld_ext)
export(vld_factor)
export(vld_false)
export(vld_file)
export(vld_flag)
export(vld_function)
export(vld_gt)
export(vld_gte)
export(vld_identical)
export(vld_integer)
export(vld_is)
export(vld_join)
export(vld_length)
export(vld_lgl)
export(vld_list)
export(vld_logical)
export(vld_lt)
export(vld_lte)
export(vld_match)
export(vld_matrix)
export(vld_missing)
export(vld_named)
export(vld_no_missing)
export(vld_not_any_na)
export(vld_not_empty)
export(vld_not_missing)
export(vld_not_null)
export(vld_not_subset)
export(vld_null)
export(vld_number)
export(vld_numeric)
export(vld_orderset)
export(vld_range)
export(vld_raw)
export(vld_s3_class)
export(vld_s4_class)
export(vld_scalar)
export(vld_setequal)
export(vld_sorted)
export(vld_string)
export(vld_subset)
export(vld_superset)
export(vld_true)
export(vld_tz)
export(vld_unique)
export(vld_unused)
export(vld_used)
export(vld_valid_name)
export(vld_vector)
export(vld_whole_number)
export(vld_whole_numeric)
export(vld_wnum)
export(wrn)
import(lifecycle)
import(rlang)
importFrom(lifecycle,deprecated)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# chk <img src="man/figures/logo.png" align="right" alt="chk logo of completed tick mark" />

<!-- badges: start -->

[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![R-CMD-check](https://github.com/poissonconsulting/chk/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/poissonconsulting/chk/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/poissonconsulting/chk/graph/badge.svg)](https://app.codecov.io/gh/poissonconsulting/chk)
[![License:
MIT](https://img.shields.io/badge/License-MIT-green.svg)](https://opensource.org/license/mit)
[![CRAN
status](https://www.r-pkg.org/badges/version/chk)](https://cran.r-project.org/package=chk)
![CRAN downloads](https://cranlogs.r-pkg.org/badges/chk)
<!-- badges: end -->

`chk` is an R package for developers to check user-supplied function
arguments.

It is designed to be simple, customizable and fast.

## Demonstration

`chk` provides simple commonly used checks as (`chk_` functions) which
can be combined together for more complex checking.

``` r
library(chk)

y <- "a"

chk_string(y)
chk_flag(y)
#> Error:
#> ! `y` must be a flag (TRUE or FALSE).

data <- data.frame(x = 1:2)
chk_range(nrow(data), c(3, 8))
#> Error:
#> ! `nrow(data)` must be between 3 and 8, not 2.
```

Or used inside functions to test user-provided arguments.

``` r
my_fun <- function(x) {
  chk_flag(x)
  x
}
my_fun(TRUE)
#> [1] TRUE
my_fun(NA)
#> Error in `my_fun()`:
#> ! `x` must be a flag (TRUE or FALSE).
```

Error messages follow the [tidyverse style
guide](https://style.tidyverse.org/errors.html) while the errors
themselves are [rlang
errors](https://rlang.r-lib.org/reference/abort.html) of class
`chk_error`.

## Information

For more information see the [Get
Started](https://poissonconsulting.github.io/chk/articles/chk.html)
vignette.

## Installation

### Release

To install the release version from
[CRAN](https://CRAN.R-project.org/package=chk).

``` r
install.packages("chk")
```

The website for the release version is at
<https://poissonconsulting.github.io/chk/>.

### Development

To install the development version from
[GitHub](https://github.com/poissonconsulting/chk)

``` r
# install.packages("remotes")
remotes::install_github("poissonconsulting/chk")
```

or from [r-universe](https://poissonconsulting.r-universe.dev/chk).

``` r
install.packages("chk", repos = c("https://poissonconsulting.r-universe.dev", "https://cloud.r-project.org"))
```

## Inspiration

- [datacheckr](https://github.com/poissonconsulting/datacheckr/)
- [checkr](https://github.com/poissonconsulting/checkr/)
- [err](https://github.com/poissonconsulting/err/)
- [testthat](https://github.com/r-lib/testthat/)

## Contribution

Please report any
[issues](https://github.com/poissonconsulting/chk/issues).

[Pull requests](https://github.com/poissonconsulting/chk/pulls) are
always welcome.

## Code of Conduct

Please note that the chk project is released with a [Contributor Code of
Conduct](https://contributor-covenant.org/version/2/0/CODE_OF_CONDUCT.html).
By contributing to this project, you agree to abide by its terms.

#' Check All Equal
#'
#' @description
#' Checks all elements in x equal using
#'
#'  `length(x) < 2L || all(vapply(x, vld_equal, TRUE, y = x[[1]], tolerance = tolerance))`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family equal_checkers
#' @family all_checkers
#' @seealso [length()]
#' @seealso [vld_equal()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_all_equal
#' chk_all_equal(c(1, 1.00000001))
#' try(chk_all_equal(c(1, 1.0000001)))
#' chk_all_equal(list(c(x = 1), c(x = 1)))
#' try(chk_all_equal(list(c(x = 1), c(y = 1))))
#' @export
chk_all_equal <- function(x, tolerance = sqrt(.Machine$double.eps), x_name = NULL) {
  if (vld_all_equal(x, tolerance = tolerance)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must have equal elements", x = x, tolerance = tolerance)
}

#' @describeIn chk_all_equal Validate All Equal
#'
#' @examples
#' # vld_all_equal
#' vld_all_equal(c(1, 1L))
#' @export
vld_all_equal <- function(x, tolerance = sqrt(.Machine$double.eps)) {
  length(x) < 2L || all(vapply(x, vld_equal, TRUE, y = x[[1]], tolerance = tolerance))
}

test_that("vld_all", {
  expect_true(vld_all(character(0), vld_true))
  expect_true(vld_all(NULL, vld_null))
  expect_false(vld_all(NULL, vld_not_null))
  expect_false(vld_all(FALSE, vld_true))
  expect_true(vld_all(FALSE, vld_false))
})

test_that("chk_all", {
  expect_identical(chk_all(character(0), chk_true), character(0), chk_true)
  expect_invisible(chk_all(character(0), chk_true))
  expect_identical(chk_all(NULL, chk_null), NULL, chk_null)
  expect_invisible(chk_all(NULL, chk_null))
  expect_identical(chk_all(TRUE, chk_true), TRUE, chk_true)
  expect_identical(chk_all(FALSE, chk_false), FALSE, chk_false)
  expect_identical(chk_all(c(TRUE, TRUE), chk_true), c(TRUE, TRUE), chk_true)
  expect_identical(chk_all(1.1, chk_gt, 1), 1.1, chk_gt, 1)

  expect_chk_error(
    chk_all(NULL, chk_flag),
    "^`NULL` must be a flag [(]TRUE or FALSE[)][.]$"
  )
  expect_chk_error(
    chk_all(NULL, chk_flag, x_name = "NULL"),
    "^NULL must be a flag [(]TRUE or FALSE[)][.]$"
  )
  expect_chk_error(
    chk_all(FALSE, chk_true),
    "^All elements of `FALSE` must be TRUE[.]$"
  )
  expect_chk_error(
    chk_all(FALSE, chk_true, x_name = TRUE),
    "^All elements of TRUE must be TRUE[.]$"
  )
  expect_chk_error(
    chk_all(1.1, chk_gt, 2),
    "^All elements of `1.1` must be greater than 2, not 1.1[.]$"
  )
})

test_that("vld_all_identical", {
  expect_true(vld_all_identical(NULL))
  expect_true(vld_all_identical(character(0)))
  expect_true(vld_all_identical(1))
  expect_true(vld_all_identical(c(TRUE, TRUE)))
  expect_true(vld_all_identical(c(NA, NA)))
  expect_false(vld_all_identical(c(1, 2)))
})

test_that("chk_all_identical", {
  expect_identical(chk_all_identical(NULL), NULL)
  expect_invisible(chk_all_identical(NULL))
  expect_chk_error(chk_all_identical(c(1, 2)), "^`c[(]1, 2[)]` must have identical elements[.]$")
})


test_that("vld_all_equal", {
  expect_true(vld_all_equal(NULL))
  expect_true(vld_all_equal(character(0)))
  expect_true(vld_all_equal(1))
  expect_true(vld_all_equal(c(TRUE, TRUE)))
  expect_true(vld_all_equal(c(NA, NA)))
  expect_false(vld_all_equal(c(1, 2)))
  expect_false(vld_all_equal(c(1, 1.0000001), tolerance = 1.490116e-08))
  expect_true(vld_all_equal(c(1, 1.00000001), tolerance = 1.490116e-08))
  expect_true(vld_all_equal(list(c(x = 1), c(x = 1))))
  expect_false(vld_all_equal(list(c(x = 1), c(y = 1))))
})

test_that("chk_all_equal", {
  expect_identical(chk_all_equal(NULL), NULL)
  expect_invisible(chk_all_equal(NULL), NULL)
  expect_chk_error(chk_all_equal(c(1, 2)), "^`c[(]1, 2[)]` must have equal elements[.]$")
})

test_that("vld_all_equivalent", {
  expect_true(vld_all_equivalent(NULL))
  expect_true(vld_all_equivalent(character(0)))
  expect_true(vld_all_equivalent(1))
  expect_true(vld_all_equivalent(c(TRUE, TRUE)))
  expect_true(vld_all_equivalent(c(NA, NA)))
  expect_false(vld_all_equivalent(c(1, 2)))
  expect_false(vld_all_equivalent(c(1, 1.0000001), tolerance = 1.490116e-08))
  expect_true(vld_all_equivalent(c(1, 1.00000001), tolerance = 1.490116e-08))
  expect_true(vld_all_equivalent(list(c(x = 1), c(x = 1))))
  expect_true(vld_all_equivalent(list(c(x = 1), c(y = 1))))
})

test_that("chk_all_equivalent", {
  expect_identical(chk_all_equivalent(NULL), NULL)
  expect_invisible(chk_all_equivalent(NULL))
  expect_chk_error(chk_all_equivalent(c(1, 2)), "^`c[(]1, 2[)]` must have equivalent elements[.]$")
})

#' Check Matrix
#'
#' @description
#' Checks if is a matrix using
#'
#' `is.matrix(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family data_structure_checkers
#'
#' @seealso [is.matrix()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_matrix
#' chk_matrix(matrix(1))
#' try(chk_matrix(array(1)))
#' @export
chk_matrix <- function(x, x_name = NULL) {
  if (vld_matrix(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be an matrix", x = x)
}

#' @describeIn chk_matrix Validate Matrix
#'
#' @examples
#' # vld_matrix
#' vld_matrix(1)
#' vld_matrix(matrix(1))
#' @export
vld_matrix <- function(x) is.matrix(x)

test_that("vld_matrix", {
  expect_false(vld_matrix(1))
  expect_false(vld_matrix(array(1)))
  expect_true(vld_matrix(matrix()))
  expect_true(vld_matrix(matrix(NA)))
  expect_true(vld_matrix(matrix(1)))
})

test_that("chk_matrix", {
  expect_identical(chk_matrix(matrix(1)), matrix(1))
  expect_invisible(chk_matrix(matrix(1)))
  expect_chk_error(
    chk_matrix(1),
    "^`1` must be an matrix[.]$"
  )
})

#' Check Set Equal
#'
#' @description
#' Checks if equal set using
#'
#' `setequal(x, values)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family set_checkers
#'
#' @seealso [setequal()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_setequal
#' chk_setequal(1:2, 2:1)
#' try(chk_setequal(1, 1:2))
#' @export
chk_setequal <- function(x, values, x_name = NULL) {
  if (vld_setequal(x, values)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must equal set: ", cc(values), x = x, values = values)
}

#' @describeIn chk_setequal Validate Set Equal
#'
#' @examples
#' # vld_setequal
#' vld_setequal(1, 1)
#' vld_setequal(1:2, 2:1)
#' vld_setequal(1, 2:1)
#' vld_setequal(1:2, 2)
#' @export
vld_setequal <- function(x, values) setequal(x, values)

test_that("vld_setequal", {
  expect_true(vld_setequal(1, 1))
  expect_true(vld_setequal(1L, 1))
  expect_true(vld_setequal(character(0), character(0)))
  expect_false(vld_setequal(1:2, 1))
  expect_false(vld_setequal(1, 1:2))
})

test_that("chk_setequal", {
  expect_identical(chk_setequal(1, 1), 1, 1)
  expect_invisible(chk_setequal(1, 1))
  expect_chk_error(chk_setequal(1, 1:2), "^`1` must equal set: 1, 2[.]$")
  expect_chk_error(chk_setequal(1, 1:2, x_name = 2), "^2 must equal set: 1, 2[.]$")
})

#' Checks range of non-missing values
#'
#' @description
#' Checks all non-missing values fall within range using
#'
#' If inclusive
#'
#' `all(x[!is.na(x)] >= range[1] & x[!is.na(x)] <= range[2])`
#'
#' else
#'
#' `all(x[!is.na(x)] > range[1] & x[!is.na(x)] < range[2])`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family range_checkers
#'
#' @seealso [all()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_range
#' chk_range(0)
#' try(chk_range(-0.1))
#' @export
chk_range <- function(x, range = c(0, 1), inclusive = TRUE, x_name = NULL) {
  if (vld_range(x, range, inclusive)) {
    return(invisible(x))
  }

  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))

  if (length(x) == 1L) {
    if (inclusive && range[1] == range[2]) {
      abort_chk(x_name, " must be ", cc(range[1]), ", not ", cc(x), x = x, range = range)
    }
    abort_chk(
      x_name, " must be between ", cc(range, " and "),
      ifelse(isTRUE(inclusive), "", " exclusive"),
      ", not ", cc(x)
    )
  }
  if (inclusive && range[1] == range[2]) {
    abort_chk(x_name, " must have values of ", cc(range[1]), x = x, range = range)
  }
  abort_chk(x_name, " must have values between ", cc(range, " and "),
    ifelse(isTRUE(inclusive), "", " exclusive"),
    x = x, range = range
  )
}

#' @describeIn chk_range Validate Range
#'
#' @examples
#' # vld_range
#' vld_range(numeric(0))
#' vld_range(0)
#' vld_range(-0.1)
#' vld_range(c(0.1, 0.2, NA))
#' vld_range(c(0.1, 0.2, NA), range = c(0, 1))
#' @export
vld_range <- function(x, range = c(0, 1), inclusive = TRUE) {
  if (isTRUE(inclusive)) {
    return(all(x[!is.na(x)] >= range[1] & x[!is.na(x)] <= range[2]))
  }
  all(x[!is.na(x)] > range[1] & x[!is.na(x)] < range[2])
}

test_that("vld_range", {
  expect_true(vld_range(numeric(0)))
  expect_true(vld_range(numeric(0), c(1, 1)))
  expect_true(vld_range(NA_real_))
  expect_true(vld_range(1))
  expect_true(vld_range(1:3 / 3))
  expect_true(vld_range(numeric(0), 1L))

  expect_false(vld_range(-1))
  expect_true(vld_range(c(-1, -1), c(-2, -1)))
  expect_true(vld_range(c(NA, 1), 1:2))
})

test_that("chk_range", {
  expect_identical(chk_range(1), 1)
  expect_invisible(chk_range(1))

  expect_chk_error(chk_range(-1), "^`-1` must be between 0 and 1, not -1[.]$")
  expect_chk_error(chk_range(c(-1, NA)), "^`c[(]-1, NA[)]` must have values between 0 and 1[.]$")
  expect_chk_error(chk_range(2, c(1, 1)), "^`2` must be 1, not 2[.]$")
  expect_chk_error(chk_range(c(2, 1), c(1, 1)), "^`c[(]2, 1[)]` must have values of 1[.]$")
  expect_chk_error(chk_range(2, c(1, 1), x_name = 1), "^1 must be 1, not 2[.]$")

  expect_chk_error(chk_range(-1, inclusive = FALSE), "^`-1` must be between 0 and 1 exclusive, not -1[.]$")
  expect_chk_error(chk_range(c(-1, NA), inclusive = FALSE), "^`c[(]-1, NA[)]` must have values between 0 and 1 exclusive[.]$")
  expect_chk_error(chk_range(2, c(1, 1), inclusive = FALSE), "^`2` must be between 1 and 1 exclusive, not 2[.]$")
  expect_chk_error(chk_range(c(2, 1), c(1, 1), inclusive = FALSE), "^`c[(]2, 1[)]` must have values between 1 and 1 exclusive[.]$")
  expect_chk_error(chk_range(2, c(3, 4), inclusive = FALSE, x_name = 1), "^1 must be between 3 and 4 exclusive, not 2[.]$")
})

test_that("vld_lt", {
  expect_true(vld_lt(numeric(0)))
  expect_true(vld_lt(NA))
  expect_true(vld_lt(-1))
  expect_true(vld_lt(c(-1, -Inf)))
  expect_false(vld_lt(0))
  expect_false(vld_lt(1))
})

test_that("chk_lt", {
  expect_identical(chk_lt(-1), -1)
  expect_invisible(chk_lt(-1))
  expect_chk_error(chk_lt(1), "^`1` must be less than 0, not 1[.]$")
  expect_chk_error(chk_lt(c(1, -1)), "^`c[(]1, -1[)]` must have values less than 0[.]$")
  expect_chk_error(chk_lt(1, x_name = -1), "^-1 must be less than 0, not 1[.]$")
})

test_that("vld_lte", {
  expect_true(vld_lte(numeric(0)))
  expect_true(vld_lte(NA))
  expect_true(vld_lte(-1))
  expect_true(vld_lte(c(-1, -Inf)))
  expect_true(vld_lte(0))
  expect_false(vld_lte(1))
})

test_that("chk_lte", {
  expect_identical(chk_lte(-1), -1)
  expect_invisible(chk_lte(-1))
  expect_chk_error(chk_lte(1), "^`1` must be less than or equal to 0, not 1[.]$")
  expect_chk_error(chk_lte(c(1, -1)), "^`c[(]1, -1[)]` must have values less than or equal to 0[.]$")
  expect_chk_error(chk_lte(1, x_name = "0"), "^0 must be less than or equal to 0, not 1[.]$")
})

test_that("vld_gt", {
  expect_true(vld_gt(numeric(0)))
  expect_true(vld_gt(NA))
  expect_true(vld_gt(1))
  expect_true(vld_gt(c(1, Inf)))
  expect_false(vld_gt(0))
  expect_false(vld_gt(-1))
})

test_that("chk_gt", {
  expect_identical(chk_gt(1), 1)
  expect_invisible(chk_gt(1))
  expect_chk_error(chk_gt(-1), "^`-1` must be greater than 0, not -1[.]$")
  expect_chk_error(chk_gt(c(1, -1)), "^`c[(]1, -1[)]` must have values greater than 0[.]$")
  expect_chk_error(chk_gt(-1, x_name = "`a number`"), "^`a number` must be greater than 0, not -1[.]$")
})

test_that("vld_gte", {
  expect_true(vld_gte(numeric(0)))
  expect_true(vld_gte(NA))
  expect_true(vld_gte(1))
  expect_true(vld_gte(c(1, Inf)))
  expect_true(vld_gte(0))
  expect_false(vld_gte(-1))
})

test_that("chk_gte", {
  expect_identical(chk_gte(1), 1)
  expect_invisible(chk_gte(1))
  expect_chk_error(chk_gte(-1), "^`-1` must be greater than or equal to 0, not -1[.]$")
  expect_chk_error(chk_gte(c(1, -1)), "^`c[(]1, -1[)]` must have values greater than or equal to 0[.]$")
  expect_chk_error(chk_gte(-1, x_name = "another number"), "^Another number must be greater than or equal to 0, not -1[.]$")
})

#' Check Names
#'
#' @description
#' Checks the names of an object.
#'
#' @inheritParams params
#' @param names A character vector of the required names.
#' @param exclusive A flag specifying whether x must only contain the required names.
#' @param order A flag specifying whether the order of the required names in x must match the order in names.
#' @return An informative error if the test fails or an invisible copy of x.
#'
#' @family check
#'
#' @examples
#' x <- c(x = 1, y = 2)
#' check_names(x, c("y", "x"))
#' try(check_names(x, c("y", "x"), order = TRUE))
#' try(check_names(x, "x", exclusive = TRUE))
#' @export
check_names <- function(x, names = character(0), exclusive = FALSE, order = FALSE,
                        x_name = NULL) {
  chk_s3_class(names, "character")
  chk_unique(names)
  chk_not_any_na(names)
  chk_flag(exclusive)
  chk_flag(order)
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  chk_string(x_name)
  chk_named(x, x_name = x_name)

  x_names <- names(x)

  if (!length(names)) {
    if (exclusive && length(x_names)) {
      abort_chk(x_name, " must not have any elements")
    }
    return(invisible(x))
  }

  x_name <- backtick_chk(p0("names(", unbacktick_chk(x_name), ")"))

  if (length(setdiff(names, x_names))) {
    abort_chk(x_name, " must include ", cc(setdiff(names, x_names), conj = " and "))
  }

  if (exclusive && length(setdiff(x_names, names))) {
    abort_chk(x_name, " must not include ", cc(setdiff(x_names, names), conj = " or "))
  }

  if (order && !identical(intersect(names, x_names), intersect(x_names, names))) {
    abort_chk(x_name, " must include ", cc(names, conj = " and "), " in that order")
  }
  invisible(x)
}

test_that("check_names", {
  expect_identical(check_names(c(x = 1)), c(x = 1))
  expect_invisible(check_names(c(x = 1)))
  expect_identical(check_names(c(x = 1), "x"), check_names(c(x = 1), "x"))
  expect_identical(
    check_names(c(x = 1, y = 2), "x"),
    check_names(c(x = 1, y = 2), "x")
  )
  expect_identical(
    check_names(c(x = 1, y = 2), c("y", "x")),
    check_names(c(x = 1, y = 2), c("y", "x"))
  )
})

test_that("check_names fails", {
  expect_chk_error(
    check_names(character(0)),
    "^`character[(]0[)]` must be named[.]$"
  )
  x <- structure(list(), .Names = character(0))
  x <- structure(list(), .Names = character(0))
  expect_chk_error(
    check_names(c(x = 1), exclusive = TRUE),
    "^`c[(]x = 1[)]` must not have any elements[.]$"
  )
  expect_chk_error(
    check_names(x, "x"),
    "^`names[(]x[)]` must include 'x'[.]$"
  )
  expect_chk_error(
    check_names(c(x = 1), c("x", "y")),
    "`names[(]c[(]x = 1[)][)]` must include 'y'[.]$"
  )
  expect_chk_error(
    check_names(c(x = 1, z = 2), "x", exclusive = TRUE),
    "^`names[(]c[(]x = 1, z = 2[)][)]` must not include 'z'[.]$"
  )

  expect_chk_error(
    check_names(c(x = 1, y = 2), c("y", "x"), order = TRUE),
    "`names[(]c[(]x = 1, y = 2[)][)]` must include 'y' and 'x' in that order[.]$"
  )
  expect_chk_error(
    check_names(c(x = 1, y = 2), c("y", "x"), order = TRUE, x_name = "b"),
    "`names[(]b[)]` must include 'y' and 'x' in that order[.]$"
  )
})

#' Check Files Exist
#'
#' @description
#' Checks if all files exist (or if exists = FALSE do not exist as files or directories).
#'
#' @inheritParams params
#' @return An informative error if the test fails or an invisible copy of x.
#'
#' @family check
#'
#' @examples
#' check_files(tempfile("unlikely-that-exists-chk"), exists = FALSE)
#' try(check_files(tempfile("unlikely-that-exists-chk")))
#' @export
check_files <- function(x, exists = TRUE, x_name = NULL) {
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  chk_string(x_name)
  chk_character(x, x_name = x_name)
  chk_vector(x, x_name = x_name)
  chk_not_any_na(x, x_name = x_name)
  chk_flag(exists)

  dirs <- vapply(x, vld_dir, TRUE)
  if (any(dirs)) {
    abort_chk(x_name, " must specify files ('", x[dirs][1], "' is a directory)", x = x)
  }
  x <- x[vapply(x, vld_file, TRUE) != exists]
  if (!length(x)) {
    return(invisible(x))
  }
  x <- x[1]
  if (exists) {
    abort_chk(x_name, " must specify existing files ('", x, "' can't be found)", x = x)
  }
  abort_chk(x_name, " must not specify existing files ('", x, "' exists)", x = x)
}

test_that("check_files works", {
  expect_identical(check_files(character(0)), character(0))
  expect_invisible(check_files(character(0)))
  expect_identical(
    check_files(character(0), exists = TRUE),
    check_files(character(0), exists = TRUE)
  )
  expect_invisible(check_files(character(0), exists = TRUE))

  tmp <- withr::local_tempfile()
  expect_identical(check_files(tmp, exists = FALSE), check_files(tmp, exists = FALSE))
  expect_invisible(check_files(tmp, exists = FALSE))
  writeLines(tmp, text = "some test data")
  expect_identical(check_files(tmp, exists = TRUE), check_files(tmp, exists = TRUE))
  expect_invisible(check_files(tmp, exists = TRUE))
})

test_that("check_files errors", {
  expect_chk_error(check_files(NA_character_))

  tmp <- withr::local_tempfile()
  expect_chk_error(
    check_files(tmp),
    "^`tmp` must specify existing files [(]'.*' can't be found[)][.]$"
  )
  expect_chk_error(
    check_files(tempdir()),
    "^`tempdir[(][)]` must specify files [(]'.*' is a directory[)][.]$"
  )
  expect_chk_error(
    check_files(tempdir(), exists = FALSE),
    "^`tempdir[(][)]` must specify files [(]'.*' is a directory[)][.]$"
  )
  writeLines(tmp, text = "some test data")
  expect_chk_error(
    check_files(tmp, exists = FALSE),
    "^`tmp` must not specify existing files [(]'.*' exists[)][.]$"
  )
})

#' Check Join
#'
#' @description
#' Checks if all rows in x match at least one in y.
#'
#' @inheritParams params
#' @param y A data.frame with columns in by.
#' @inherit params return
#'
#' @family misc_checkers
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_join
#' chk_join(data.frame(z = 1), data.frame(z = 1:2), by = "z")
#' try(chk_join(data.frame(z = 1), data.frame(z = 2), by = "z"))
#' @export
chk_join <- function(x, y, by, x_name = NULL) {
  if (vld_join(x, y, by)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  y_name <- as_label(y)
  obj_of <- if (grepl("^<", y_name)) "the y object of class " else NULL
  abort_chk("All rows in ", x_name, " must match at least one in ",
    obj_of, y_name,
    x = x, y = y, by = by
  )
}

#' @describeIn chk_join Validate Join
#'
#' @examples
#' # vld_join
#' vld_join(data.frame(z = 1), data.frame(z = 1:2), by = "z")
#' vld_join(data.frame(z = 1), data.frame(z = 2), by = "z")
#' vld_join(data.frame(z = 1), data.frame(a = 1:2), by = c(z = "a"))
#' vld_join(data.frame(z = 1), data.frame(a = 2), by = c(z = "a"))
#' @export
vld_join <- function(x, y, by) {
  if (is.null(by)) {
    return(FALSE)
  }
  identical(
    nrow(x),
    nrow(merge(as.data.frame(x), unique(as.data.frame(y)[by]),
      by.x = if (is.null(names(by))) by else names(by), by.y = by
    ))
  )
}

test_that("vld_join", {
  expect_true(vld_join(data.frame(z = 1), data.frame(z = 1:2), by = "z"))
  expect_false(vld_join(data.frame(z = 1), data.frame(z = 2), by = "z"))
  expect_true(vld_join(data.frame(z = 1:2), data.frame(z = 1:2), by = "z"))
})

test_that("vld_join with by = NULL", {
  expect_false(vld_join(data.frame(z = 1), data.frame(z = 1:2), by = NULL))
})

test_that("vld_join with names", {
  expect_true(vld_join(data.frame(z = 1), data.frame(a = 1:2), by = c(z = "a")))
  expect_false(vld_join(data.frame(z = 1), data.frame(a = 2), by = c(z = "a")))
  expect_error(
    vld_join(data.frame(z = 1), data.frame(a = 2), by = c(a = "z")),
    "undefined columns selected"
  )
  expect_true(vld_join(data.frame(z = 1, z2 = 2), data.frame(z = 1:2, z2 = 2:1), by = "z"))
  expect_true(vld_join(data.frame(z = 1, z2 = 2), data.frame(z = 1:2, z2 = 2:1), by = c("z", "z2")))
  expect_true(vld_join(data.frame(z = 1, z2 = 2), data.frame(z = 1:2, z2 = 1:2), by = "z"))
  expect_false(vld_join(data.frame(z = 1, z2 = 2), data.frame(z = 1:2, z2 = 1:2), by = c("z", "z2")))
})

test_that("vld_join with zero length", {
  expect_true(vld_join(data.frame(z = numeric(0)), data.frame(z = 2), by = c("z")))
  expect_true(vld_join(data.frame(z = numeric(0)), data.frame(z = numeric(0)), by = c("z")))
  expect_false(vld_join(data.frame(z = 1), data.frame(z = numeric(0)), by = c("z")))
})

test_that("vld_join with missing values", {
  expect_false(vld_join(data.frame(z = NA_integer_), data.frame(z = 2), by = c("z")))
  expect_false(vld_join(data.frame(z = 2), data.frame(z = NA_integer_), by = c("z")))
  expect_true(vld_join(data.frame(z = NA_integer_), data.frame(z = NA_integer_), by = c("z")))
})

test_that("chk_join", {
  expect_identical(
    chk_join(data.frame(z = 1), data.frame(z = 1), by = "z"),
    chk_join(data.frame(z = 1), data.frame(z = 1), by = "z")
  )
  expect_invisible(chk_join(data.frame(z = 1), data.frame(z = 1), by = "z"))
})

test_that("chk_join", {
  expect_chk_error(
    chk_join(data.frame(z = 1), data.frame(z = 2), by = "z"),
    "^All rows in `data.frame[(]z [=] 1[)]` must match at least one in the y object of class <d"
  )
})

#' Check Less Than or Equal To
#'
#' @description
#' Checks if all non-missing values are less than or equal to y using
#'
#' `all(x[!is.na(x)] <= value)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family range_checkers
#'
#' @seealso [all()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_lte
#' chk_lte(0)
#' try(chk_lte(0.1))
#' @export
chk_lte <- function(x, value = 0, x_name = NULL) {
  if (vld_lte(x, value)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  if (length(x) == 1L) {
    abort_chk(
      x_name, " must be less than or equal to ", cc(value), ", not ", cc(x),
      ""
    )
  }
  abort_chk(x_name, " must have values less than or equal to ", cc(value), x = x, value = value)
}

#' @describeIn chk_lte Validate Less Than or Equal To
#'
#' @examples
#' # vld_lte
#' vld_lte(numeric(0))
#' vld_lte(0)
#' vld_lte(0.1)
#' vld_lte(c(-0.1, -0.2, NA))
#' vld_lte(c(-0.1, -0.2, NA), value = -1)
#' @export
vld_lte <- function(x, value = 0) all(x[!is.na(x)] <= value)

#' Check Count
#'
#' @description
#' Checks if non-negative whole number using
#'
#' `vld_whole_number(x) && x >= 0`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family scalar_checkers
#' @family whole_number_checkers
#'
#' @seealso [vld_whole_number()]
#' @seealso [length()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_count
#' chk_count(1)
#' try(chk_count(1.5))
#' @export
chk_count <- function(x, x_name = NULL) {
  if (vld_count(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be a count (non-negative non-missing integer scalar or double equivalent)", x = x)
}

#' @describeIn chk_count Validate Count
#'
#' @examples
#' # vld_count
#' vld_count(1)
#' vld_count(0L)
#' vld_count(-1)
#' vld_count(0.5)
#' @export
vld_count <- function(x) {
  vld_whole_number(x) && x >= 0
}

test_that("vld_count", {
  expect_true(vld_count(1))
  expect_true(vld_count(0))
  expect_true(vld_count(100L))
  expect_false(vld_count(0.5))
  expect_false(vld_count(-1L))
  expect_false(vld_count(NA_integer_))
  expect_false(vld_count(integer(0)))
})

test_that("chk_count", {
  expect_identical(chk_count(1), 1)
  expect_invisible(chk_count(1))
  expect_chk_error(chk_count(1.1), "^`1.1` must be a count [(]non-negative non-missing integer scalar or double equivalent[)][.]$")
  expect_chk_error(chk_count(1.1, x_name = 1L), "^1 must be a count [(]non-negative non-missing integer scalar or double equivalent[)][.]$")
})

#' Check Array
#'
#' @description
#' Checks if is an array using
#'
#' `is.array(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family data_structure_checkers
#'
#' @seealso [is.array()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_array
#' chk_array(array(1))
#' try(chk_array(matrix(1)))
#' @export
chk_array <- function(x, x_name = NULL) {
  if (vld_array(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be an array", x = x)
}

#' @describeIn chk_array Validate Array
#'
#' @examples
#'
#' # vld_array
#' vld_array(1)
#' vld_array(array(1))
#' @export
vld_array <- function(x) is.array(x)

test_that("vld_array", {
  expect_false(vld_array(1))
  expect_true(vld_array(array()))
  expect_true(vld_array(array(NA)))
  expect_true(vld_array(array(1)))
})

test_that("chk_array", {
  expect_identical(chk_array(array(1)), array(1))
  expect_invisible(chk_array(array(1)))
  expect_chk_error(
    chk_array(1),
    "^`1` must be an array[.]$"
  )
})

#' Check Whole Numeric
#'
#' @description
#' Checks if integer vector or double equivalent using
#'
#' `is.integer(x) || (is.double(x) && vld_true(all.equal(x, as.integer(x))))`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family whole_number_checkers
#'
#' @seealso [is.integer()]
#' @seealso [is.double()]
#' @seealso [vld_true()]
#' @seealso [all.equal()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_whole_numeric
#' chk_whole_numeric(1)
#' try(chk_whole_numeric(1.1))
#' @export
chk_whole_numeric <- function(x, x_name = NULL) {
  if (vld_whole_numeric(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(
    x_name,
    " must be a whole numeric vector (integer vector or double equivalent)",
    x = x
  )
}

#' @describeIn chk_whole_numeric Validate Whole Numeric
#'
#' @examples
#' # vld_whole_numeric
#' vld_whole_numeric(1)
#' vld_whole_numeric(NA_real_)
#' vld_whole_numeric(1:2)
#' vld_whole_numeric(double(0))
#' vld_whole_numeric(TRUE)
#' vld_whole_numeric(1.5)
#' @export
vld_whole_numeric <- function(x) {
  is.integer(x) || (is.double(x) &&
    vld_true(all.equal(x[!is.na(x)], trunc(x[!is.na(x)]))))
}

test_that("vld_s3_class", {
  expect_true(vld_s3_class(1, "numeric"))
  expect_false(vld_s3_class(1L, "numeric"))
  expect_true(vld_s3_class(1L, "integer"))

  x <- list()
  class(x) <- c("a", "b")
  expect_true(vld_s3_class(x, "a"))
  expect_true(vld_s3_class(x, "b"))
  expect_false(vld_s3_class(getClass("MethodDefinition"), "classRepresentation"))
})

test_that("chk_s3_class", {
  expect_identical(chk_s3_class(1L, "integer"), 1L, "integer")
  expect_invisible(chk_s3_class(1L, "integer"))
  expect_chk_error(chk_s3_class(1, "integer"), "^`1` must inherit from S3 class 'integer'[.]$")
  expect_chk_error(
    chk_s3_class(matrix(1), "numeric"),
    "`matrix[(]1[)]` must inherit from S3 class 'numeric'[.]$"
  )
  x <- list()
  class(x) <- c("a", "b")
  expect_chk_error(chk_s3_class(x, "c"), "`x` must inherit from S3 class 'c'")
  expect_chk_error(chk_s3_class(x, "c", x_name = "c"), "C must inherit from S3 class 'c'")
})


test_that("vld_s4_class", {
  expect_false(vld_s4_class(1, "numeric"))
  expect_true(vld_s4_class(getClass("MethodDefinition"), "classRepresentation"))
})

test_that("chk_s4_class", {
  expect_identical(
    chk_s4_class(getClass("MethodDefinition"), "classRepresentation"),
    getClass("MethodDefinition"), "classRepresentation"
  )
  expect_invisible(chk_s4_class(getClass("MethodDefinition"), "classRepresentation"))
  expect_chk_error(chk_s4_class(1, "integer"), "^`1` must inherit from S4 class 'integer'[.]$")
  expect_chk_error(
    chk_s4_class(matrix(1), "numeric"),
    "`matrix[(]1[)]` must inherit from S4 class 'numeric'[.]$"
  )
})

test_that("vld_whole_numeric", {
  expect_true(vld_whole_numeric(numeric(0)))
  expect_true(vld_whole_numeric(integer(0)))
  expect_false(vld_whole_numeric(logical(0)))
  expect_true(vld_whole_numeric(Inf))
  expect_true(vld_whole_numeric(NA_integer_))
  expect_true(vld_whole_numeric(NA_real_))
  expect_false(vld_whole_numeric(NA))
  expect_true(vld_whole_numeric(1))
  expect_true(vld_whole_numeric(1L))
  expect_false(vld_whole_numeric(1.1))
  expect_false(vld_whole_numeric(-1.1))
  expect_true(vld_whole_numeric(c(1, 2)))
  expect_false(vld_whole_numeric(TRUE))
})

test_that("chk_whole_numeric", {
  expect_identical(chk_whole_numeric(1L), 1L)
  expect_invisible(chk_whole_numeric(1L))
  expect_identical(chk_whole_numeric(1), 1)
  expect_identical(chk_whole_numeric(Inf), Inf)
  expect_chk_error(chk_whole_numeric(TRUE), "^`TRUE` must be a whole numeric vector [(]integer vector or double equivalent[)][.]$")
  expect_chk_error(chk_whole_numeric(TRUE, x_name = "`c(1,2)`"), "^`c[(]1,2[)]` must be a whole numeric vector [(]integer vector or double equivalent[)][.]$")
})

test_that("vld_list", {
  expect_true(vld_list(list()))
  expect_true(vld_list(list(1)))
  expect_true(vld_list(list(x = 1)))
  x <- list(x = 1)
  class(x) <- "nlist"
  expect_true(vld_list(x))
  expect_false(vld_list(1))
})

test_that("chk_list", {
  expect_identical(chk_list(list()), list())
  expect_invisible(chk_list(list()))
  expect_chk_error(chk_list(1), "^`1` must be a list[.]$")
  expect_chk_error(chk_list(1, x_name = "`list()`"), "^`list[(][)]` must be a list[.]$")
})

test_that("vld_function", {
  expect_true(vld_function(c))
  expect_true(vld_function(function() {}))
  expect_false(vld_function(1))
  expect_true(vld_function(function() {}, formals = 0))
  expect_false(vld_function(function() {}, formals = 1))
  expect_true(vld_function(function(x = 1) {}, formals = 1))
  expect_false(vld_function(function(x = 1, y = 3) {}, formals = 1))
})

test_that("chk_function", {
  expect_identical(chk_function(c), c)
  expect_invisible(chk_function(c))
  expect_chk_error(chk_function(1), "^`1` must be a function[.]$")
  expect_chk_error(chk_function(1, x_name = "Function()"), "^Function[(][)] must be a function[.]$")
  expect_chk_error(
    chk_function(mean, formals = 3),
    "^`mean` must have 3 formal arguments [(]not 2[)][.]$"
  )
})

test_that("vld_scalar", {
  expect_true(vld_scalar(1))
  expect_true(vld_scalar(NA))
  expect_false(vld_scalar(integer(0)))
  expect_false(vld_scalar(1:2))
  expect_false(vld_scalar(list()))
  expect_true(vld_scalar(list(x = 1)))
  expect_true(vld_scalar(matrix(1)))
  expect_false(vld_scalar(matrix(1:2)))
  expect_true(vld_scalar(data.frame(x = 1:10)))
  expect_false(vld_scalar(data.frame(x = 1, y = 2)))
})

test_that("chk_scalar", {
  expect_identical(chk_scalar(1), 1)
  expect_invisible(chk_scalar(1))
  expect_chk_error(chk_scalar(1:2), "^`1:2` must be a scalar [(]length 1[)][.]$")
})

# from https://stevenmortimer.com/the-unfinished-duplicated-function/
new_duplicated <- function(x, incomparables = FALSE, fromLast = FALSE, ...) {
  if (!identical(incomparables, FALSE)) {
    n <- ncol(x)
    nmx <- names(x)
    nmincomparables <- names(incomparables)
    lincomparables <- length(incomparables)
    if (is.null(nmincomparables)) {
      if (lincomparables < n) {
        # pad any incomparables lists with the default value if list is shorter than the number columns in the supplied data.frame
        tmp <- c(incomparables, as.list(rep_len(FALSE, n - lincomparables)))
        names(tmp) <- nmx
        incomparables <- tmp
      }
      if (lincomparables > n) {
        # if the list is unnamed and there are more elements in the list than there are columns, then only first n elements
        warning(paste("more columns in 'incomparables' than x, only using the first", n, "elements"))
        incomparables <- incomparables[1:n]
      }
    } else {
      # for named lists, find match, else default value
      tmp <- as.list(rep_len(FALSE, n))
      names(tmp) <- nmx
      i <- match(nmincomparables, nmx, 0L)
      if (any(i <= 0L)) {
        warning("not all columns named in 'incomparables' exist")
      }
      tmp[i[i > 0L]] <- incomparables[i > 0L]
      incomparables <- tmp[nmx]
    }

    # first determine duplicates, then override when an incomparable value is found in a row since the existence of even 1 incomparable value in a row means it cannot be a duplicate
    res <- duplicated(do.call("paste", c(x, sep = "\r")), fromLast = fromLast)

    # for better performance only bother with the columns that have incomparable values not set to the default: !identical(x, FALSE)
    run_incomp_check <- sapply(incomparables, FUN = function(x) {
      !identical(x, FALSE)
    })
    if (sum(run_incomp_check) > 0L) {
      incomp_check <- mapply(FUN = function(column, incomparables) {
        match(column, incomparables)
      }, x[run_incomp_check], incomparables[run_incomp_check])
      # any rows with an incomparable match means, TRUE, it can override the duplicated result
      overwrite <- apply(data.frame(incomp_check), 1, function(x) {
        any(!is.na(x))
      })
      res[overwrite] <- FALSE
    }

    return(res)
  } else if (length(x) != 1L) {
    duplicated(do.call("paste", c(x, sep = "\r")), fromLast = fromLast)
  } else {
    duplicated(x[[1L]], fromLast = fromLast, ...)
  }
}

#' Check Raw
#'
#' @description
#' Checks if raw using
#'
#' `is.raw(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family data_type_checkers
#'
#' @seealso [is.raw()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_raw
#' chk_raw(as.raw(1))
#' try(chk_raw(1))
#' @export
chk_raw <- function(x, x_name = NULL) {
  if (vld_raw(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be raw", x = x)
}

#' @describeIn chk_raw Validate Raw
#'
#' @examples
#' # vld_raw
#' vld_raw(as.raw(1))
#' vld_raw(raw(0))
#' vld_raw(1)
#' vld_raw(TRUE)
#' @export
vld_raw <- function(x) is.raw(x)

test_that("vld_raw", {
  expect_false(vld_raw(1L))
  expect_false(vld_raw(matrix(1:4)))
  expect_true(vld_raw(raw(0)))
  expect_true(vld_raw(as.raw(1)))
  expect_true(vld_raw(as.raw(c(1,2))))
  expect_false(vld_raw(1))
  expect_false(vld_raw(TRUE))
})

test_that("chk_raw", {
  expect_identical(chk_raw(as.raw(1L)), as.raw(1L))
  expect_invisible(chk_raw(as.raw(1L)))
  expect_chk_error(
    chk_raw(TRUE),
    "^`TRUE` must be raw[.]$"
  )
})

#' Check Matches
#'
#' @description
#' Checks if all values match regular expression using
#'
#' `all(grepl(regexp, x[!is.na(x)]))`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family regex_checkers
#'
#' @seealso [all()]
#' @seealso [grepl()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_match
#' chk_match("1")
#' try(chk_match("1", regexp = "2"))
#' @export
chk_match <- function(x, regexp = ".+", x_name = NULL) {
  if (vld_match(x, regexp)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))

  if (length(x) == 1L) {
    abort_chk(x_name, " must match regular expression '", regexp, "'", x = x, regexp = regexp)
  }
  abort_chk(x_name, " must have values matching regular expression '", regexp, "'", x = x, regexp = regexp)
}

#' @describeIn chk_match Validate Matches
#'
#' @examples
#' # vld_match
#' vld_match("1")
#' vld_match("a", regexp = "a")
#' vld_match("")
#' vld_match("1", regexp = "2")
#' vld_match(NA_character_, regexp = ".*")
#' @export
vld_match <- function(x, regexp = ".+") all(grepl(regexp, x[!is.na(x)]))

test_that("vld_string", {
  expect_false(vld_string(character(0)))
  expect_false(vld_string(NA_character_))
  expect_true(vld_string(""))
  expect_true(vld_string("a"))
  expect_false(vld_string(c("a", "b")))
})

test_that("chk_string", {
  expect_identical(chk_string(""), "")
  expect_invisible(chk_string(""))
  expect_chk_error(chk_string(1), "^`1` must be a string [(]non-missing character scalar[)][.]$")
  expect_chk_error(chk_string(1, x_name = "''"), "^'' must be a string [(]non-missing character scalar[)][.]$")
})

test_that("vld_match", {
  expect_true(vld_match(character(0)))
  expect_true(vld_match(1))
  expect_true(vld_match("a"))
  expect_true(vld_match(NA))
  expect_true(vld_match(c(NA, "b"), "b"))
  expect_false(vld_match(""))
  expect_true(vld_match(c("a", "b")))
})

test_that("chk_match", {
  expect_identical(chk_match("1"), "1")
  expect_invisible(chk_match("1"))
  expect_chk_error(chk_match(""), "^`\"\"` must match regular expression '.+'[.]$")
  expect_chk_error(
    chk_match(c("a", "b"), "b"),
    "`c[(]\"a\", \"b\"[)]` must have values matching regular expression 'b'[.]$"
  )
  expect_chk_error(
    chk_match(c("a", "b"), "b"),
    "`c[(]\"a\", \"b\"[)]` must have values matching regular expression 'b'[.]$"
  )
})

#' Check Logical
#'
#' @description
#' Checks if logical using
#'
#' `is.logical(x)`
#'
#' If you want to check if it is a scalar,
#' meaning that in addition to being of logical type,
#' it has `length(x) == 1`, you should use [chk_lgl()]
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family logical_checkers
#' @family data_type_checkers
#'
#' @seealso [is.logical()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_logical
#' chk_logical(TRUE)
#' try(chk_logical(1))
#' @export
chk_logical <- function(x, x_name = NULL) {
  if (vld_logical(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be logical", x = x)
}

#' @describeIn chk_logical Validate Logical
#'
#' @examples
#' # vld_logical
#' vld_logical(TRUE)
#' vld_logical(matrix(TRUE))
#' vld_logical(logical(0))
#' vld_logical(NA)
#' vld_logical(1)
#' vld_logical("TRUE")
#' @export
vld_logical <- function(x) is.logical(x)

test_that("vld_logical", {
  expect_true(vld_logical(TRUE))
  expect_true(vld_logical(matrix(TRUE)))
  expect_true(vld_logical(logical(0)))
  expect_true(vld_logical(NA))
  expect_false(vld_logical(1))
  expect_false(vld_logical("TRUE"))
})

test_that("chk_logical", {
  expect_identical(chk_logical(TRUE), TRUE)
  expect_invisible(chk_logical(FALSE))
  expect_chk_error(
    chk_logical(1),
    "^`1` must be logical[.]$"
  )
})

#' Check Function
#'
#' @description
#' Checks if is a function using
#'
#' `is.function(x) && (is.null(formals) || length(formals(x)) == formals)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family function_checkers
#' @family ellpisis_checkers
#' @family missing_checkers
#'
#' @seealso [is.function()] [formals()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_function
#' chk_function(mean)
#' try(chk_function(1))
#' @export
chk_function <- function(x, formals = NULL, x_name = NULL) {
  if (vld_function(x, formals)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  if (!is.function(x)) abort_chk(x_name, " must be a function", x = x, formals = formals)
  abort_chk(x_name, " must have ", formals, " formal arguments (not ", length(formals(x)), ")", x = x, formals = formals)
}

#' @describeIn chk_function Validate Function
#'
#' @examples
#' # vld_function
#' vld_function(mean)
#' vld_function(function(x) x)
#' vld_function(1)
#' vld_function(list(1))
#' @export
vld_function <- function(x, formals = NULL) {
  is.function(x) && (is.null(formals) || length(formals(x)) == formals)
}

#' Check Dimension
#'
#' @description
#' Checks dimension of an object.
#'
#' @inheritParams params
#' @param dim A function returning a non-negative whole number of the dimension.
#' @param values A flag or a whole numeric vector of the value, value range or possible values.
#' @param dim_name A string of the name of the dim function.
#' @return An informative error if the test fails or an invisible copy of x.
#'
#' @family check
#'
#' @examples
#' check_dim(1)
#' try(check_dim(1, values = FALSE))
#' try(check_dim(1, values = c(10, 2)))
#' try(check_dim(data.frame(x = 1), dim = nrow, values = c(10, 10, 2)))
#' @export
check_dim <- function(x, dim = length, values = numeric(0), x_name = NULL, dim_name = NULL) {
  chk_function(dim)
  if (!vld_flag(values) && !vld_whole_numeric(values)) {
    chkor_vld(vld_flag(values), vld_whole_numeric(values))
  }
  chk_whole_number(dim(x))
  chk_gte(dim(x))

  if (is.null(x_name)) x_name <- deparse((substitute(x)))
  chk_string(x_name)
  if (is.null(dim_name)) dim_name <- deparse((substitute(dim)))
  chk_string(dim_name)

  x_name <- backtick_chk(p0(dim_name, "(", x_name, ")"))

  if (!length(values)) {
    return(invisible(x))
  }

  if (vld_true(values)) {
    return(chk_gt(dim(x), x_name = x_name))
  }
  if (vld_false(values)) values <- 0L
  if (identical(length(values), 2L)) {
    if (!identical(values[1], values[2])) {
      return(chk_range(dim(x), sort(values), x_name = x_name))
    }
    values <- unique(values)
  }
  if (identical(length(values), 1L)) {
    return(chk_equal(dim(x), values, x_name = x_name))
  }
  chk_subset(dim(x), sort(unique(values)), x_name = x_name)
}

test_that("check_dim length", {
  expect_identical(check_dim(1), 1)
  expect_invisible(check_dim(1))
  expect_chk_error(
    check_dim(1:2, dim = identity),
    "`dim[(]x[)]` must be a whole number [(]non-missing integer scalar or double equivalent[)][.]"
  )
  expect_identical(check_dim(integer(0)), integer(0))
  expect_chk_error(
    check_dim(integer(0), values = TRUE),
    "^`length[(]integer[(]0[)][)]` must be greater than 0, not 0[.]$"
  )
  expect_chk_error(
    check_dim(1, values = FALSE),
    "^`length[(]1[)]` must be equal to 0L[.]"
  )
  expect_identical(check_dim(2, values = 1), check_dim(2, values = 1))
  expect_chk_error(
    check_dim(1, values = 2),
    "^`length[(]1[)]` must be equal to 2[.]"
  )
  expect_chk_error(
    check_dim(1, values = 2:3),
    "^`length[(]1[)]` must be between 2 and 3, not 1[.]"
  )
  expect_chk_error(
    check_dim(1, values = 3:2),
    "^`length[(]1[)]` must be between 2 and 3, not 1[.]"
  )
  expect_identical(check_dim(1:2, values = 2:3), check_dim(1:2, values = 2:3))
  expect_chk_error(
    check_dim(2, values = c(2, 2)),
    "`length[(]2[)]` must be equal to 2[.]"
  )
  expect_chk_error(
    check_dim(2, values = c(2, 2, 2)),
    "`length[(]2[)]` must match 2, not 1[.]"
  )
  expect_chk_error(
    check_dim(2, values = c(3, 2, 2)),
    "`length[(]2[)]` must match 2 or 3, not 1[.]"
  )

  expect_chk_error(
    check_dim(1, values = 1.1), "^At least one of the following conditions must be met:\n[*] `values` must be a flag [(]TRUE or FALSE[)].\n[*] `values` must be a whole numeric vector \\(integer vector or double equivalent\\)\\.$"
  )
})

test_that("check_dim nrow", {
  expect_identical(
    check_dim(data.frame(x = 1), dim = nrow),
    check_dim(data.frame(x = 1), dim = nrow)
  )
  expect_invisible(check_dim(data.frame(x = 1), dim = nrow))
  expect_chk_error(
    check_dim(data.frame(x = integer(0)), values = TRUE, dim = nrow),
    "`nrow[(]data.frame[(]x = integer[(]0[)][)][)]` must be greater than 0, not 0[.]"
  )
  expect_chk_error(
    check_dim(data.frame(x = integer(0)), dim = nrow, dim_name = "ncol", values = TRUE),
    "`ncol[(]data.frame[(]x = integer[(]0[)][)][)]` must be greater than 0, not 0[.]"
  )
  expect_chk_error(
    check_dim(data.frame(x = integer(0)), dim = nrow, x_name = "y", dim_name = "ncol", values = TRUE),
    "`ncol[(]y[)]` must be greater than 0, not 0[.]"
  )
})

#' Check Character Scalar
#'
#' @description
#' Checks if character scalar using
#'
#' `is.character(x) && length(x) == 1L`
#'
#' `r lifecycle::badge("deprecated")`
#'
#' @family deprecated
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family chk_character
#'
#' @examples
#' chk_chr("a")
#' try(chk_chr(1))
#' @export
chk_chr <- function(x, x_name = NULL) {
  if (vld_chr(x)) {
    return(invisible(x))
  }
  deprecate_soft(
    "0.6.1",
    what = "chk::chk_chr()",
    details = "Please use `chk::chk_scalar(x);` `chk::chk_character(x)` instead",
    id = "chk_chr"
  )

  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be a character scalar", x = x)
}

#' @describeIn chk_chr Validate Character Scalar
#'
#' `r lifecycle::badge("deprecated")`
#'
#' @examples
#' # vld_chr
#' vld_chr("")
#' vld_chr("a")
#' vld_chr(NA_character_)
#' vld_chr(c("a", "b"))
#' vld_chr(1)
#' @export
vld_chr <- function(x) {
  deprecate_soft(
    "0.6.1",
    what = "chk::chk_chr()",
    details = "Please use `chk::chk_scalar(x);` `chk::chk_character(x)` instead",
    id = "chk_chr"
  )

  is.character(x) && length(x) == 1L
}

test_that("vld_chr", {
  rlang::local_options(lifecycle_verbosity = "quiet")

  lifecycle::expect_deprecated(chk_chr(""))

  expect_false(vld_chr(character(0)))
  expect_true(vld_chr(NA_character_))
  expect_true(vld_chr(""))
  expect_true(vld_chr("a"))
  expect_false(vld_chr(c("a", "b")))
})

test_that("chk_chr", {
  rlang::local_options(lifecycle_verbosity = "quiet")

  lifecycle::expect_deprecated(chk_chr(""))

  expect_identical(chk_chr(""), "")
  expect_invisible(chk_chr(""))
  expect_chk_error(chk_chr(1), "^`1` must be a character scalar[.]$")
  expect_chk_error(chk_chr(1, x_name = "''"), "^'' must be a character scalar[.]$")
})

#' Concatenate Strings
#'
#' A wrapper on [base::paste()].
#'
#' @inheritParams base::paste
#'
#' @return A character vector.
#' @export
#'
#' @examples
#' p("a", "b")
#' p(c("a", "b"), collapse = " ")
p <- function(..., sep = " ", collapse = NULL) {
  paste(..., sep = sep, collapse = collapse)
}

#' @describeIn p A wrapper on [base::paste0()]
#' @export
#'
#' @examples
#' p0("a", "b")
#' p0(c("a", "b"), collapse = "")
p0 <- function(..., collapse = NULL) {
  paste0(..., collapse = collapse)
}

#' Check Atomic
#'
#' @description
#' Checks if atomic using
#'
#' `is.atomic(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family data_structure_checkers
#'
#' @seealso [is.atomic()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_atomic
#' chk_atomic(1)
#' try(chk_atomic(list(1)))
#' @export
chk_atomic <- function(x, x_name = NULL) {
  if (vld_atomic(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be atomic", x = x)
}

#' @describeIn chk_atomic Validate Atomic
#'
#' @examples
#' # vld_atomic
#' vld_atomic(1)
#' vld_atomic(matrix(1:3))
#' vld_atomic(character(0))
#' vld_atomic(list(1))
#' vld_atomic(NULL)
#' @export
vld_atomic <- function(x) is.atomic(x)

test_that("vld_atomic", {
  expect_true(vld_atomic(1))
  expect_true(vld_atomic(matrix(1:3)))
  expect_true(vld_atomic(character(0)))
  expect_false(vld_atomic(list(1)))

  skip_on_cran()
  if (sub("(R version )((\\d+\\.){2,2}\\d+)(.*)", "\\2", R.version$version.string) <= "4.4.0") {
    expect_true(vld_atomic(NULL))
  } else {
    expect_false(vld_atomic(NULL))
  }
})

test_that("chk_atomic", {
  expect_identical(chk_atomic(1), 1)
  expect_invisible(chk_atomic(1))
  expect_chk_error(chk_atomic(list(1)), "^`list[(]1[)]` must be atomic[.]$")
  expect_chk_error(chk_atomic(list(1), x_name = 1), "^1 must be atomic[.]$")
})

#' Check Whole Numeric Scalar
#'
#' @description
#' Checks if whole numeric scalar using
#'
#' `is.numeric(x) && length(x) == 1L && (is.integer(x) || vld_true(all.equal(x, trunc(x))))`
#'
#' `r lifecycle::badge("deprecated")`
#'
#' @inheritParams params
#' @inherit params return
#'
#'
#' @family deprecated
#'
#' @examples
#' # chk_wnum
#' chk_wnum(1)
#' try(chk_wnum(1.1))
#' @export
chk_wnum <- function(x, x_name = NULL) {
  if (vld_wnum(x)) {
    return(invisible(x))
  }
  deprecate_soft(
    "0.6.1",
    what = "chk::chk_wnum()",
    details = "Please use `chk::chk_scalar(x);` `chk::chk_whole_numeric(x)` instead",
    id = "chk_wnum"
  )
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be a whole numeric scalar", x = x)
}

#' @describeIn chk_wnum Validate Whole Numeric Scalar
#'
#' `r lifecycle::badge("deprecated")`
#'
#' @examples
#' # vld_wnum
#' vld_wnum(1)
#' vld_wnum(double(0))
#' vld_wnum(NA_real_)
#' vld_wnum(c(1, 1))
#' vld_wnum(1L)
#' @export
vld_wnum <- function(x) {
  deprecate_soft(
    "0.6.1",
    what = "chk::chk_wnum()",
    details = "Please use `chk::chk_scalar(x);` `chk::chk_whole_numeric(x)` instead",
    id = "chk_wnum"
  )

  is.numeric(x) && length(x) == 1L && (is.integer(x) || vld_true(all.equal(x, trunc(x))))
}

test_that("vld_wnum", {
  rlang::local_options(lifecycle_verbosity = "quiet")

  lifecycle::expect_deprecated(vld_wnum(1))

  expect_false(vld_wnum(numeric(0)))
  expect_true(vld_wnum(NA_integer_))
  expect_true(vld_wnum(1))
  expect_true(vld_wnum(1L))
  expect_false(vld_wnum(1.1))
  expect_true(vld_wnum(-1))
  expect_true(vld_wnum(Inf))
  expect_false(vld_wnum(c(1, 2)))
  expect_false(vld_wnum(TRUE))
})

test_that("chk_wnum", {
  rlang::local_options(lifecycle_verbosity = "quiet")

  lifecycle::expect_deprecated(chk_wnum(1))

  expect_identical(chk_wnum(1), 1)
  expect_invisible(chk_wnum(1))
  expect_chk_error(chk_wnum(TRUE), "^`TRUE` must be a whole numeric scalar[.]$")
  expect_chk_error(chk_wnum(TRUE, x_name = 1L), "^1 must be a whole numeric scalar[.]$")
})

#' Check Integer
#'
#' @description
#' Checks if integer using
#'
#' `is.integer(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family data_type_checkers
#'
#' @seealso [is.integer()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_integer
#' chk_integer(1L)
#' try(chk_integer(1))
#' @export
chk_integer <- function(x, x_name = NULL) {
  if (vld_integer(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be integer", x = x)
}

#' @describeIn chk_integer Validate Integer
#'
#' @examples
#' # vld_integer
#' vld_integer(1L)
#' vld_integer(matrix(1:4, nrow = 2L))
#' vld_integer(integer(0))
#' vld_integer(NA_integer_)
#' vld_integer(1)
#' vld_integer(TRUE)
#' @export
vld_integer <- function(x) is.integer(x)

test_that("vld_integer", {
  expect_true(vld_integer(1L))
  expect_true(vld_integer(matrix(1:4)))
  expect_true(vld_integer(integer(0)))
  expect_true(vld_integer(NA_integer_))
  expect_false(vld_integer(1))
  expect_false(vld_integer(TRUE))
})

test_that("chk_integer", {
  expect_identical(chk_integer(1L), 1L)
  expect_invisible(chk_integer(1L))
  expect_chk_error(
    chk_integer(TRUE),
    "^`TRUE` must be integer[.]$"
  )
})

#' Check Numeric
#'
#' @description
#' Checks if numeric using
#'
#' `is.numeric(x)`
#'
#' **Pass**: `1`, `1:2`, `NA_real_`, `integer(0)`, `matrix(1:3)`.
#'
#' **Fail**: `TRUE`, `"1"`, `NA`, `NULL`.
#'
#' @inheritParams chk_flag
#' @inherit params return
#'
#' @family data_type_checkers
#'
#' @seealso [is.numeric()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_numeric
#' chk_numeric(1)
#' try(chk_numeric("1"))
#' @export
chk_numeric <- function(x, x_name = NULL) {
  if (vld_numeric(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be numeric", x = x)
}

#' @describeIn chk_numeric Validate Numeric
#'
#' @examples
#' # vld_numeric
#' vld_numeric(1)
#' vld_numeric(1:2)
#' vld_numeric(NA_real_)
#' vld_numeric(integer(0))
#' vld_numeric("1")
#' vld_numeric(TRUE)
#' @export
vld_numeric <- function(x) is.numeric(x)

test_that("vld_numeric", {
  expect_true(vld_numeric(1))
  expect_true(vld_numeric(matrix(1:3)))
  expect_true(vld_numeric(integer(0)))
  expect_false(vld_numeric(NULL))
  expect_false(vld_numeric("1"))
  expect_false(vld_numeric(NA))
})

test_that("chk_numeric", {
  expect_identical(chk_numeric(1), 1)
  expect_invisible(chk_numeric(1))
  expect_chk_error(chk_numeric("1"), "`\"1\"` must be numeric[.]$")
  expect_chk_error(chk_numeric("1", x_name = 1), "^1 must be numeric[.]$")
})

#' Check Complex Number
#'
#' @description
#'  Checks if non-missing complex scalar using
#'
#' `is.complex(x) && length(x) == 1L && !anyNA(x)`
#'
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family scalar_checker
#'
#' @seealso [is.integer()]
#' @seealso [vld_true()]
#' @seealso [vld_number()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' \href{https://poissonconsulting.github.io/chk/articles/chk-families.html}{chk families}.
#'
#' @examples
#' # chk_complex_number
#' chk_complex_number(as.complex(1.1))
#' try(chk_complex_number(1.1))
#' @export
chk_complex_number <- function(x, x_name = NULL) {
  if (vld_complex_number(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(
    x_name,
    " must be a complex number (non-missing complex scalar)",
    x = x
  )
}

#' @describeIn chk_complex_number Validate Complex Number
#'
#' @examples
#' # vld_complex_number
#' vld_complex_number(as.complex(2))
#' @export
vld_complex_number <- function(x) {
  is.complex(x) && length(x) == 1L && !anyNA(x)
}

test_that("vld_complex_number", {
  expect_false(vld_complex_number(complex(0)))
  expect_false(vld_complex_number(NA_complex_))
  expect_true(vld_complex_number(1i))
  expect_true(vld_complex_number(as.complex(1)))
  expect_false(vld_complex_number(as.complex(c(1, 2))))
  expect_false(vld_complex_number(TRUE))
})

test_that("chk_complex_number", {
  expect_identical(chk_complex_number(as.complex(1)), as.complex(1))
  expect_invisible(chk_complex_number(as.complex(1)))
  expect_chk_error(chk_complex_number(TRUE), "^`TRUE` must be a complex number [(]non-missing complex scalar[)][.]$")
  expect_chk_error(chk_complex_number(TRUE, x_name = 1L), "^1 must be a complex number [(]non-missing complex scalar[)][.]$")
})

#' Check NULL
#'
#' @description
#' Checks if NULL using
#'
#' `is.null(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family null_checkers
#'
#' @seealso [is.null()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_null
#' try(chk_null(1))
#' chk_null(NULL)
#' @export
chk_null <- function(x, x_name = NULL) {
  if (vld_null(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be NULL", x = x)
}

#' @describeIn chk_null Validate NULL
#'
#' @examples
#' # vld_null
#' vld_null(NULL)
#' vld_null(1)
#' @export
vld_null <- function(x) is.null(x)

test_that("vld_null", {
  expect_true(vld_null(NULL))
  expect_false(vld_null(1))
})

test_that("chk_null", {
  expect_identical(chk_null(NULL), NULL)
  expect_invisible(chk_null(NULL))

  expect_chk_error(chk_null(1), "^`1` must be NULL[.]$")
  expect_chk_error(chk_null(1, x_name = "NULL"), "^NULL must be NULL[.]$")
})

test_that("vld_not_null", {
  expect_true(vld_not_null(1))
  expect_false(vld_not_null(NULL))
})

test_that("chk_not_null", {
  expect_identical(chk_not_null(1), 1)
  expect_invisible(chk_not_null(1))

  expect_chk_error(chk_not_null(NULL), "^`NULL` must not be NULL[.]$")
  expect_chk_error(chk_not_null(NULL, x_name = "not NULL"), "^Not NULL must not be NULL[.]$")
})

#' Check Valid Name
#'
#' @description
#' Checks if valid name using
#'
#' `identical(make.names(x[!is.na(x)]), as.character(x[!is.na(x)]))`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family name_checkers
#'
#' @seealso [identical()]
#' @seealso [make.names()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_valid_name
#' chk_valid_name("text")
#' try(chk_valid_name(".1"))
#' @export
chk_valid_name <- function(x, x_name = NULL) {
  if (vld_valid_name(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be valid name%s", n = length(x))
}

#' @describeIn chk_valid_name Validate Valid Name
#'
#' @examples
#' # vld_valid_name
#' vld_valid_name(".1")
#' @export
vld_valid_name <- function(x) identical(make.names(x[!is.na(x)]), as.character(x[!is.na(x)]))

test_that("vld_valid_name", {
  expect_true(vld_valid_name(numeric(0)))
  expect_true(vld_valid_name(NA))
  expect_true(vld_valid_name("name"))
  expect_true(vld_valid_name(c("name", NA)))
  expect_true(vld_valid_name(c("name", "name")))
  expect_true(vld_valid_name("."))
  expect_false(vld_valid_name(".1"))
  expect_false(vld_valid_name("1"))
  expect_false(vld_valid_name(c("1", NA)))
})

test_that("chk_valid_name", {
  expect_identical(chk_valid_name("x"), "x")
  expect_invisible(chk_valid_name("x"))
  expect_chk_error(chk_valid_name(".1"), '^`".1"` must be valid name\\.$')
  expect_chk_error(chk_valid_name(".1", x_name = 1L), "^1 must be valid name\\.$")
})

#' Check ... Unused
#'
#' @description
#' Checks if ... is unused
#'
#' `length(list(...)) == 0L`
#'
#' @inheritParams params
#' @inherit params return
#' @return The `chk_` function throws an informative error if the test fails.
#'
#' @family ellipsis_checkers
#'
#' @seealso [length()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_unused
#' fun <- function(x, ...) {
#'   chk_unused(...)
#'   x
#' }
#' fun(1)
#' try(fun(1, 2))
#' @export
chk_unused <- function(...) {
  if (!length(list(...))) {
    return(invisible())
  }
  abort_chk("`...` must be unused")
}

#' @describeIn chk_unused Validate ... Unused
#'
#' @examples
#' # vld_unused
#' fun <- function(x, ...) {
#'   vld_unused(...)
#' }
#' fun(1)
#' try(fun(1, 2))
#' @export
vld_unused <- function(...) length(list(...)) == 0L

test_that("vld_unused", {
  expect_true(vld_unused())
  expect_false(vld_unused(1))
})

test_that("chk_unused", {
  expect_null(chk_unused())
  expect_invisible(chk_unused())
  expect_chk_error(chk_unused(1), "^`...` must be unused[.]$")
})

test_that("vld_used", {
  expect_true(vld_used(1))
  expect_false(vld_used())
})

test_that("chk_used", {
  expect_null(chk_used(1))
  expect_invisible(chk_used(1))
  expect_chk_error(chk_used(), "^`...` must be used[.]$")
})

#' Construct Tidyverse Style Message
#'
#' If `tidy = TRUE` constructs a tidyverse style message by
#'
#' \itemize{
#'   \item Capitalizing the first character if possible.
#'   \item Adding a trailing . if missing.
#' }
#'
#' Also if `n != NULL` replaces the recognized `sprintf`-like types.
#'
#' @section `sprintf`-like types:
#'
#' The following recognized `sprintf`-like types can be used in a message:
#'
#' \describe{
#'   \item{`n`}{The value of n.}
#'   \item{`s`}{'' if n == 1 otherwise 's'}
#'   \item{`r`}{'is' if n == 1 otherwise 'are'}
#'   \item{`y`}{'y' if n == 1 otherwise 'ie'}
#' }
#'
#' @param ... Multiple objects that are converted to a string using
#' `paste0(..., collapse = '')`.
#' @param n The value of n for converting `sprintf`-like types.
#' @param tidy A flag specifying whether capitalize the first character and add a missing period.
#'
#' @return A string of the message.
#' @export
#'
#' @examples
#' message_chk("there %r %n", " problem director%y%s")
#' message_chk("there %r %n", " problem director%y%s", n = 1)
#' message_chk("There %r %n", " problem director%y%s.", n = 3)
message_chk <- function(..., n = NULL, tidy = TRUE) {
  string <- p0(..., collapse = "")
  if (!is.null(n)) {
    string <- gsub("%r", if (n == 1) "is" else "are", string, fixed = TRUE)
    string <- gsub("%s", if (n == 1) "" else "s", string, fixed = TRUE)
    string <- gsub("%y", if (n == 1) "y" else "ie", string, fixed = TRUE)
    string <- gsub("%n", n, string, fixed = TRUE)
  }
  if (vld_true(tidy)) {
    if (!grepl("([.]|[?]|[!])$", string)) string <- p0(string, ".")
    string <- p0(toupper(substr(string, 1, 1)), substr(string, 2, nchar(string)))
  }
  string
}

#' Stop, Warning and Message Messages
#'
#' The functions call [message_chk()] to process
#' the message and then
#' [rlang::abort()], [rlang::warn()] and
#'  [rlang::inform()], respectively.
#'
#'  The user can set the subclass.
#' @param .subclass A string of the class of the error message.
#' @inheritParams base::stop
#' @inheritParams rlang::abort
#' @inheritParams message_chk
#' @name err
NULL

#' @describeIn err Error
#'
#' @export
#'
#' @examples
#'
#' # err
#' try(err("there %r %n problem value%s", n = 2))
err <- function(..., n = NULL, tidy = TRUE, .subclass = NULL, class = NULL, call = rlang::caller_call(3)) {
  if (!is.null(.subclass)) {
    deprecate_soft("0.8.1", "err(.subclass)", "err(class)")
    class <- .subclass
  }
  args <- list2(...)
  named <- (names2(args) != "")
  msg <- exec(p0, !!!args[!named], collapse = "")
  msg <- message_chk(msg, n = n, tidy = tidy)

  if (!is.null(args$y)) {
    msg <- paste0(
      msg,
      "\nUse `rlang::last_error()$y` to show the object compared to."
    )
  }

  rlang::abort(
    msg,
    class = class,
    !!!args[named],
    call = call
  )
}

#' @describeIn err Warning
#'
#' @export
#'
#' @examples
#'
#' # wrn
#' wrn("there %r %n problem value%s", n = 2)
wrn <- function(..., n = NULL, tidy = TRUE, .subclass = NULL, class = NULL) {
  if (!is.null(.subclass)) {
    deprecate_soft("0.8.1", "wrn(.subclass)", "wrn(class)")
    class <- .subclass
  }
  warn(message_chk(..., n = n, tidy = tidy), class = class)
}

#' @describeIn err Message
#'
#' @export
#'
#' @examples
#'
#' # msg
#' msg("there %r %n problem value%s", n = 2)
msg <- function(..., n = NULL, tidy = TRUE, .subclass = NULL, class = NULL) {
  if (!is.null(.subclass)) {
    deprecate_soft("0.8.1", "msg(.subclass)", "msg(class)")
    class <- .subclass
  }
  inform(message_chk(..., n = n, tidy = tidy), class = class)
}

test_that("message_chk", {
  expect_match(message_chk(), "^[.]$")
  expect_match(message_chk("", ""), "^[.]$")
  expect_match(message_chk("a"), "^A[.]$")
  expect_match(message_chk("`a`"), "^`a`[.]$")
  expect_match(message_chk("a", "b"), "^Ab[.]$")
  expect_match(message_chk(" a", " b"), "^ a b[.]$")
  expect_match(message_chk(" A", "b"), "^ Ab[.]$")
  expect_match(message_chk("Ab", "b"), "^Abb[.]$")
  expect_match(message_chk("Ab.", "b."), "^Ab[.]b[.]$")

  expect_match(message_chk("a"), "^A[.]$")
  expect_match(message_chk("a."), "^A[.]$")
  expect_match(message_chk("a!"), "^A!$")
  expect_match(message_chk("a?"), "^A[?]$")

  expect_match(
    message_chk("there %r %n problem director%y%s", n = 1),
    "There is 1 problem directory[.]$"
  )
  expect_match(
    message_chk("there %r %n problem director%y%s", n = 2),
    "There are 2 problem directories[.]$"
  )
  expect_match(
    message_chk("there %r %n problem director%y%s", n = 1.5),
    "There are 1.5 problem directories[.]$"
  )
  expect_match(
    message_chk("there %r %n problem director%y%s", n = 0),
    "There are 0 problem directories[.]$"
  )
  expect_match(
    message_chk("there %r %n problem director%y%s", n = 0, tidy = FALSE),
    "there are 0 problem directories$"
  )
})

test_that("err", {
  expect_error(err(), "^[.]$")
  expect_error(err("this", "is"), "^Thisis[.]$")
  expect_error(
    err("There %r %n problem value%s"),
    "There %r %n problem value%s[.]$"
  )
  expect_error(
    err("there %r %n problem value%s", n = 2),
    "There are 2 problem values[.]$"
  )
  expect_error(
    err("there %r %n problem value%s", n = 0, tidy = FALSE),
    "there are 0 problem values$"
  )
})

test_that("err .subclass deprecated", {
  lifecycle::expect_deprecated(expect_s3_class(expect_error(err(.subclass = "xx"), "^[.]$"), "xx"))
  expect_s3_class(expect_error(err(class = "xx"), "^[.]$"), "xx")
})

test_that("msg", {
  expect_message(msg(), "^[.]")
  expect_message(msg("this", "is"), "^Thisis[.]")
  expect_message(
    msg("There %r %n problem value%s"),
    "^There %r %n problem value%s[.]"
  )
  expect_message(
    msg("there %r %n problem value%s", n = 2),
    "^There are 2 problem values[.]"
  )
  expect_message(
    msg("there %r %n problem value%s", n = 0, tidy = FALSE),
    "there are 0 problem values"
  )
})

test_that("msg .subclass deprecated", {
  lifecycle::expect_deprecated(expect_s3_class(expect_message(msg(.subclass = "xx"), "^[.]$"), "xx"))
  expect_s3_class(expect_message(msg(class = "xx"), "^[.]$"), "xx")
})

test_that("wrn", {
  expect_warning(wrn(), "^[.]$")
  expect_warning(wrn("this", "is"), "^Thisis[.]$")
  expect_warning(
    wrn("there %r %n problem value%s"),
    "^There %r %n problem value%s."
  )
  expect_warning(
    wrn("there %r %n problem value%s", n = 2),
    "^There are 2 problem values[.]$"
  )
  expect_warning(
    wrn("there %r %n problem value%s", n = 0, tidy = FALSE),
    "there are 0 problem values$"
  )
})

test_that("wrn .subclass deprecated", {
  lifecycle::expect_deprecated(expect_s3_class(expect_warning(wrn(.subclass = "xx"), "^[.]$"), "xx"))
  expect_s3_class(expect_warning(wrn(class = "xx"), "^[.]$"), "xx")
})

#' Check Key
#'
#' @description
#' Checks if columns have unique rows.
#'
#' @inheritParams params
#' @param key A character vector of the columns that represent a unique key.
#' @param na_distinct A flag specifying whether missing values should be considered distinct.
#' @return An informative error if the test fails or an invisible copy of x.
#'
#' @family check
#'
#' @examples
#' x <- data.frame(x = c(1, 2), y = c(1, 1))
#' check_key(x)
#' try(check_key(x, "y"))
#' @export
check_key <- function(x, key = character(0), na_distinct = FALSE, x_name = NULL) {
  chk_data(x)
  chk_s3_class(key, "character")
  if (!length(key)) {
    return(invisible(x))
  }

  chk_unique(key)
  chk_not_any_na(key)
  chk_flag(na_distinct)

  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  chk_string(x_name)

  check_names(x, key, x_name = x_name)

  # to ensure not tripped up by sf objects.
  x2 <- as.data.frame(x)

  incomparables <- if (na_distinct) NA else FALSE
  if (any(new_duplicated(x2[key], incomparables = incomparables))) {
    abort_chk("Column%s ", cc(key, conj = " and "), " in ", x_name, " must be a unique key.", n = length(key))
  }
  invisible(x)
}

test_that("check_key works", {
  expect_identical(check_key(data.frame()), data.frame())
  expect_invisible(check_key(data.frame()))
  expect_invisible(check_key(data.frame(x = 1)))
  expect_invisible(check_key(data.frame(x = 1), "x"))
  expect_invisible(check_key(data.frame(x = c(1, 1), y = 1:2), c("y", "x")))
  expect_invisible(check_key(data.frame(x = c(1, 1), y = c(NA, 1)), c("y", "x")))
  expect_invisible(check_key(data.frame(x = c(1, 1), y = NA), c("y", "x"), na_distinct = TRUE))
})

test_that("check_key fails", {
  x <- data.frame(x = c(1, 2), y = c(1, 1), z = NA)
  expect_chk_error(
    check_key(x, c("y", "y")),
    "`key` must be unique[.]$"
  )
  expect_chk_error(
    check_key(x, "y"),
    "^Column 'y' in `x` must be a unique key[.]$"
  )
  expect_chk_error(
    check_key(x, c("y", "z")),
    "^Columns 'y' and 'z' in `x` must be a unique key[.]$"
  )
})

#' Check Environment
#'
#' @description
#' Checks if environment using
#'
#' `is.environment(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family data_type_checkers
#'
#' @seealso [is.environment()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_environment
#' chk_environment(.GlobalEnv)
#' try(chk_environment(1))
#' @export
chk_environment <- function(x, x_name = NULL) {
  if (vld_environment(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be an environment", x = x)
}

#' @describeIn chk_environment Validate Environment
#'
#' @examples
#' # vld_environment
#' vld_environment(1)
#' vld_environment(list(1))
#' vld_environment(.GlobalEnv)
#' vld_environment(environment())
#' @export
vld_environment <- function(x) is.environment(x)

test_that("vld_environment", {
  expect_false(vld_environment(1))
  expect_false(vld_environment(list(1)))
  expect_true(vld_environment(.GlobalEnv))
  expect_true(vld_environment(environment()))
})

test_that("chk_environment", {
  expect_identical(chk_environment(.GlobalEnv), .GlobalEnv)
  expect_invisible(chk_environment(.GlobalEnv))
  expect_chk_error(chk_environment(1), "^`1` must be an environment[.]$")
  expect_chk_error(chk_environment(list(1), x_name = 1), "^1 must be an environment[.]$")
})

#' Check Not Any Missing Values
#'
#' @description
#' Checks if not any missing values using
#'
#' `!anyNA(x)`
#'
#' **Pass**: `1`, `1:2`, `"1"`, `logical(0)`.
#'
#' **Fail**: `NA`, `c(1, NA)`.
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family misc_checkers
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_not_any_na
#' chk_not_any_na(1)
#' try(chk_not_any_na(NA))
#' @export
chk_not_any_na <- function(x, x_name = NULL) {
  if (vld_not_any_na(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must not have any missing values", x = x)
}

#' @describeIn chk_not_any_na Validate Not Any Missing Values
#'
#' @examples
#' # vld_not_any_na
#' vld_not_any_na(1)
#' vld_not_any_na(1:2)
#' vld_not_any_na(NA_real_)
#' vld_not_any_na(integer(0))
#' vld_not_any_na(c(NA, 1))
#' vld_not_any_na(TRUE)
#' @export
vld_not_any_na <- function(x) !anyNA(x)

test_that("vld_not_any_na", {
  expect_true(vld_not_any_na(1))
  expect_true(vld_not_any_na(matrix(1:3)))
  expect_true(vld_not_any_na(character(0)))
  expect_true(vld_not_any_na(NULL))
  expect_false(vld_not_any_na(NA))
  expect_false(vld_not_any_na(c(1, NA)))
})

test_that("chk_not_any_na", {
  expect_identical(chk_not_any_na(1), 1)
  expect_invisible(chk_not_any_na(1))
  expect_chk_error(chk_not_any_na(NA), "^`NA` must not have any missing values[.]$")
  expect_chk_error(chk_not_any_na(NA, x_name = 1), "^1 must not have any missing values[.]$")
})

#' Check Superset
#'
#' @description
#' Checks if includes all values using
#'
#' `all(values %in% x)`
#'
#' Pay attention to the order of the arguments `value` and `x`
#' in this function compared to [chk_subset()]
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family set_checkers
#'
#' @seealso [all()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_superset
#' chk_superset(1:3, 1)
#' try(chk_superset(1:3, 4))
#' @export
chk_superset <- function(x, values, x_name = NULL) {
  if (vld_superset(x, values)) {
    return(invisible(x))
  }
  values <- sort(unique(values), na.last = TRUE)
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  values <- values[!values %in% x]
  abort_chk(x_name, " must include ", cc(values, " and "), x = x, values = values)
}

#' @describeIn chk_superset Validates Superset
#'
#' @examples
#' # vld_superset
#' vld_superset(1:3, 1)
#' vld_superset(1:3, 4)
#' vld_superset(integer(0), integer(0))
#' @export
vld_superset <- function(x, values) all(values %in% x)

test_that("vld_superset", {
  expect_true(vld_superset(1, 1))
  expect_true(vld_superset(1:2, 1))
  expect_true(vld_superset(1:3, 1:2))
  expect_true(vld_superset(c(1:3, NA), 1:2))
  expect_false(vld_superset(c(1:3), c(1:2, NA)))
  expect_true(vld_superset(c(1:3, NA), c(1:2, NA)))
  expect_false(vld_superset(1, 1:2))
})

test_that("chk_superset", {
  expect_identical(chk_superset(1, 1), 1, 1)
  expect_invisible(chk_superset(1, 1))
  expect_chk_error(chk_superset(1, 1:2), "^`1` must include 2[.]$")
  expect_chk_error(chk_superset(1, 1:3), "^`1` must include 2 and 3[.]$")
  expect_chk_error(chk_superset(2, 1), "^`2` must include 1[.]$")
})

#' Check File Exists
#'
#' @description
#' Checks if file exists using
#'
#' `vld_string(x) && file.exists(x) && !dir.exists(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family file_checkers
#'
#' @seealso [vld_string()]
#' @seealso [file.exists()]
#' @seealso [dir.exists()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_file
#' try(chk_file(tempfile()))
#' @export
chk_file <- function(x, x_name = NULL) {
  if (vld_file(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  chk_string(x, x_name = x_name)
  if (dir.exists(x)) {
    abort_chk(x_name, " must specify a file ('", x, "' is a directory)", x = x)
  }
  abort_chk(x_name, " must specify an existing file ('", x, "' can't be found)", x = x)
}

#' @describeIn chk_file Validate File Exists
#'
#' @examples
#' # vld_file
#' vld_file(tempfile())
#' @export
vld_file <- function(x) vld_string(x) && file.exists(x) && !dir.exists(x)

test_that("vld_file", {
  expect_false(vld_file(character(0)))
  expect_false(vld_file(tempdir()))
  file <- withr::local_tempfile()
  expect_false(vld_file(file))

  write.csv(data.frame(x = 1), file)
  expect_true(vld_file(file))
  expect_false(vld_file(dirname(file)))
})

test_that("chk_file", {
  expect_chk_error(
    chk_file(character(0)),
    "^`character[(]0[)]` must be a string [(]non-missing character scalar[)][.]$"
  )

  expect_chk_error(chk_file(tempdir()), "`tempdir[(][)]` must specify a file [(]'.*' is a directory[)][.]$")

  file1 <- withr::local_tempfile(fileext = ".csv")
  expect_chk_error(
    chk_file(file1),
    "^`file1` must specify an existing file [(]'.*[.]csv' can't be found[)][.]$"
  )
  write.csv(data.frame(x = 1), file1)
  expect_identical(chk_file(file1), file1)
  expect_invisible(chk_file(file1))

  expect_identical(chk_all(character(0), chk_file), character(0), chk_file)
  expect_invisible(chk_all(character(0), chk_file))
  expect_identical(chk_all(file1, chk_file), file1, chk_file)
  expect_invisible(chk_all(file1, chk_file))
  expect_identical(chk_all(c(file1, file1), chk_file), c(file1, file1), chk_file)
  expect_invisible(chk_all(c(file1, file1), chk_file))
  expect_chk_error(
    chk_all(c(file1, p0(file1, "b")), chk_file, x_name = "`vec`"),
    "^All elements of `vec` must specify an existing file [(]'.*[.]csvb' can't be found[)][.]$"
  )
})

test_that("vld_dir", {
  expect_false(vld_dir(character(0)))
  expect_true(vld_dir(tempdir()))
  expect_false(vld_dir(tempfile()))
})

test_that("chk_dir", {
  expect_identical(chk_dir(tempdir()), tempdir())
  expect_invisible(chk_dir(tempdir()))
  expect_chk_error(
    chk_dir(tempfile()),
    "^`tempfile[(][)]` must specify an existing directory [(]'.*' can't be found[)][.]$"
  )

  file1 <- withr::local_tempfile(fileext = ".csv")
  write.csv(data.frame(x = 1), file1)
  expect_chk_error(
    chk_dir(file1),
    "^`file1` must specify a directory [(]'.*[.]csv' is a file[)][.]$"
  )

  path <- withr::local_tempdir()
  unlink(path, recursive = TRUE)
  expect_chk_error(chk_dir(path), "^`path` must specify an existing directory [(]'.*' can't be found[)][.]$")
  expect_chk_error(chk_dir(1), "^`1` must be a string [(]non-missing character scalar[)][.]$")
  expect_invisible(chk_all(c(tempdir(), tempdir()), chk_dir))
  expect_chk_error(
    chk_all(c(tempdir(), p0(tempdir(), "b")), chk_dir, x_name = "`vec`"),
    "^All elements of `vec` must specify an existing directory [(]'.*b' can't be found[)][.]$"
  )
})

test_that("vld_ext", {
  expect_false(vld_ext(character(0)))
  expect_false(vld_ext("file.pdf", "PDF"))
  expect_false(vld_ext(c("file.pdf", "file.pdf"), "pdf"))
  expect_true(vld_ext("file.pdf", "pdf"))
  expect_true(vld_ext("file.png", c("pdf", "png")))
})

test_that("chk_ext", {
  expect_chk_error(
    chk_ext(character(0)),
    "^`character[(]0[)]` must be a string [(]non-missing character scalar[)][.]$"
  )
  expect_chk_error(
    chk_ext("file.pdf", "png"),
    "^`\"file.pdf\"` must have extension 'png' [(]not 'pdf'[)][.]$"
  )
  expect_chk_error(
    chk_ext("file.pdf", c("png", "PDF")),
    "^`\"file.pdf\"` must have extension 'png' or 'PDF' [(]not 'pdf'[)][.]$"
  )
  expect_identical(chk_ext("file.pdf", "pdf"), "file.pdf", "pdf")
  expect_invisible(chk_ext("file.pdf", "pdf"))
})

#' Check Complex
#'
#' @description
#' Checks if complex using
#'
#' `is.complex(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family data_type_checkers
#'
#' @seealso [is.complex()]
#' @seealso For more details about the use of this function,
#' please read the article
#' \href{https://poissonconsulting.github.io/chk/articles/chk-families.html}{chk families}.
#'
#' @examples
#' # chk_complex
#' chk_complex(1i)
#' try(chk_complex(1))
#' @export
chk_complex <- function(x, x_name = NULL) {
  if (vld_complex(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be complex", x = x)
}

#' @describeIn chk_complex Validate Complex
#'
#' @examples
#' # vld_complex
#' vld_complex(1i)
#' vld_complex(complex())
#' vld_complex(NA_complex_)
#' vld_complex(1)
#' vld_complex(TRUE)
#' @export
vld_complex <- function(x) is.complex(x)

test_that("vld_complex", {
  expect_true(vld_complex(as.complex(1)))
  expect_true(vld_complex(as.complex(1:2)))
  expect_true(vld_complex(complex(0)))
  expect_true(vld_complex(NA_complex_))
  expect_false(vld_complex(1))
  expect_false(vld_complex(TRUE))
})

test_that("chk_complex", {
  expect_identical(chk_complex(as.complex(1)), as.complex(1))
  expect_invisible(chk_complex(as.complex(1)))
  expect_chk_error(
    chk_complex(TRUE),
    "^`TRUE` must be complex[.]$"
  )
})

#' Check Time Zone
#'
#' @description
#' Checks if non-missing valid scalar timezone using
#'
#' `is.character(x) && length(x) == 1L && !anyNA(x) && x %in% OlsonNames()`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family tz_checkers
#' @family date_checkers
#' @family scalar_checkers
#'
#' @seealso [length()]
#' @seealso [OlsonNames()]
#' @seealso [is.character()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' chk_tz("UTC")
#' try(chk_tz("TCU"))
#' @export
chk_tz <- function(x, x_name = NULL) {
  if (vld_tz(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be a tz (non-missing scalar character of a recognized timezone)", x = x)
}

#' @describeIn chk_tz Validate Time Zone
#'
#' @examples
#' vld_tz("UTC")
#' vld_tz("TCU")
#' @export
vld_tz <- function(x) {
  is.character(x) && length(x) == 1L && !anyNA(x) &&
    x %in% OlsonNames()
}

test_that("chk_tz", {
  expect_identical(chk_tz("UTC"), "UTC")
  expect_invisible(chk_tz("UTC"))
  expect_chk_error(
    chk_tz("TCU"),
    "^`\"TCU\"` must be a tz [(]non-missing scalar character of a recognized timezone[)][.]$"
  )
})

test_that("vld_tz", {
  expect_true(vld_tz("UTC"))
  expect_false(vld_tz("TCU"))
  expect_false(vld_tz(1))
  expect_false(vld_tz(character(0)))
  expect_false(vld_tz(c("UTC", "UTC")))
  expect_false(vld_tz(NA_character_))
})

#' Check Inherits from S4 Class
#'
#' @description
#' Checks inherits from S4 class using
#'
#' `isS4(x) && methods::is(x, class)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family id_checkers
#'
#' @seealso [methods::is()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_s4_class
#' try(chk_s4_class(1, "numeric"))
#' chk_s4_class(getClass("MethodDefinition"), "classRepresentation")
#' @export
chk_s4_class <- function(x, class, x_name = NULL) {
  if (vld_s4_class(x, class)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  .class <- class
  abort_chk(x_name, " must inherit from S4 class '", .class, "'", x = x, .class = .class)
}

#' @describeIn chk_s4_class Validate Inherits from S4 Class
#'
#' @examples
#' # vld_s4_class
#' vld_s4_class(numeric(0), "numeric")
#' vld_s4_class(getClass("MethodDefinition"), "classRepresentation")
#' @export
vld_s4_class <- function(x, class) isS4(x) && methods::is(x, class)

#' Check Named
#'
#' @description
#' Checks if is named using
#'
#' `!is.null(names(x))`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family name_checkers
#'
#' @seealso [names()]
#' @seealso [is.null()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_named
#' chk_named(c(x = 1))
#' try(chk_named(list(1)))
#' @export
chk_named <- function(x, x_name = NULL) {
  if (vld_named(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be named", x = x)
}

#' @describeIn chk_named Validate Named
#'
#' @examples
#' # vld_named
#' vld_named(c(x = 1))
#' vld_named(list(x = 1))
#' vld_named(c(x = 1)[-1])
#' vld_named(list(x = 1)[-1])
#' vld_named(1)
#' vld_named(list(1))
#' @export
vld_named <- function(x) !is.null(names(x))

test_that("vld_unique", {
  expect_true(vld_unique(integer(0)))
  expect_true(vld_unique(NA))
  expect_true(vld_unique(1))
  expect_true(vld_unique(c(1, 2)))
  expect_true(vld_unique(c(1, NA)))
  expect_false(vld_unique(c(NA, NA)))
  expect_true(vld_unique(c(NA, NA), incomparables = NA))
  expect_true(vld_unique(c(NA, NA, 1), incomparables = NA))
  expect_false(vld_unique(c(1, 1)))
  expect_false(vld_unique(c(1, 2, 1)))
})

test_that("chk_unique", {
  expect_identical(chk_unique(1), 1)
  expect_invisible(chk_unique(1))
  expect_chk_error(chk_unique(c(1, 1)), "^`c[(]1, 1[)]` must be unique[.]$")
  expect_chk_error(chk_unique(c(1, 1), x_name = "unicorn"), "^Unicorn must be unique[.]$")
})

test_that("vld_unique data frame", {
  data <- data.frame(x = 1:2)
  expect_true(vld_unique(data))
  expect_true(vld_unique(data, incomparables = NA))
  data <- data.frame(x = c(1, 1))
  expect_false(vld_unique(data))
  expect_false(vld_unique(data, incomparables = NA))
  data <- data.frame(y = c(NA, NA))
  expect_true(vld_unique(data, incomparables = NA))
  data <- data.frame(x = 1:2, y = c(NA, NA))
  expect_true(vld_unique(data))
  expect_true(vld_unique(data, incomparables = NA))
  data <- data.frame(x = c(1, 1), y = c(2, 2))
  expect_true(vld_unique(data, incomparables = 2))
  data <- data.frame(x = c(1, 1), y = c(NA, NA))
  expect_true(vld_unique(data, incomparables = NA))
})

test_that("chk_unique data frame", {
  data <- data.frame(x = 1:2)
  expect_identical(chk_unique(data), data)
  expect_invisible(chk_unique(data))
  data <- data.frame(y = c(NA, NA))
  expect_chk_error(chk_unique(data), "^`data` must be unique[.]$")
  data <- data.frame(x = 1:2, y = c(NA, NA))
  data <- data.frame(x = c(1, 1), y = c(2, 2))
  expect_chk_error(chk_unique(data), "^`data` must be unique[.]$")
  data <- data.frame(x = c(1, 1), y = c(NA, NA))
  expect_chk_error(chk_unique(data), "^`data` must be unique[.]$")
})

test_that("vld_named", {
  expect_false(vld_named(list()))
  expect_false(vld_named(list(1)))
  expect_true(vld_named(list(x = 1)))
  expect_true(vld_named(list(x = 1)[-1]))
  expect_true(vld_named(c(x = 1)))
  expect_true(vld_named(c(x = 1)[-1]))
})

test_that("chk_named", {
  expect_identical(chk_named(c(x = 1)), c(x = 1))
  expect_invisible(chk_named(c(x = 1)))
  expect_chk_error(chk_named(1), "^`1` must be named[.]$")
  expect_chk_error(chk_named(1, x_name = "new born"), "^New born must be named[.]$")
})

#' Check Date Time
#'
#' @description
#' Checks if non-missing POSIXct scalar using
#'
#' `inherits(x, "POSIXct") && length(x) == 1L && !anyNA(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family scalar_checkers
#' @family datetime_checkers
#'
#' @seealso [inherits()], [length()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_date_time
#' chk_date_time(as.POSIXct("2001-01-02"))
#' try(chk_date_time(1))
#' @export
chk_date_time <- function(x, x_name = NULL) {
  if (vld_date_time(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be a date time (non-missing POSIXct scalar)", x = x)
}

#' @describeIn chk_date_time Check Date Time (Deprecated)
#'
#' `r lifecycle::badge("deprecated")`
#'
#' @export
chk_datetime <- function(x, x_name = NULL) {
  deprecate_warn("0.4.1",
    what = "chk::chk_datetime()",
    with = "chk::chk_date_time()",
    id = "chk_datetime"
  )
  if (vld_date_time(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be a date time (non-missing POSIXct scalar)", x = x)
}

#' @describeIn chk_date_time Validate Date Time
#'
#' @examples
#' # vld_date_time
#' vld_date_time(as.POSIXct("2001-01-02"))
#' vld_date_time(Sys.time())
#' vld_date_time(1)
#' vld_date_time("2001-01-02")
#' vld_date_time(c(Sys.time(), Sys.time()))
#' @export
vld_date_time <- function(x) {
  inherits(x, "POSIXct") && length(x) == 1L && !anyNA(x)
}

#' @describeIn chk_date_time Validate Date Time (Deprecated)
#'
#' `r lifecycle::badge("deprecated")`
#'
#' @export
vld_datetime <- function(x) {
  deprecate_warn("0.4.1",
    what = "chk::vld_datetime()",
    with = "chk::vld_date_time()",
    id = "chk_datetime"
  )
  vld_date_time(x)
}

test_that("vld_date_time", {
  expect_false(vld_date_time(NULL))
  expect_false(vld_date_time(Sys.time()[-1]))
  expect_false(vld_date_time(Sys.time()[c(1, 1)]))
  expect_true(vld_date_time(Sys.time()))
})

test_that("chk_date_time", {
  time <- Sys.time()
  expect_identical(chk_date_time(time), time)
  expect_invisible(chk_date_time(Sys.time()))
  expect_chk_error(chk_date_time(1), "^`1` must be a date time [(]non-missing POSIXct scalar[)][.]$")
  expect_chk_error(chk_date(1, x_name = 1), "^1 must be a date [(]non-missing Date scalar[)][.]$")
})

test_that("vld_datetime", {
  rlang::local_options(lifecycle_verbosity = "quiet")
  expect_false(vld_datetime(NULL))
  expect_false(vld_datetime(Sys.time()[-1]))
  expect_false(vld_datetime(Sys.time()[c(1, 1)]))
  expect_true(vld_datetime(Sys.time()))
})

test_that("chk_datetime", {
  rlang::local_options(lifecycle_verbosity = "quiet")
  time <- Sys.time()
  expect_identical(chk_date_time(time), time)
  expect_invisible(chk_datetime(Sys.time()))
  expect_chk_error(chk_datetime(1), "^`1` must be a date time [(]non-missing POSIXct scalar[)][.]$")
  expect_chk_error(chk_date(1, x_name = 1), "^1 must be a date [(]non-missing Date scalar[)][.]$")
})

#' Abort Check
#'
#' A wrapper on [err()] that sets the subclass to be `'chk_error'`.
#'
#' It is exported to allow users to easily construct their own `chk_` functions.
#'
#' @inheritParams message_chk
#' @inheritParams rlang::abort
#' @return Throws an error of class `'chk_error'`.
#' @seealso [err()]
#' @export
#'
#' @examples
#' try(abort_chk("x must be NULL"))
#' try(abort_chk("`x` must be NULL"))
#' try(abort_chk("there %r %n problem value%s", n = 1))
#' try(abort_chk("there %r %n problem value%s", n = 1.5))
abort_chk <- function(..., n = NULL, tidy = TRUE, call = rlang::caller_call(2)) {
  err(..., n = n, tidy = tidy, class = "chk_error", call = call)
}

#' Deparse Backtick
#'
#' `deparse_backtick_chk` is a wrapper on [deparse()]
#' and `backtick_chk`.
#'
#' It is exported to allow users to easily construct their own `chk_` functions.
#'
#' @param x A substituted object to deparse.
#'
#' @return A string of the backticked substituted object.
#' @seealso [deparse()]
#' @export
#'
#' @examples
#'
#' # deparse_backtick_chk
#' deparse_backtick_chk(2)
#' deparse_backtick_chk(2^2)
deparse_backtick_chk <- function(x) backtick_chk(deparse(x))

#' @describeIn deparse_backtick_chk Backtick
#' @export
backtick_chk <- function(x) p0("`", unbacktick_chk(x), "`")

#' @describeIn deparse_backtick_chk Unbacktick
#' @export
unbacktick_chk <- function(x) gsub("`", "", x)

test_that("deparse_backtick_chk", {
  expect_identical(deparse_backtick_chk(1), "`1`")
  expect_identical(deparse_backtick_chk(NA), "`NA`")
  expect_identical(deparse_backtick_chk("1"), "`\"1\"`")
  expect_identical(deparse_backtick_chk(substitute(1)), "`1`")
  expect_identical(deparse_backtick_chk(substitute(NA)), "`NA`")
  expect_identical(deparse_backtick_chk(substitute("1")), "`\"1\"`")
})

test_that("backtick_chk", {
  expect_identical(backtick_chk(1), "`1`")
  expect_identical(backtick_chk("`1`"), "`1`")
})

test_that("unbacktick_chk", {
  expect_identical(unbacktick_chk(1), "1")
  expect_identical(unbacktick_chk("`1`"), "1")
  expect_identical(unbacktick_chk("``1``"), "1")
})

#' Check Values and Class
#'
#' @description
#' Checks values and S3 class of an atomic object.
#'
#' @details
#' To check the class simply pass a vector of the desired class.
#'
#' To check that x does not include missing values pass
#' a single non-missing value (of the correct class).
#'
#' To allow it to include missing values include a missing value.
#'
#' To check that it only includes missing values only pass
#' a missing value (of the correct class).
#'
#' To check the range of the values in x pass two non-missing values
#' (as well as the missing value if required).
#'
#' To check that x only includes specific values
#' pass three or more non-missing values.
#'
#' In the case of a factor ensure values has two levels to
#' check that the levels of x are an ordered superset of the levels of value
#' and three or more levels to check that they are identical.
#'
#' @inheritParams params
#' @param values An atomic vector specifying the S3 class and possible values.
#' @return An informative error if the test fails or an invisible copy of x.
#'
#' @family check
#'
#' @examples
#' check_values(1, numeric(0))
#' check_values(1, 2)
#' try(check_values(1, 1L))
#' try(check_values(NA_real_, 1))
#' @export
check_values <- function(x, values, x_name = NULL) {
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  chk_string(x_name)
  chk_atomic(x, x_name = x_name)
  chk_vector(values)
  chk_atomic(values)

  class <- class(values)[1]
  chk_s3_class(x, class, x_name = x_name)
  if (is.factor(values) && nlevels(values) > 1) {
    x_name_levels <- backtick_chk(p0("levels(", unbacktick_chk(x_name), ")"))
    if (nlevels(values) > 2) {
      chk_identical(levels(x), levels(values), x_name = x_name_levels)
    } else {
      chk_superset(levels(x), levels(values), x_name = x_name_levels)
      chk_orderset(levels(x), levels(values), x_name = x_name_levels)
    }
  }
  if (!length(x) || !length(values)) {
    return(invisible(x))
  }
  if (vld_not_any_na(values)) chk_not_any_na(x, x_name = x_name)
  x <- x[!is.na(x)]
  values <- values[!is.na(values)]
  if (!length(x) || length(values) < 2) {
    return(invisible(x))
  }
  if (identical(length(values), 2L)) {
    values <- sort(unique(values))
    if (identical(length(values), 1L)) {
      return(chk_all(x, chk_equal, values, x_name = x_name))
    }
    if (is.factor(values) && (nlevels(values) < 3 || !is.ordered(values))) {
      return(invisible())
    }
    return(chk_range(x, values, x_name = x_name))
  }
  chk_subset(x, sort(unique(values)), x_name = x_name)
}

test_that("check_values pass", {
  expect_identical(check_values(1, 1), check_values(1, 1))
  expect_invisible(check_values(1, 1))
  expect_identical(check_values(1, numeric(0)), check_values(1, numeric(0)))
  expect_identical(check_values(integer(0), 1:3), check_values(integer(0), 1:3))
  expect_identical(check_values(1L, integer(0)), check_values(1L, integer(0)))
  expect_identical(check_values(NA_real_, numeric(0)), check_values(NA_real_, numeric(0)))
  expect_identical(check_values(1, -1), check_values(1, -1))
  expect_identical(check_values(1, NA_real_), check_values(1, NA_real_))
  expect_identical(check_values(1, c(2, NA_real_)), check_values(1, c(2, NA_real_)))
  expect_identical(check_values(1, c(1, 2, NA_real_)), check_values(1, c(1, 2, NA_real_)))
  expect_identical(check_values(1, c(1, 1)), check_values(1, c(1, 1)))
  expect_identical(check_values(1, c(0.5, 2, NA_real_)), check_values(1, c(0.5, 2, NA_real_)))
  expect_identical(
    check_values(c(1, NA_real_), c(0.5, 2, NA_real_)),
    check_values(c(1, NA_real_), c(0.5, 2, NA_real_))
  )
  expect_identical(check_values(factor(1), factor(2)), check_values(factor(1), factor(2)))
  expect_identical(
    check_values(factor(c(1, NA)), factor(c(2, NA))),
    check_values(factor(c(1, NA)), factor(c(2, NA)))
  )
  expect_identical(check_values(factor(1:3), factor(1:2)), check_values(factor(1:3), factor(1:2)))
  expect_identical(
    check_values(factor(1:3), factor(1:2, levels = 1:2)),
    check_values(factor(1:3), factor(1:2, levels = 1:2))
  )
  expect_identical(
    check_values(ordered(1:2), factor(c(1:2, NA))),
    check_values(ordered(1:2), factor(c(1:2, NA)))
  )
  expect_identical(
    check_values(ordered(1:3), factor(c(1:2))),
    check_values(ordered(1:3), factor(c(1:2)))
  )
  expect_identical(
    check_values(as.character(1:2), as.character(2:1)),
    check_values(as.character(1:2), as.character(2:1))
  )
  expect_identical(check_values(factor(1:3), factor(1:3)), check_values(factor(1:3), factor(1:3)))
})

test_that("check_values fail", {
  expect_chk_error(
    check_values(1.1, c(1, 1)),
    "^All elements of `1.1` must be equal to 1[.]"
  )
  expect_chk_error(
    check_values(1L, 1),
    "^`1L` must inherit from S3 class 'numeric'[.]$"
  )
  expect_chk_error(check_values(1, 1L), "`1` must inherit from S3 class 'integer'[.]")
  expect_chk_error(check_values(1L, numeric(0)), "`1L` must inherit from S3 class 'numeric'[.]")
  expect_chk_error(
    check_values(NA_real_, 1),
    "`NA_real_` must not have any missing values[.]"
  )
  expect_chk_error(
    check_values(1L, 2:3),
    "`1L` must be between 2 and 3, not 1[.]"
  )
  expect_chk_error(
    check_values(1:10, 21:22),
    "`1:10` must have values between 21 and 22[.]"
  )
  expect_chk_error(
    check_values(1:10, 22:21),
    "`1:10` must have values between 21 and 22[.]"
  )
  expect_chk_error(
    check_values(1:10, 21:23),
    "`1:10` must have values matching 21, 22 or 23[.]"
  )
  expect_chk_error(
    check_values(factor(1:2), as.character(1:3)),
    "^`factor[(]1:2[)]` must inherit from S3 class 'character'[.]$"
  )

  expect_chk_error(
    check_values(factor(1:2), ordered(1:3)),
    "^`factor[(]1:2[)]` must inherit from S3 class 'ordered'[.]$"
  )

  expect_chk_error(
    check_values(factor(1:2), factor(2:3)),
    "^`levels[(]factor[(]1:2[)][)]` must include '3'[.]$"
  )

  expect_chk_error(
    check_values(factor(1:2), factor(1:3)),
    "^`levels[(]factor[(]1:2[)][)]` must be identical to the y object of class <chr>[.]"
  )

  expect_chk_error(
    check_values(ordered(c(1, 3), levels = 1:3), ordered(c(1, 2), levels = 1:3)),
    "^`ordered[(]c[(]1, 3[)], levels = 1:3[)]` must have values between '1' and '2'[.]$"
  )

  expect_chk_error(
    check_values(factor(1:2), factor(1:2, levels = 2:1)),
    "^`levels[(]factor[(]1:2[)][)]` must have [(]the first occurence of[)] each of the following elements in the following order: '2', '1'[.]$"
  )

  expect_chk_error(
    check_values(ordered(1:2), ordered(1:2, levels = 2:1)),
    "^`levels[(]ordered[(]1:2[)][)]` must have [(]the first occurence of[)] each of the following elements in the following order: '2', '1'[.]$"
  )
})

#' Check Flag
#'
#' @description
#' Checks if non-missing logical scalar using
#'
#' `is.logical(x) && length(x) == 1L && !anyNA(x)`
#'
#' **Pass**: `TRUE`, `FALSE`.
#'
#' **Fail**: `logical(0)`, `c(TRUE, TRUE)`, `"TRUE"`, `1`, `NA`.
#'
#' Do not confuse this function with [chk_lgl()],
#' which also checks for logical scalars of `length(x) == 1`
#' but can include `NA`s.
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family logical_checkers
#' @family scalar_checkers
#'
#' @seealso [is.logical()] [length()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_flag
#' chk_flag(TRUE)
#' try(vld_flag(1))
#' @export
chk_flag <- function(x, x_name = NULL) {
  if (vld_flag(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be a flag (TRUE or FALSE)", x = x)
}

#' @describeIn chk_flag Validate Flag
#'
#' @examples
#' # vld_flag
#' vld_flag(TRUE)
#' vld_flag(1)
#' @export
vld_flag <- function(x) is.logical(x) && length(x) == 1L && !anyNA(x)

test_that("vld_true", {
  expect_true(vld_true(TRUE))
  expect_true(vld_true(c(x = TRUE)))
  expect_true(vld_true(structure(TRUE, x = "y")))
  expect_false(vld_true(FALSE))
  expect_false(vld_true(NA))
  expect_false(vld_true(c(TRUE, TRUE)))
  expect_false(vld_true(1))
})

test_that("chk_true", {
  expect_identical(chk_true(TRUE), TRUE)
  expect_invisible(chk_true(TRUE))
  expect_identical(chk_true(c(x = TRUE)), c(x = TRUE))

  expect_chk_error(chk_true(FALSE), "^`FALSE` must be TRUE[.]$")
  expect_chk_error(chk_true(FALSE, x_name = TRUE), "^TRUE must be TRUE[.]$")
})

test_that("vld_false", {
  expect_true(vld_false(FALSE))
  expect_false(vld_false(TRUE))
  expect_true(vld_false(c(x = FALSE)))
  expect_true(vld_false(structure(FALSE, x = "y")))
  expect_false(vld_false(NA))
  expect_false(vld_false(c(FALSE, FALSE)))
  expect_false(vld_false(0))
})

test_that("chk_false", {
  expect_identical(chk_false(FALSE), FALSE)
  expect_invisible(chk_false(FALSE))
  expect_identical(chk_false(c(x = FALSE)), c(x = FALSE))

  expect_chk_error(chk_false(TRUE), "^`TRUE` must be FALSE[.]$")
  expect_chk_error(chk_false(TRUE, x_name = FALSE), "^FALSE must be FALSE[.]$")
})

test_that("vld_flag", {
  expect_true(vld_flag(FALSE))
  expect_true(vld_flag(TRUE))
  expect_false(vld_flag(logical(0)))
  expect_false(vld_flag(NA))
  expect_false(vld_flag(c(FALSE, TRUE)))
  expect_false(vld_flag(1))
})

test_that("chk_flag", {
  expect_identical(chk_flag(TRUE), TRUE)
  expect_invisible(chk_flag(TRUE))
  expect_chk_error(chk_flag(1), "^`1` must be a flag [(]TRUE or FALSE[)][.]$")
  expect_chk_error(chk_flag(1, x_name = "2"), "^2 must be a flag [(]TRUE or FALSE[)][.]$")
})

test_that("expect-chk-error", {
  expect_s3_class(expect_chk_error(chk_flag(1)), "chk_error")
  expect_error(expect_chk_error(chk_flag(TRUE)), "",
    class = "expectation_failure"
  )
  expect_error(expect_chk_error(chk_flag(TRUE), class = "chk_error"), "`class` must be NULL.")
})

test_that("chkor", {
  rlang::local_options(lifecycle_verbosity = "quiet")

  expect_null(chkor())
  expect_invisible(chkor())
  expect_null(chkor(chk_flag(TRUE)))
  expect_null(chkor(chk_flag(1), chk_flag(TRUE)))
  expect_chk_error(chkor(chk_flag(1)), "^`1` must be a flag [(]TRUE or FALSE[)][.]$")
  expect_chk_error(chkor(chk_flag(1), chk_flag(1)), "^`1` must be a flag [(]TRUE or FALSE[)][.]$")
  expect_chk_error(chkor(chk_flag(1), chk_flag(2)), "^At least one of the following conditions must be met:\n[*] `1` must be a flag [(]TRUE or FALSE[)][.]\n[*] `2` must be a flag [(]TRUE or FALSE[)][.]$")
  expect_chk_error(chkor(chk_flag(1), chk_flag(2), chk_flag(0)), "^At least one of the following conditions must be met:\n[*] `1` must be a flag [(]TRUE or FALSE[)].\n[*] `2` must be a flag [(]TRUE or FALSE[)].\n[*] `0` must be a flag [(]TRUE or FALSE[)].$")
})

test_that("chkor_vld", {
  expect_null(chkor_vld())
  expect_invisible(chkor_vld())
  expect_null(chkor_vld(vld_flag(TRUE)))
  expect_null(chkor_vld(vld_flag(TRUE), vld_number(1)))
  expect_null(chkor_vld(vld_flag(1), vld_flag(TRUE)))
  expect_chk_error(chkor_vld(vld_flag(1)), "^`1` must be a flag [(]TRUE or FALSE[)][.]$")
  expect_chk_error(chkor_vld(vld_flag(1), vld_flag(1)), "^`1` must be a flag [(]TRUE or FALSE[)][.]$")
  expect_chk_error(chkor_vld(vld_flag(1), vld_flag(2)), "^At least one of the following conditions must be met:\n[*] `1` must be a flag [(]TRUE or FALSE[)][.]\n[*] `2` must be a flag [(]TRUE or FALSE[)][.]$")
  expect_chk_error(chkor_vld(vld_flag(1), vld_flag(2), vld_flag(0)), "^At least one of the following conditions must be met:\n[*] `1` must be a flag [(]TRUE or FALSE[)].\n[*] `2` must be a flag [(]TRUE or FALSE[)].\n[*] `0` must be a flag [(]TRUE or FALSE[)].$")
})

#' Check Data
#'
#' @description
#' Checks column names, values, number of rows and key for a data.frame.
#'
#' @inheritParams params
#' @param values A uniquely named list of atomic vectors of the column values.
#' @return An informative error if the test fails or an invisible copy of x.
#'
#' @family check
#'
#' @export
#' @examples
#' check_data(data.frame())
#' check_data(data.frame(x = 2), list(x = 1))
#' try(check_data(data.frame(x = 2), list(y = 1L)))
#' try(check_data(data.frame(x = 2), list(y = 1)))
#' try(check_data(data.frame(x = 2), nrow = 2))
check_data <- function(x, values = NULL, exclusive = FALSE, order = FALSE, nrow = numeric(0), key = character(0), x_name = NULL) {
  chk_data(x, "data.frame")

  if (is.null(values)) values <- structure(list(), .Names = character(0))
  chk_list(values)
  chk_named(values)
  chk_unique(names(values))
  chk_all(values, chk_fun = chk_atomic)

  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  chk_string(x_name)

  check_dim(x,
    dim = base::nrow, values = nrow, x_name = x_name,
    dim_name = "nrow"
  )
  check_names(x,
    names = names(values), exclusive = exclusive,
    order = order, x_name = x_name
  )
  lapply(names(values), function(name) {
    check_values(x[[name]], values[[name]],
      x_name = backtick_chk(p0(unbacktick_chk(x_name), "$", name))
    )
  })
  check_key(x, key = key, x_name = x_name)
  invisible(x)
}

test_that("check_data works", {
  expect_identical(check_data(data.frame()), data.frame())
  expect_invisible(check_data(data.frame()))
  expect_identical(check_data(data.frame(x = 1)), data.frame(x = 1))
  expect_identical(
    check_data(data.frame(x = 1, values = list(x = 1))),
    check_data(data.frame(x = 1, values = list(x = 1)))
  )
  expect_identical(check_data(data.frame(x = 1), nrow = c(1, Inf)), data.frame(x = 1))
})

test_that("check_data fails", {
  expect_chk_error(
    check_data(data.frame(), nrow = TRUE),
    "`nrow[(]data.frame[(][)][)]` must be greater than 0, not 0."
  )
  expect_chk_error(
    check_data(data.frame(), values = list(x = 1)),
    "`names[(]data.frame[(][)][)]` must include 'x'."
  )

  expect_chk_error(check_data(data.frame(x = 1, y = 2), values = list(x = 1), exclusive = TRUE), "^`names[(]data.frame[(]x = 1, y = 2[)][)]` must not include 'y'[.]$")
  expect_chk_error(
    check_data(data.frame(x = 1), values = list(x = 1L)),
    "`data.frame[(]x = 1[)][$]x` must inherit from S3 class 'integer'."
  )
  expect_chk_error(
    check_data(data.frame(x = c(1, 1)), key = "x"),
    "^Column 'x' in `data.frame[(]x = c[(]1, 1[)][)]` must be a unique key[.]$"
  )
  expect_chk_error(
    check_data(data.frame(x = ordered(1:2)), values = list(x = ordered(1:2, levels = 2:1))),
    "^`levels[(]data.frame[(]x = ordered[(]1:2[)][)][$]x[)]` must have [(]the first occurence of[)] each of the following elements in the following order: '2', '1'[.]$"
  )
})

#' Check Identical
#'
#' @description
#' Checks if is identical to y using
#'
#' `identical(x, y)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family equal_checkers
#'
#' @seealso [identical()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_identical
#' chk_identical(1, 1)
#' try(chk_identical(1, 1L))
#' chk_identical(c(1, 1), c(1, 1))
#' try(chk_identical(1, c(1, 1)))
#' @export
chk_identical <- function(x, y, x_name = NULL) {
  if (identical(x, y)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  y_name <- as_label(y)
  obj_of <- if (grepl("^<", y_name)) "the y object of class " else NULL
  abort_chk(x_name, " must be identical to ",
    obj_of, y_name,
    x = x, y = y
  )
}

#' @describeIn chk_identical Validate Identical
#'
#' @examples
#' vld_identical(1, 1)
#' @export
vld_identical <- function(x, y) identical(x, y)

test_that("vld_identical", {
  expect_true(vld_identical(1, 1))
  expect_true(vld_identical(1L, 1L))
  expect_false(vld_identical(1, 1L))
  expect_false(vld_identical(1L, 1))

  expect_true(vld_identical(c(1L), 1L))
  expect_false(vld_identical(c(x = 1L), 1L))
  expect_false(vld_identical(1L, c(x = 1L)))
})

test_that("chk_identical", {
  expect_identical(chk_identical(1, 1), 1, 1)
  expect_invisible(chk_identical(1, 1))
  expect_chk_error(
    chk_identical(1L, 2),
    "^`1L` must be identical to 2[.]\n"
  )

  expect_chk_error(chk_identical(1, 1L), "^`1` must be identical to 1L[.]\n")
  expect_chk_error(
    chk_identical(1, 1:10),
    "^`1` must be identical to the y object of class [<]int[>][.]\n"
  )
  expect_chk_error(
    chk_identical(1, 1:100),
    "^`1` must be identical to the y object of class [<]int[>][.]\n"
  )
  expect_chk_error(
    chk_identical(1, c(1, 5, 1, 9)),
    "^`1` must be identical to the y object of class [<]dbl[>][.]\n"
  )
  expect_chk_error(
    chk_identical(1, 2, x_name = 2),
    "^2 must be identical to 2[.]\n"
  )
})


test_that("vld_equal", {
  expect_true(vld_equal(1, 1))
  expect_true(vld_equal(1L, 1L))
  expect_true(vld_equal(1, 1L))
  expect_true(vld_equal(1L, 1))

  expect_true(vld_equal(c(1L), 1L))
  expect_false(vld_equal(c(x = 1L), 1L))
  expect_false(vld_equal(1L, c(x = 1L)))

  expect_true(vld_equal(1, 1.00000001))
  expect_true(vld_equal(1, 1.001, 0.01))
  expect_false(vld_equal(1, 1.001, 0.0001))
})

test_that("chk_equal", {
  expect_identical(chk_equal(1, 1), 1, 1)
  expect_invisible(chk_equal(1, 1))
  expect_chk_error(
    chk_equal(c(x = 1L), 1L),
    "^`c[(]x = 1L[)]` must be equal to 1L[.]\n"
  )
  expect_chk_error(
    chk_equal(1L, 2),
    "^`1L` must be equal to 2[.]\n"
  )
  expect_chk_error(
    chk_equal(1, c(1, 5, 1, 9)),
    "^`1` must be equal to the y object of class [<]dbl[>][.]\n"
  )
  expect_chk_error(
    chk_equal(1, 2, x_name = 2),
    "^2 must be equal to 2[.]\n"
  )
})

test_that("vld_equivalent", {
  expect_true(vld_equivalent(1, 1))
  expect_true(vld_equivalent(1L, 1L))
  expect_true(vld_equivalent(1, 1L))
  expect_true(vld_equivalent(1L, 1))

  expect_true(vld_equivalent(c(1L), 1L))
  expect_true(vld_equivalent(c(x = 1L), 1L))
  expect_true(vld_equivalent(c(x = 1L), 1L))
  expect_true(vld_equivalent(1L, c(x = 1L)))
  expect_true(vld_equivalent(1L, c(x = 1L)))

  expect_true(vld_equivalent(1, 1.00000001))
  expect_true(vld_equivalent(1, 1.001, 0.01))

  expect_false(vld_equivalent(1, 1.001, 0.0001))
})

test_that("chk_equivalent", {
  expect_identical(chk_equivalent(1, 1), 1, 1)
  expect_invisible(chk_equivalent(1, 1))
  expect_chk_error(chk_equivalent(1, 1.001, 0.0001), c("^`1` must be equivalent to 1[.]001[.]\n"))
  expect_chk_error(
    chk_equivalent(1, c(1, 5, 1, 9)),
    "^`1` must be equivalent to the y object of class [<]dbl[>][.]\n"
  )
  expect_chk_error(
    chk_equivalent(1, 2, x_name = 2),
    "^2 must be equivalent to 2[.]\n"
  )
})

#' Check OR
#'
#' The `chkor()` function has been deprecated for the faster `chkor_vld()`.
#'
#' `r lifecycle::badge("deprecated")`
#'
#' @param ... Multiple `chk_` functions.
#'
#' @return An informative error if the test fails.
#'
#' @seealso [chk_null_or()]
#'
#' @export
#'
#' @examples
#' chkor()
#' chkor(chk_flag(TRUE))
#' try(chkor(chk_flag(1)))
#' try(chkor(chk_flag(1), chk_flag(2)))
#' chkor(chk_flag(1), chk_flag(TRUE))
chkor <- function(...) {
  lifecycle::deprecate_soft("0.6.1", "chkor()", "chkor_vld()")
  quos <- enquos(...)

  invisible(chkor_quos(quos))
}

#' Deprecated functions
#'
#'
#' Deprecated `chk_()` functions.
#'
#' @family deprecated
#'
#' @inheritParams chk_flag
#' @keywords internal
#' @name chk_deprecated
NULL

#' @describeIn chk_deprecated Check Directories Exist
#'
#' `r lifecycle::badge("deprecated")`
#'
#' Replace with `[chk_all](x, [chk_dir])`
#'
#' @export
chk_dirs <- function(x) {
  deprecate_warn("0.2.0",
    what = "chk::chk_dirs()",
    details = "Please use `chk_all(x, chk_dir)` instead"
  )
  chk_all(x, chk_dir)
}

#' @describeIn chk_deprecated Check Files Exist
#'
#' `r lifecycle::badge("deprecated")`
#'
#' Replace with `[chk_all](x, [chk_file])`
#'
#' @export
chk_files <- function(x) {
  deprecate_warn("0.2.0",
    what = "chk::chk_files()",
    details = "Please use `chk_all(x, chk_file)` instead"
  )
  chk_all(x, chk_file)
}

#' @describeIn chk_deprecated Check Has
#'
#' `r lifecycle::badge("deprecated")`
#'
#' Replace by [chk_superset()]
#'
#' @export
chk_has <- function(x, values, x_name = NULL) {
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  chk_superset(x, values, x_name = x_name)
  deprecate_warn("0.2.0",
    what = "chk::chk_has()",
    with = "chk::chk_superset()"
  )
  invisible()
}

#' @describeIn chk_deprecated Check In
#'
#' `r lifecycle::badge("deprecated")`
#'
#' Replace by [chk_subset()]
#'
#' @export
chk_in <- function(x, values, x_name = NULL) {
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  chk_subset(x, values, x_name = x_name)
  deprecate_warn("0.2.0",
    what = "chk::chk_in()",
    with = "chk::chk_subset()"
  )
  invisible()
}

#' @describeIn chk_deprecated Check No Missing Values
#'
#' `r lifecycle::badge("deprecated")`
#'
#' Replace with [chk_not_any_na()]
#'
#' @export
chk_no_missing <- function(x, x_name = NULL) {
  deprecate_warn("0.2.0",
    what = "chk::chk_no_missing()", with = "chk::chk_not_any_na()",
    id = "chk_no_missing"
  )
  if (vld_no_missing(x)) {
    return(invisible())
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must not have missing values", x = x)
}

#' @describeIn chk_deprecated Validate No Missing Values
#'
#' `r lifecycle::badge("deprecated")`
#'
#' Replace with [vld_not_any_na()]
#'
#' @export
vld_no_missing <- function(x) {
  deprecate_warn("0.2.0",
    what = "chk::vld_no_missing()", with = "chk::vld_not_any_na()",
    id = "chk_no_missing"
  )
  !anyNA(x)
}

#' @describeIn chk_deprecated Turns checking off
#'
#' `r lifecycle::badge("deprecated")`
#'
#' This approach is no longer recommended
#'
#' @export
chk_off <- function() {
  deprecate_stop("0.2.0",
    what = "chk::chk_off()",
    details = "This approach is no longer recommended"
  )
  options(chk.on = FALSE)
}

#' @describeIn chk_deprecated Turns checking on
#'
#' `r lifecycle::badge("deprecated")`
#'
#' This approach is no longer recommended
#'
#' @export
chk_on <- function() {
  deprecate_stop("0.2.0",
    what = "chk::chk_on()",
    details = "This approach is no longer recommended"
  )
  options(chk.on = TRUE)
}

#' @describeIn chk_deprecated Tests checking on
#'
#' `r lifecycle::badge("deprecated")`
#'
#' This approach is no longer recommended
#'
#' @export
is_chk_on <- function() {
  deprecate_stop("0.2.0",
    what = "chk::is_chk_on()",
    details = "This approach is no longer recommended"
  )
  getOption("chk.on", TRUE)
}

#' @describeIn chk_deprecated Check Proportion
#'
#' `r lifecycle::badge("deprecated")`
#'
#' Replace by `[chk_number](x); [chk_range](x)`
#'
#' @export
chk_proportion <- function(x, x_name = NULL) {
  deprecate_warn("0.2.0",
    what = "chk::chk_proportion()",
    details = "Please use `chk_number(x)` or `chk_range(x)` instead"
  )
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  chk_number(x)
  chk_range(x)
}

#' @describeIn chk_deprecated Deparse Backtick
#'
#' `r lifecycle::badge("deprecated")`
#'
#' Replace with [deparse_backtick_chk()]
#'
#' @export
deparse_backtick <- function(x) {
  deprecate_warn("0.2.0",
    what = "chk::deparse_backtick()",
    with = "chk::deparse_backtick_chk()"
  )
  deparse_backtick_chk(x)
}

test_that("vld_no_missing", {
  rlang::local_options(lifecycle_verbosity = "quiet")

  expect_true(vld_no_missing(1))
  expect_true(vld_no_missing(integer(0)))
  expect_false(vld_no_missing(NA))
  expect_false(vld_no_missing(c(NA, 1)))
})

test_that("chk_no_missing", {
  rlang::local_options(lifecycle_verbosity = "quiet")

  expect_null(chk_no_missing(1))
  expect_invisible(chk_no_missing(1))
  expect_chk_error(chk_no_missing(NA), "^`NA` must not have missing values[.]$")
  expect_chk_error(chk_no_missing(NA, x_name = "1"), "^1 must not have missing values[.]$")
})


test_that("warning messages are generated for dep functions", {
  rlang::local_options(lifecycle_verbosity = "warning")

  expect_warning(chk_dirs(tempdir()))
  expect_warning(chk_has(1:3, 1))
  expect_warning(chk_in(1, 1:3))
  expect_warning(expect_warning(chk_no_missing(1)))
  expect_warning(vld_no_missing(1))
  expect_warning(chk_proportion(1))
  expect_warning(deparse_backtick(2))

  file <- tempfile()
  writeLines(file, text = "some test data")
  expect_warning(chk_files(file))
})

#' Check ... Used
#'
#' @description
#' Checks if is ... used using
#'
#' `length(list(...)) != 0L`
#'
#' @inheritParams params
#' @inherit params return
#' @return The `chk_` function throws an informative error if the test fails.
#'
#' @family ellipsis_checkers
#'
#' @seealso [length()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_used
#' fun <- function(x, ...) {
#'   chk_used(...)
#'   x
#' }
#' try(fun(1))
#' fun(1, 2)
#' @export
chk_used <- function(...) {
  if (vld_used(...)) {
    return(invisible())
  }
  abort_chk("`...` must be used")
}

#' @describeIn chk_used Validate ... Used
#'
#' @examples
#' # vld_used
#' fun <- function(x, ...) {
#'   vld_used(...)
#' }
#' fun(1)
#' fun(1, 2)
#' @export
vld_used <- function(...) length(list(...)) != 0L

#' Check Not Missing Argument
#'
#' @description
#' Checks argument not missing using
#'
#' `!missing(x)`
#'
#' @details
#' Currently only checks if value is available
#' (as opposed to whether it was specified).
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family missing_checkers
#'
#' @seealso [missing()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_not_missing
#' fun <- function(x) {
#'   chk_not_missing(x)
#' }
#' fun(1)
#' try(fun())
#' @export
chk_not_missing <- function(x, x_name = "`x`") {
  if (vld_not_missing(x)) {
    return(invisible(x))
  }
  abort_chk(x_name, " must not be missing.", tidy = FALSE)
}

#' @describeIn chk_not_missing Validate Not Missing Argument
#'
#' @examples
#' # vld_not_missing
#' fun <- function(x) {
#'   vld_not_missing(x)
#' }
#' fun()
#' fun(1)
#' @export
vld_not_missing <- function(x) {
  !missing(x)
}

test_that("vld_not_missing", {
  expect_false(vld_not_missing())
  expect_true(vld_not_missing(1))
})

test_that("vld_not_missing in function", {
  fun <- function(x) vld_not_missing(x)
  expect_false(fun())
  expect_true(fun(1))
})

test_that("vld_not_missing in function default", {
  fun <- function(x = 1) vld_not_missing(x)
  # this might be expected to be false if testing whether specified
  expect_true(fun())
  expect_true(fun(1))
})

test_that("chk_not_missing", {
  expect_identical(chk_not_missing(1), 1)
  expect_invisible(chk_not_missing(1))
  expect_chk_error(chk_not_missing(), "^`x` must not be missing\\.$")
  expect_chk_error(chk_not_missing(x_name = "`y`"), "^`y` must not be missing\\.$")
})

test_that("chk_not_missing in function", {
  fun <- function(x) chk_not_missing(x)

  expect_chk_error(fun(), "^`x` must not be missing\\.$")
  expect_identical(fun(1), 1)
})

test_that("chk_not_missing in function default", {
  fun <- function(x = 1) chk_not_missing(x)

  expect_identical(fun(NULL), NULL)
  expect_identical(fun(), 1)
  expect_identical(fun(2), 2)
})

#' Check TRUE
#'
#' @description
#' Checks if TRUE using
#'
#' `is.logical(x) && length(x) == 1L && !anyNA(x) && x`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family logical_checkers
#' @family scalar_checkers
#'
#' @seealso [is.logical()]
#' @seealso [length()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_true
#' chk_true(TRUE)
#' try(chk_true(1))
#' @export
chk_true <- function(x, x_name = NULL) {
  if (vld_true(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be TRUE", x = x)
}

#' @describeIn chk_true Validate TRUE
#'
#' @examples
#' # vld_true
#' vld_true(TRUE)
#' vld_true(FALSE)
#' vld_true(NA)
#' vld_true(0)
#' vld_true(c(TRUE, TRUE))
#' @export
vld_true <- function(x) is.logical(x) && length(x) == 1L && !anyNA(x) && x

#' Expect Chk Error
#'
#' [expect_chk_error()] checks that code throws an error
#' of class `"chk_error"` with a message that matches regexp.
#' See below for more details.
#'
#' @inherit testthat::expect_error
#' @param class Must be NULL.
#' @export
#'
#' @examples
#' expect_chk_error(chk_true(FALSE))
#' try(expect_chk_error(chk_false(FALSE)))
expect_chk_error <- function(object, regexp = NULL, ..., info = NULL, label = NULL, class = NULL) {
  if (!is_installed("testthat")) {
    stop("`testhat` must be installed.")
  }
  if (!is.null(class)) stop("`class` must be NULL.")
  testthat::expect_error(object, regexp = regexp, class = "chk_error", ..., info = info, label = label)
}

#' Check Not Subset
#'
#' @description
#' Checks if not all values in values using
#'
#' `!any(x %in% values) || !length(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family set_checkers
#'
#' @seealso [any()]
#' @seealso [length()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_not_subset
#' chk_not_subset(11, 1:10)
#' try(chk_not_subset(1, 1:10))
#' @export
chk_not_subset <- function(x, values, x_name = NULL) {
  if (vld_not_subset(x, values)) {
    return(invisible(x))
  }
  values <- sort(unique(values), na.last = TRUE)
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))

  if (length(x) == 1L) {
    abort_chk(x_name, " must not match ", cc(unique(c(x, values)), " or "), x = x, values = values)
  }
  abort_chk(x_name, " must not have any values matching ", cc(values, " or "), x = x, values = values)
}

#' @describeIn chk_subset Validate Not Subset
#'
#' @examples
#' # vld_not_subset
#' vld_not_subset(numeric(0), 1:10)
#' vld_not_subset(1, 1:10)
#' vld_not_subset(11, 1:10)
#' @export
vld_not_subset <- function(x, values) !any(x %in% values) || !length(x)

test_that("vld_not_subset", {
  expect_true(vld_not_subset(integer(0), 1))
  expect_true(vld_not_subset(1, 2))
  expect_false(vld_not_subset(1:3, 1:2))
  expect_true(vld_not_subset(numeric(0), 1L))

  expect_true(vld_not_subset(-1, c(0, 1, NA)))
  expect_true(vld_not_subset(1L, 0L))
  expect_true(vld_not_subset(1, NA))

  expect_false(vld_not_subset(c(NA, 1), 1))
  expect_false(vld_not_subset(NA, NA))

  expect_false(vld_not_subset(c(NA, 1), 1:2))
})

test_that("chk_not_subset", {
  expect_identical(chk_not_subset(integer(0), 1), integer(0), 1)
  expect_invisible(chk_not_subset(integer(0), 1))
  expect_chk_error(chk_not_subset(1:3, c(0, 1, NA)), "`1:3` must not have any values matching 0, 1 or NA[.]$")
  expect_chk_error(chk_not_subset(1, c(0, 1, NA)), "^`1` must not match 1, 0 or NA[.]$")
  expect_chk_error(chk_not_subset(0L, 0L), c("^`0L` must not match 0[.]$"))
})

#' Check Sorted
#'
#' @description
#' Checks if is sorted using
#'
#' `is.unsorted(x, na.rm = TRUE)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family sorted_checkers
#'
#' @seealso [is.unsorted()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_sorted
#' chk_sorted(1:2)
#' try(chk_sorted(2:1))
#' @export
chk_sorted <- function(x, x_name = NULL) {
  if (vld_sorted(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be sorted", x = x)
}

#' @describeIn chk_sorted Validate Sorted
#'
#' @examples
#' # vld_sorted
#' vld_sorted(1:2)
#' vld_sorted(2:1)
#' @export
vld_sorted <- function(x) !is.unsorted(x, na.rm = TRUE)

test_that("vld_sorted", {
  expect_true(vld_sorted(TRUE))
  expect_true(vld_sorted(1))
  expect_true(vld_sorted(character(0)))
  expect_true(vld_sorted(NA))
  expect_true(vld_sorted(1:2))
  expect_false(vld_sorted(2:1))
  expect_true(vld_sorted(c(1:2, NA)))
  expect_false(vld_sorted(c(2:1, NA)))
})

test_that("chk_sorted", {
  expect_identical(chk_sorted(TRUE), TRUE)
  expect_invisible(chk_sorted(TRUE))
  expect_identical(chk_sorted(c(x = TRUE)), c(x = TRUE))

  expect_chk_error(chk_sorted(2:1), "^`2:1` must be sorted[.]$")
})

#' Check Compatible Lengths
#'
#' Checks objects (including vectors) have lengths that could be 'strictly
#' recycled'. That is to say they must all be either zero length or the same
#' length with some of length 1.
#'
#' @inheritParams params
#' @param ... The objects to check for compatible lengths.
#' @return The `chk_` function throws an informative error if the test fails.
#' @details This function helps to check vectors could be 'strictly recycled.'
#'   For example the function will error if you had a vector of length 2 and
#'   length 4, even though the vector of length 2 could be 'loosely recycled' to
#'   match up to the vector of length 4 when combined.
#'
#'   The intent of the function is to check that only strict recycling is
#'   occurring.
#'
#' @family length_checkers
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_compatible_lengths
#'
#' a <- integer(0)
#' b <- numeric(0)
#' chk_compatible_lengths(a, b)
#'
#' a <- 1
#' b <- 2
#' chk_compatible_lengths(a, b)
#'
#' a <- 1:3
#' b <- 1:3
#' chk_compatible_lengths(a, b)
#'
#' b <- 1
#' chk_compatible_lengths(a, b)
#'
#' b <- 1:2
#' try(chk_compatible_lengths(a, b))
#'
#' b <- 1:6
#' try(chk_compatible_lengths(a, b))
#' @export
chk_compatible_lengths <- function(..., x_name = NULL) {
  if (vld_compatible_lengths(...)) {
    return(invisible())
  }
  if (is.null(x_name)) {
    x_name <- "... objects"
  }
  chk_string(x_name)

  vec_lengths <- lengths(list(...))
  length_set <- sort(unique(vec_lengths))

  abort_chk(
    x_name,
    paste0(
      " must be all zero length or the same length with some of length of 1 ",
      "but not lengths "
    ),
    cc(length_set, conj = " and ")
  )
}

#' @describeIn chk_compatible_lengths Validate Compatible Lengths
#' @export
#' @examples
#' # vld_compatible_lengths
#'
#' a <- integer(0)
#' b <- numeric(0)
#' vld_compatible_lengths(a, b)
#'
#' a <- 1
#' b <- 2
#' vld_compatible_lengths(a, b)
#'
#' a <- 1:3
#' b <- 1:3
#' vld_compatible_lengths(a, b)
#'
#' b <- 1
#' vld_compatible_lengths(a, b)
#'
#' b <- 1:2
#' vld_compatible_lengths(a, b)
#'
#' b <- 1:6
#' vld_compatible_lengths(a, b)
vld_compatible_lengths <- function(...) {
  vec_lengths <- lengths(list(...))
  length_set <- unique(vec_lengths)

  if (length(length_set) <= 1) {
    return(TRUE)
  }
  if (length(length_set) >= 3) {
    return(FALSE)
  }
  min(length_set) == 1
}

# vld -----
test_that("validate true single vector passed", {
  x <- c(1, 3, 5)
  expect_true(vld_compatible_lengths(x))
})

test_that("validate true when vectors are all the same length", {
  x <- c(1, 3, 5)
  y <- c(2, 4, 6)
  expect_true(vld_compatible_lengths(x, y))
})

test_that("validate true when one vectors is length 1 and other vectors are the
          same length", {
  x <- c(1, 3, 5)
  y <- c(2, 4, 6)
  z <- c(1)
  expect_true(vld_compatible_lengths(x, y, z))
})

test_that("validate true when vector is length 0", {
  x <- NULL
  expect_true(vld_compatible_lengths(x))
})

test_that("validate true when nothing", {
  expect_true(vld_compatible_lengths())
})

test_that("validate false if one vectors is length 0", {
  x <- c(1, 3, 5)
  y <- c(2, 4, 6)
  z <- NULL
  expect_false(vld_compatible_lengths(x, y, z))
})

test_that("validate false when two vectors have multiple lengths", {
  x <- c(1, 3, 5)
  y <- c(2, 4)
  expect_false(vld_compatible_lengths(x, y))
})

test_that("validate false when vectors are multiple lengths", {
  v <- c(1, 3, 5)
  x <- c(2, 4)
  y <- c(1, 2, 3, 4)
  z <- c(8)
  expect_false(vld_compatible_lengths(v, x, y, z))
})

test_that("validate false if data frame of two lengths passed", {
  x <- data.frame(a = c(1, 2, 3), b = c(4, 5, 6))
  y <- data.frame(a = c(1, 2), b = c(6, 7), c = c(8, 9))
  expect_false(vld_compatible_lengths(x, y))
})

test_that("validate true when data frames passed", {
  x <- data.frame(a = c(1, 2, 3), b = c(4, 5, 6))
  y <- data.frame(a = c(1, 2, 3))
  expect_true(vld_compatible_lengths(x, y))
})

test_that("validate true if list passed", {
  x <- list(a = c(1, 2, 3), b = c(4, 5, 6))
  expect_true(vld_compatible_lengths(x))
})

test_that("validate true if lists passed", {
  x <- list(a = c(1, 2, 3), b = c(4, 5, 6))
  y <- list(e = c(1, 2, 3), f = c(4, 5, 6))
  expect_true(vld_compatible_lengths(x, y))
})

# chk -----
test_that("passes when single value passed", {
  x <- 1
  expect_null(chk_compatible_lengths(x))
  expect_invisible(chk_compatible_lengths(x))
})

test_that("passes when single vector passed", {
  x <- c(1, 3, 5)
  expect_null(chk_compatible_lengths(x))
  expect_invisible(chk_compatible_lengths(x))
})

test_that("passes when multiple vectors of same length passed", {
  x <- c(1, 3, 5)
  y <- c(2, 4, 6)
  z <- c(7, 8, 0)
  expect_null(chk_compatible_lengths(x, y, z))
  expect_invisible(chk_compatible_lengths(x, y, z))
})

test_that("passes when multiple vectors of same length passed and vectors of
          length one passed", {
  x <- c(1, 3, 5)
  y <- c(2, 4, 6)
  z <- 7
  expect_null(chk_compatible_lengths(x, y, z))
  expect_invisible(chk_compatible_lengths(x, y, z))
})

test_that("passes when only vector of length zero passed", {
  x <- NULL
  expect_null(chk_compatible_lengths(x))
  expect_invisible(chk_compatible_lengths(x))
})

test_that("throws error when vector of length zero and other length passed", {
  x <- NULL
  y <- c(1, 2, 3)
  expect_error(
    chk_compatible_lengths(x, y),
    regexp = paste0(
      "^... objects must be all zero length or the same length ",
      "with some of length of 1 but not lengths 0 and 3\\.$"
    )
  )
})

test_that("throws error when vector have two different lengths", {
  x <- c(4, 5)
  y <- c(1, 2, 3)
  expect_error(
    chk_compatible_lengths(x, y),
    regexp = paste0(
      "^... objects must be all zero length or the same length ",
      "with some of length of 1 but not lengths 2 and 3\\.$"
    )
  )
})

test_that("throws error when vector have three different lengths", {
  x <- c(4, 5)
  z <- c(6, 7, 8, 9)
  y <- c(1, 2, 3)
  expect_error(
    chk_compatible_lengths(x, z, y, x_name = "`test vectors`"),
    regexp = paste0(
      "^`test vectors` must be all zero length or the same length ",
      "with some of length of 1 but not lengths 2, 3 and 4\\.$"
    )
  )
})

#' Check Length
#'
#' @description
#' Checks length is a particular value or range using
#'
#' `length(x) >= length && length(x) <= upper`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family length_checkers
#'
#' @seealso [length()], [check_length()], [check_dim()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_length
#' chk_length("text")
#' try(vld_length("text", length = 2))
#' @export
chk_length <- function(x, length = 1L, upper = length, x_name = NULL) {
  if (vld_length(x, length, upper)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  if (length == upper) {
    abort_chk(x_name, " must be length ", length, " not ", length(x), x = x, length = length)
  }
  abort_chk(x_name, " must have a length between ", length, " and ", upper, " not ", length(x), x = x, length = length)
}

#' @describeIn chk_length Validate Length
#'
#' @examples
#' # vld_length
#' vld_length(2:1, 2)
#' vld_length(2:1, 1)
#' @export
vld_length <- function(x, length = 1L, upper = length) length(x) >= length && length(x) <= upper

test_that("vld_length", {
  expect_true(vld_length(1))
  expect_true(vld_length(1:2, 2))
  expect_true(vld_length(NA))
  expect_true(vld_length(character(0), 0))
  expect_true(vld_length(data.frame(x = 1:2)))
  expect_false(vld_length(1:2))
  expect_true(vld_length(1:2, upper = 2))
  expect_true(vld_length(1:2, upper = 3))
})

test_that("chk_length", {
  expect_identical(chk_length(1), 1)
  expect_invisible(chk_length(1))
  expect_chk_error(chk_length(1:2), "^`1:2` must be length 1 not 2[.]$")
  expect_chk_error(chk_length(1:2, length = 3, upper = 4), "^`1:2` must have a length between 3 and 4 not 2[.]$")
  expect_chk_error(chk_length(1:2, x_name = 1L), "^1 must be length 1 not 2[.]$")
})

#' Check String
#'
#' @description
#' Checks if string
#'
#' `is.character(x) && length(x) == 1L && !anyNA(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family scalar_checkers
#'
#' @seealso [length()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_string
#' chk_string("1")
#' try(chk_string(1))
#' @export
chk_string <- function(x, x_name = NULL) {
  if (vld_string(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be a string (non-missing character scalar)", x = x)
}

#' @describeIn chk_string Validate String
#'
#' @examples
#' # vld_string
#' vld_string("1")
#' vld_string("")
#' vld_string(1)
#' vld_string(NA_character_)
#' vld_string(c("1", "1"))
#' @export
vld_string <- function(x) {
  is.character(x) && length(x) == 1L && !anyNA(x)
}

#' Check Character or Factor
#'
#' @description
#' Checks if character or factor using
#'
#' `is.character(x) || is.factor(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family data_type_checkers
#' @family factor_checkers
#'
#' @seealso [is.character()]
#' @seealso [is.factor()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_character_or_factor
#' chk_character_or_factor("1")
#' chk_character_or_factor(factor("1"))
#' try(chk_character(1))
#' @export
chk_character_or_factor <- function(x, x_name = NULL) {
  if (vld_character_or_factor(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be character or factor", x = x)
}

#' @describeIn chk_character_or_factor Validate Character or Factor
#'
#' @examples
#' # vld_character_or_factor
#' vld_character_or_factor("1")
#' vld_character_or_factor(matrix("a"))
#' vld_character_or_factor(character(0))
#' vld_character_or_factor(NA_character_)
#' vld_character_or_factor(1)
#' vld_character_or_factor(TRUE)
#' vld_character_or_factor(factor("text"))
#' @export
vld_character_or_factor <- function(x) is.character(x) || is.factor(x)

test_that("vld_character_or_factor", {
  expect_true(vld_character_or_factor("1"))
  expect_true(vld_character_or_factor(matrix("a")))
  expect_true(vld_character_or_factor(character(0)))
  expect_true(vld_character_or_factor(NA_character_))
  expect_false(vld_character_or_factor(1))
  expect_false(vld_character_or_factor(TRUE))
  expect_true(vld_character_or_factor(factor("text")))
})

test_that("chk_character_or_factor", {
  expect_identical(chk_character_or_factor("1"), "1")
  expect_invisible(chk_character_or_factor("1"))
  expect_chk_error(
    chk_character_or_factor(TRUE),
    "^`TRUE` must be character or factor[.]$"
  )
})

#' Check Class
#'
#' @description
#' Checks inherits from class using
#'
#' `inherits(x, class)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family id_checkers
#'
#' @seealso [inherits()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' chk_is(1, "numeric")
#' try(chk_is(1L, "double"))
#' @export
chk_is <- function(x, class, x_name = NULL) {
  if (vld_is(x, class)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  .class <- class
  abort_chk(x_name, " must inherit from class '", .class, "'", x = x, .class = .class)
}

#' @describeIn chk_is Validate Inherits from Class
#'
#' @examples
#'
#' # vld_is
#' vld_is(numeric(0), "numeric")
#' vld_is(1L, "double")
#' @export
vld_is <- function(x, class) {
  inherits(x, class)
}

test_that("vld_is", {
  expect_true(vld_is(1, "numeric"))
  expect_false(vld_is(1L, "numeric"))
  expect_true(vld_is(1L, "integer"))

  x <- list()
  class(x) <- c("a", "b")
  expect_true(vld_is(x, "a"))
  expect_true(vld_is(x, "b"))
})

test_that("chk_is", {
  expect_identical(chk_is(1L, "integer"), 1L, "integer")
  expect_invisible(chk_is(1L, "integer"))
  expect_chk_error(chk_is(1, "integer"), "^`1` must inherit from class 'integer'[.]$")
  expect_chk_error(
    chk_is(matrix(1), "numeric"),
    "`matrix[(]1[)]` must inherit from class 'numeric'[.]$"
  )
  x <- list()
  class(x) <- c("a", "b")
  expect_chk_error(chk_is(x, "c"), "`x` must inherit from class 'c'")
  expect_chk_error(chk_is(x, "c", x_name = "c"), "C must inherit from class 'c'")
})

#' Check File Extension
#'
#' @description
#' Checks extension using
#'
#' `vld_string(x) && vld_subset(tools::file_ext(x), ext)`
#'
#' The user may want to use [toupper()] or [tolower()]
#' to ensure the case matches.
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family file_checkers
#'
#' @seealso [vld_string()]
#' @seealso [vld_subset()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_ext
#' try(chk_ext("file1.pdf", "png"))
#' @export
chk_ext <- function(x, ext, x_name = NULL) {
  if (vld_ext(x, ext)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  chk_string(x, x_name = x_name)
  abort_chk(
    x_name, " must have extension ", cc(ext, " or "), " (not '",
    tools::file_ext(x), "')",
    x = x, ext = ext
  )
}

#' @describeIn chk_ext Validate File Extension
#'
#' @examples
#' # vld_ext
#' vld_ext("oeu.pdf", "pdf")
#' vld_ext(toupper("oeu.pdf"), "PDF")
#' @export
vld_ext <- function(x, ext) vld_string(x) && vld_subset(tools::file_ext(x), ext)

#' Workaround: Avoid backtraces in examples
#'
#' This example will run first and set the option for the process that builds
#' the example.
#' (By default, pkgdown builds in a separate process.)
#' This also produces a help page that isn't linked from anywhere.
#'
#' @name aaa
#' @keywords internal
#' @examples
#' options(rlang_backtrace_on_error = "none")
NULL

#' Check Scalar
#'
#' @description
#' Checks if is a vector using
#'
#' `length(x) == 1L`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family scalar_checkers
#'
#' @seealso [length()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_scalar
#' chk_scalar(1)
#' chk_scalar(list(1))
#' try(chk_scalar(1:2))
#' @export
chk_scalar <- function(x, x_name = NULL) {
  if (vld_scalar(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be a scalar (length 1)", x = x)
}

#' @describeIn chk_scalar Validate Scalar
#'
#' @examples
#' # vld_scalar
#' vld_scalar(1)
#' @export
vld_scalar <- function(x) length(x) == 1L

#'  Check List
#'
#' @description
#' Checks if is a list using
#'
#' `is.list(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family data_structure_checkers
#'
#' @seealso [is.list()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_list
#' chk_list(list())
#' try(chk_list(1))
#' @export
chk_list <- function(x, x_name = NULL) {
  if (vld_list(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be a list", x = x)
}

#' @describeIn chk_list Validate List
#'
#' @examples
#' # vld_list
#' vld_list(list())
#' vld_list(list(x = 1))
#' vld_list(mtcars)
#' vld_list(1)
#' vld_list(NULL)
#' @export
vld_list <- function(x) is.list(x)

#' Check FALSE
#'
#' @description
#' Check if FALSE using
#'
#' `is.logical(x) && length(x) == 1L && !anyNA(x) && !x`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family logical_checkers
#' @family scalar_checkers
#'
#' @seealso [is.logical()]
#' @seealso [length()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_false
#' chk_false(FALSE)
#' try(chk_false(0))
#' @export
chk_false <- function(x, x_name = NULL) {
  if (vld_false(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be FALSE", x = x)
}

#' @describeIn chk_false Validate FALSE
#'
#' @examples
#' # vld_false
#' vld_false(TRUE)
#' vld_false(FALSE)
#' vld_false(NA)
#' vld_false(0)
#' vld_false(c(FALSE, FALSE))
#' @export
vld_false <- function(x) is.logical(x) && length(x) == 1L && !anyNA(x) && !x

#'  Check Vector
#'
#' @description
#' Checks if is a vector using
#'
#' `(is.atomic(x) && !is.matrix(x) && !is.array(x)) || is.list(x)`
#'
#' @details
#' `is.vector(x)` is not reliable because it returns TRUE only
#' if the object is a vector with no attributes apart from names.
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family data_structure_checkers
#'
#' @seealso [is.atomic()], [is.matrix()], [is.array()], [is.list()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_vector
#' chk_vector(1)
#' chk_vector(list())
#' try(chk_vector(matrix(1)))
#' @export
chk_vector <- function(x, x_name = NULL) {
  if (vld_vector(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be a vector", x = x)
}

#' @describeIn chk_vector Validate Vector
#'
#' @examples
#' # vld_vector
#' vld_vector(1)
#' @export
vld_vector <- function(x) (is.atomic(x) && !is.matrix(x) && !is.array(x)) || is.list(x)

test_that("vld_vector", {
  expect_true(vld_vector(1))
  expect_true(vld_vector(NA))
  expect_true(vld_vector(1:2))
  expect_true(vld_vector(integer(0)))
  expect_true(vld_vector(list()))
  expect_true(vld_vector(list(x = 1)))
  x <- 1
  attr(x, "oeu") <- 1
  expect_true(vld_vector(x))
  x <- list()
  class(x) <- "foo"
  expect_true(vld_vector(x))
  expect_false(vld_vector(matrix(1)))
})

test_that("chk_vector", {
  expect_identical(chk_vector(1), 1)
  expect_invisible(chk_vector(1))

  expect_chk_error(chk_vector(matrix(1)), "^`matrix[(]1[)]` must be a vector[.]$")
})

#' Concatenate with Commas
#'
#' Concatenates object values into a string with each value separated by a comma
#' and the last value separated by a conjunction.
#'
#' By default, if x has more than 10 values an ellipsis is used
#' to ensure only 10 values are displayed (including the ellipsis).
#'
#' @param x The object to concatenate.
#' @param conj A string of the conjunction to separate the last value by.
#' @param sep A string of the separator.
#' @param brac A string to brac the values by.
#' @param ellipsis A numeric scalar of the maximum number of values to display
#' before using an ellipsis.
#' @param chk A flag specifying whether to check the other parameters.
#'
#' @return A string.
#' @export
#'
#' @examples
#' cc(1:2)
#' cc(1:2, conj = " or")
#' cc(3:1, brac = "'")
#' cc(1:11)
#' cc(as.character(1:2))
cc <- function(x, conj = ", ", sep = ", ",
               brac = if (is.character(x) || is.factor(x)) "'" else "",
               ellipsis = 10L, chk = TRUE) {
  if (chk) {
    chk_string(conj)
    chk_string(sep)
    chk_s3_class(brac, "character")
    chk_range(length(brac), 1:2)
    chk_whole_number(ellipsis)
    chk_gte(ellipsis, 3)
  }

  if (!length(x)) {
    return(character(0))
  }
  x <- p0(brac[1], x, brac[length(brac)])
  n <- length(x)
  if (n == 1L) {
    return(x)
  }
  if (n == 2L) {
    return(p(x, collapse = conj))
  }
  if (n > ellipsis) {
    x <- c(x[1:(ellipsis - 2)], "...", x[n])
    n <- ellipsis
  }
  p(c(p0(x[1:(n - 1)], collapse = sep), x[n]), collapse = conj)
}

# necessary hack because for data frames
# argument 'incomparables != FALSE' is not used (yet)
.anyDuplicated <- function(x, incomparables = FALSE) {
  if (vld_false(incomparables) || is.null(incomparables) || !length(incomparables)) {
    return(anyDuplicated(x))
  }
  if (!is.data.frame(x)) {
    return(anyDuplicated(x, incomparables = incomparables))
  }

  y <- as.data.frame(lapply(x, function(col, y) col %in% y, y = incomparables))
  y <- apply(y, 1, any)
  x <- x[!y, , drop = FALSE]

  anyDuplicated(x)
}

# remove when no longer support R < 3.6.0
str2lang <- function(text) {
  ex <- parse(text = text, keep.source = FALSE)
  ex[[1L]]
}

#' Check Not Empty
#'
#' @description
#' Checks if not empty using
#'
#' `length(x) != 0L`
#'
#' **Pass**: `1`, `1:2`, `NA`, `matrix(1:3)`, `list(1)`, `data.frame(x = 1)`.
#'
#' **Fail**: `NULL`, `logical(0)`, `list()`, `data.frame()`.
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family misc_checkers
#'
#' @seealso [length()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_not_empty
#' chk_not_empty(1)
#' try(chk_not_empty(numeric(0)))
#' @export
chk_not_empty <- function(x, x_name = NULL) {
  if (vld_not_empty(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must not be empty (zero length)", x = x)
}

#' @describeIn chk_not_empty Validate Not Empty
#'
#' @examples
#' # vld_not_empty
#' vld_not_empty(1)
#' vld_not_empty(matrix(1:3))
#' vld_not_empty(character(0))
#' vld_not_empty(list(1))
#' vld_not_empty(NULL)
#' vld_not_empty(list())
#' @export
vld_not_empty <- function(x) length(x) != 0L

test_that("vld_not_empty", {
  expect_true(vld_not_empty(1))
  expect_true(vld_not_empty(matrix(1:3)))
  expect_true(vld_not_empty(NA))
  expect_true(vld_not_empty(list(1)))
  expect_true(vld_not_empty(matrix(1)))
  expect_true(vld_not_empty(data.frame(x = 1)))
  expect_false(vld_not_empty(NULL))
  expect_false(vld_not_empty(integer(0)))
  expect_false(vld_not_empty(list()))
  expect_false(vld_not_empty(matrix(integer(0))))
  expect_false(vld_not_empty(data.frame()))
})

test_that("chk_not_empty", {
  expect_identical(chk_not_empty(1), 1)
  expect_invisible(chk_not_empty(1))
  expect_chk_error(chk_not_empty(NULL), "^`NULL` must not be empty [(]zero length[)][.]$")
  expect_chk_error(chk_not_empty(NULL, x_name = 1), "^1 must not be empty [(]zero length[)][.]$")
})

#' Check Equal
#'
#' @description
#' Checks if is equal (identical within tolerance) to y using
#'
#' `vld_true(all.equal(x, y, tolerance))`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family equal_checkers
#'
#' @seealso [vld_true()]
#' @seealso [all.equal()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_equal
#' chk_equal(1, 1.00000001)
#' try(chk_equal(1, 1.0000001))
#' chk_equal(1, 1L)
#' chk_equal(c(x = 1), c(x = 1L))
#' try(chk_equal(c(x = 1), c(y = 1L)))
#' @export
chk_equal <- function(x, y, tolerance = sqrt(.Machine$double.eps),
                      x_name = NULL) {
  if (vld_equal(x, y, tolerance = tolerance)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  y_name <- as_label(y)
  obj_of <- if (grepl("^<", y_name)) "the y object of class " else NULL
  abort_chk(x_name, " must be equal to ",
    obj_of, y_name,
    x = x, y = y
  )
}

#' @describeIn chk_equal Validate Equal
#'
#' @examples
#' vld_equal(1, 1.00000001)
#' @export
vld_equal <- function(x, y, tolerance = sqrt(.Machine$double.eps)) {
  vld_true(all.equal(x, y, tolerance))
}

#' Check Unique
#'
#' @description
#' Checks if unique using
#'
#' `!anyDuplicated(x, incomparables = incomparables)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family misc_checkers
#'
#' @seealso [anyDuplicated()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_unique
#' chk_unique(c(NA, 2))
#' try(chk_unique(c(NA, NA, 2)))
#' chk_unique(c(NA, NA, 2), incomparables = NA)
#' @export
chk_unique <- function(x, incomparables = FALSE, x_name = NULL) {
  if (vld_unique(x, incomparables = incomparables)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be unique", x = x, incomparables = incomparables)
}

#' @describeIn chk_unique Validate Unique
#'
#' @examples
#' # vld_unique
#' vld_unique(NULL)
#' vld_unique(numeric(0))
#' vld_unique(c(NA, 2))
#' vld_unique(c(NA, NA, 2))
#' vld_unique(c(NA, NA, 2), incomparables = NA)
#' @export
vld_unique <- function(x, incomparables = FALSE) {
  !.anyDuplicated(x, incomparables = incomparables)
}

#' Check All
#'
#' @description
#' Checks all elements using
#'
#' `all(vapply(x, chk_fun, TRUE, ...))`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family all_checkers
#'
#' @seealso [all()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_all
#' chk_all(TRUE, chk_lgl)
#' # FIXME try(chk_all(1, chk_lgl))
#' chk_all(c(TRUE, NA), chk_lgl)
#' @export
chk_all <- function(x, chk_fun, ..., x_name = NULL) {
  if (is.null(x)) {
    if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
    return(chk_fun(x, ..., x_name = x_name))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  x_name <- paste0("all elements of ", x_name)

  args <- list(...)
  args$X <- x
  args$FUN <- chk_fun
  args$x_name <- x_name

  do.call("lapply", args)
  invisible(x)
}

#' @describeIn chk_all Validate All
#'
#' @examples
#' # vld_all
#' vld_all(c(TRUE, NA), vld_lgl)
#' @export
vld_all <- function(x, vld_fun, ...) {
  if (is.null(x)) {
    return(vld_fun(x, ...))
  }

  args <- list(...)
  args$X <- x
  args$FUN <- vld_fun
  args$FUN.VALUE <- TRUE

  all(do.call("vapply", args))
}

#' Check Missing Argument
#'
#' @description
#' Checks argument missing using
#'
#' `missing(x)`
#'
#' @details
#' Currently only checks if value is available
#' (as opposed to whether it was specified).
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family missing_checkers
#'
#' @seealso [missing()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_missing
#' fun <- function(x) {
#'   chk_missing(x)
#' }
#' fun()
#' try(fun(1))
#' @export
chk_missing <- function(x, x_name = NULL) {
  if (vld_missing(x)) {
    return(invisible(NULL))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be missing")
}

#' @describeIn chk_missing Validate Missing Argument
#'
#' @examples
#' # vld_missing
#' fun <- function(x) {
#'   vld_missing(x)
#' }
#' fun()
#' fun(1)
#' @export
vld_missing <- function(x) {
  missing(x)
}

test_that("vld_missing", {
  expect_true(vld_missing())
  expect_false(vld_missing(1))
})

test_that("vld_missing in function", {
  fun <- function(x) vld_missing(x)
  expect_true(fun())
  expect_false(fun(1))
})

test_that("vld_missing in function default", {
  fun <- function(x = 1) vld_missing(x)
  # this might be expected to be true if testing whether specified
  expect_false(fun())
  expect_false(fun(1))
})

test_that("chk_missing", {
  expect_null(chk_missing())
  expect_invisible(chk_missing())
  expect_chk_error(chk_missing(1), "^`1` must be missing\\.$")
  expect_chk_error(chk_missing(1, x_name = "`y`"), "^`y` must be missing\\.$")
})

test_that("chk_missing in function", {
  fun <- function(x) chk_missing(x)

  expect_null(fun())
  expect_invisible(fun())
  expect_chk_error(fun(1), "^`x` must be missing\\.$")
})

test_that("chk_missing in function default", {
  fun <- function(x = 1) chk_missing(x)

  # this might be expected to be an error if testing whether specified
  expect_chk_error(fun(), "^`x` must be missing\\.$")
  # this might be expected to be an error if testing whether specified
  expect_chk_error(fun(NULL), "^`x` must be missing\\.$")
  expect_chk_error(fun(1), "^`x` must be missing\\.$")
})

#' Check Greater Than
#'
#' @description
#' Checks if all non-missing values are greater than value using
#'
#' `all(x[!is.na(x)] > value)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family range_checkers
#'
#' @seealso [all()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_gt
#' chk_gt(0.1)
#' try(chk_gt(c(0.1, -0.2)))
#' @export
chk_gt <- function(x, value = 0, x_name = NULL) {
  if (vld_gt(x, value)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  if (length(x) == 1L) {
    abort_chk(x_name, " must be greater than ", cc(value), ", not ", cc(x), x = x, value = value)
  }
  abort_chk(x_name, " must have values greater than ", cc(value), x = x, value = value)
}

#' @describeIn chk_gt Validate Greater Than
#'
#' @examples
#' # vld_gt
#' vld_gt(numeric(0))
#' vld_gt(0)
#' vld_gt(0.1)
#' vld_gt(c(0.1, 0.2, NA))
#' vld_gt(c(0.1, -0.2))
#' vld_gt(c(-0.1, 0.2), value = -1)
#' vld_gt("b", value = "a")
#' @export
vld_gt <- function(x, value = 0) all(x[!is.na(x)] > value)

#' Check NULL Or
#'
#' @description
#' Checks if NULL or passes test.
#'
#' @inheritParams params
#' @param chk A chk function. Soft-deprecated for vld. `r lifecycle::badge("deprecated")`

#' @param vld A vld function.
#' @param ... Arguments passed to chk.
#'
#' @return An informative error if the test fails.
#' @export
#'
#' @examples
#' chk_null_or(NULL, chk_number)
#' chk_null_or(1, chk_number)
#' try(chk_null_or("1", chk_number))
chk_null_or <- function(x, chk, ..., vld, x_name = NULL) {
  if (!missing(chk)) {
    lifecycle::deprecate_soft("0.6.1", "chk_null_or(chk)", "chk_null_or(vld)")
  }
  if (is.null(x)) {
    return(invisible(x))
  }
  if (missing(vld)) {
    try <- try_chk(chk(x, ...))
    if (is.null(try)) {
      return(invisible(x))
    }

    if (is.null(x_name)) {
      x_name <- deparse_backtick_chk(substitute(x))
      try <- try_chk(chk(x, ..., x_name = x_name))
    }
    msg <- try_msg(try)
    msg <- sub("[.]$", " or NULL.", msg)
    abort_chk(msg, x = x)
  }
  if (vld(x, ...)) {
    return(invisible(x))
  }

  if (is.null(x_name)) {
    x_name <- deparse_backtick_chk(substitute(x))
  }
  chk <- deparse(substitute(vld))
  chk <- sub("^(chk::){0,1}vld_", "chk_", chk)
  chk <- paste0(chk, "(x, ..., x_name = x_name)")
  chk <- parse(text = chk)
  try <- try_chk(eval(chk))
  msg <- try_msg(try)
  msg <- sub("[.]$", " or NULL.", msg)
  abort_chk(msg, x = x)
}

test_that("chk_null_or works vld", {
  expect_identical(chk_null_or(NULL, vld = vld_number), NULL)
  expect_invisible(chk_null_or(NULL, vld = vld_number), NULL)
  expect_identical(chk_null_or(1, vld = vld_number), 1)

  expect_chk_error(
    chk_null_or("1", vld = vld_number),
    "^`\"1\"` must be a number [(]non-missing numeric scalar[)] or NULL[.]$"
  )
})

test_that("chk_null_or still works chk", {
  rlang::local_options(lifecycle_verbosity = "quiet")

  expect_identical(chk_null_or(NULL, chk_number), NULL, chk_number)
  expect_invisible(chk_null_or(NULL, chk_number))
  expect_identical(chk_null_or(1, chk_number), 1, chk_number)

  expect_chk_error(
    chk_null_or("1", chk_number),
    "^`\"1\"` must be a number [(]non-missing numeric scalar[)] or NULL[.]$"
  )
})

test_that("chk_null_or vld overrides chk", {
  rlang::local_options(lifecycle_verbosity = "quiet")
  expect_identical(chk_null_or(1, chk_flag, vld = vld_number), 1)
  expect_chk_error(
    chk_null_or(1, chk = chk_number, vld = vld_flag),
    "^`1` must be a flag [(]TRUE or FALSE[)] or NULL[.]$"
  )
})

test_that("chk_null_or gives correct error message", {
  rlang::local_options(lifecycle_verbosity = "quiet")
  expect_error(
    chk_null_or("2000", vld = vld_whole_number),
    "^`\"2000\"` must be a whole number \\(non-missing integer scalar or double equivalent\\) or NULL\\.$"
  )
  expect_error(
    chk_null_or("2000", vld = chk::vld_whole_number),
    "^`\"2000\"` must be a whole number \\(non-missing integer scalar or double equivalent\\) or NULL\\.$"
  )
})

#' Check Set Ordered
#'
#' @description
#' Checks if the first occurrence of each shared element
#' in x is equivalent to the first occurrence of each shared element in values using
#' `vld_equivalent(unique(x[x %in% values]), values[values %in% x])`.
#'
#' @inheritParams params
#' @return
#' The `chk_` function throws an informative error if the test fails.
#'
#' The `vld_` function returns a flag indicating whether the test was met.
#'
#' @family set_checkers
#'
#' @seealso [vld_equivalent()]
#' @seealso [unique()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#' @export
#'
#' @examples
#'
#' # chk_orderset
#' chk_orderset(1:2, 1:2)
#' try(chk_orderset(2:1, 1:2))
chk_orderset <- function(x, values, x_name = NULL) {
  if (vld_orderset(x, values)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  values <- values[values %in% x]
  abort_chk(x_name, " must have (the first occurence of) each of the following elements in the following order: ", cc(values), x = x, values = values)
}

#' @describeIn chk_setequal Validate Set Ordered
#'
#' @export
#'
#' @examples
#'
#' # vld_orderset
#' vld_orderset(1, 1)
#' vld_orderset(1:2, 2:1)
#' vld_orderset(1, 2:1)
#' vld_orderset(1:2, 2)
vld_orderset <- function(x, values) {
  vld_equivalent(unique(x[x %in% values]), values[values %in% x])
}

test_that("vld_orderset", {
  expect_true(vld_orderset(numeric(0), numeric(0)))
  expect_true(vld_orderset(NA, NA))
  expect_true(vld_orderset(1, 1))
  expect_true(vld_orderset(c(1, NA), c(1, NA)))
  expect_true(vld_orderset(1, 2:1))
  expect_true(vld_orderset(1, 2))
  expect_true(vld_orderset(1:2, 2))
  expect_false(vld_orderset(1:2, 2:1))
  expect_false(vld_orderset(c(NA, 1), c(1, NA)))
})

test_that("chk_orderset", {
  expect_identical(chk_orderset(1, 1), 1, 1)
  expect_invisible(chk_orderset(1, 1))
  expect_chk_error(chk_orderset(2:1, 1:2), "`2:1` must have [(]the first occurence of[)] each of the following elements in the following order: 1, 2[.]$")
})

#' Parameter Descriptions for chk Package
#'
#' Default parameter descriptions which may be overridden in individual
#' functions.
#'
#' A flag is a non-missing logical scalar.
#'
#' A string is a non-missing character scalar.
#'
#' @param x The object to check.
#' @param x_name A string of the name of object x or NULL.
#' @param y An object to check against.
#' @param chk A flag specifying whether to check the other parameters.
#' @param chk_fun A chk_ function.
#' @param tolerance A non-negative numeric scalar.
#' @param ext A character vector of the permitted file extensions
#'   (without the .).
#' @param exists A flag specifying whether the files/directories must (or must not) exist.
#' @param value A non-missing scalar of a value.
#' @param range A non-missing sorted vector of length 2 of the lower and
#'   upper permitted values.
#' @param inclusive A flag specifying whether the range is exclusive.
#' @param regexp A string of a regular expression.
#' @param values A vector of the permitted values.
#' @param class A string specifying the class.
#' @param length A count of the length.
#' @param upper A count of the max length.
#' @param formals A count of the number of formal arguments.
#' @param incomparables A vector of values that cannot be compared.
#'   FALSE means that all values can be compared.
#' @param by A character vector specifying the column names to join x and y on.
#' If named the names are the corresponding columns in x.
#' @param exclusive A flag specifying whether x must only include columns named in values.
#' @param order A flag specifying whether the order of columns in x must match names in values.
#' @param nrow A flag or a whole numeric vector of the value, value range or possible values.
#' @param key A character vector of the columns that represent a unique key.
#' @param vld_fun A vld_ function.
#' @param ... Additional arguments.
#' @return
#' The `chk_` function throws an informative error if the test fails or
#' returns the original object if successful so it can used in pipes.
#'
#' The `vld_` function returns a flag indicating whether the test was met.
#'
#' @keywords internal
#' @aliases parameters arguments args
#' @usage NULL
# nocov start
params <- function(...) NULL
# nocov end

#' Check Logical Scalar
#'
#' @description
#' Checks if logical scalar using
#'
#' `is.logical(x) && length(x) == 1L`
#'
#' If you only want to check the data type (not whether `length(x) == 1`),
#' you should use the [chk_logical()] function.
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family logical_checkers
#' @family scalar_checkers
#'
#' @seealso [is.logical()]
#' @seealso [length()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_lgl
#' chk_lgl(NA)
#' try(chk_lgl(1))
#' @export
chk_lgl <- function(x, x_name = NULL) {
  if (vld_lgl(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be a logical scalar (TRUE, FALSE or NA)", x = x)
}

#' @describeIn chk_lgl Validate Logical Scalar
#'
#' @examples
#' # vld_lgl
#' vld_lgl(TRUE)
#' vld_lgl(FALSE)
#' vld_lgl(NA)
#' vld_lgl(1)
#' vld_lgl(c(TRUE, TRUE))
#' @export
vld_lgl <- function(x) {
  deprecate_soft(
    "0.9.3",
    what = "chk::chk_lgl()",
    details = "Please use `chk::chk_scalar(x);` `chk::chk_logical(x)` instead",
    id = "chk_lgl"
  )
  is.logical(x) && length(x) == 1L
}

test_that("vld_lgl", {
  rlang::local_options(lifecycle_verbosity = "quiet")

  expect_true(vld_lgl(TRUE))
  expect_true(vld_lgl(FALSE))
  expect_false(vld_lgl(NA_real_))
  expect_false(vld_lgl(logical(0)))
  expect_false(vld_flag(c(FALSE, TRUE)))
  expect_false(vld_lgl(1))
})

test_that("chk_lgl", {
  rlang::local_options(lifecycle_verbosity = "quiet")

  expect_identical(chk_lgl(NA), NA)
  expect_invisible(chk_lgl(NA))
  expect_chk_error(chk_lgl(1), "^`1` must be a logical scalar [(]TRUE, FALSE or NA[)][.]$")
  expect_chk_error(chk_lgl(1, x_name = 2), "^2 must be a logical scalar [(]TRUE, FALSE or NA[)][.]$")
})

#' Check Double
#'
#' @description
#' Checks if double using
#'
#' `is.double(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family data_type_checkers
#'
#' @seealso [is.double()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_double
#' chk_double(1)
#' try(chk_double(1L))
#' @export
chk_double <- function(x, x_name = NULL) {
  if (vld_double(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be double", x = x)
}

#' @describeIn chk_double Validate Double
#'
#' @examples
#' # vld_double
#' vld_double(1)
#' vld_double(matrix(c(1, 2, 3, 4), nrow = 2L))
#' vld_double(double(0))
#' vld_double(numeric(0))
#' vld_double(NA_real_)
#' vld_double(1L)
#' vld_double(TRUE)
#' @export
vld_double <- function(x) is.double(x)

test_that("vld_double", {
  expect_true(vld_double(1))
  expect_true(vld_double(matrix(c(1, 2))))
  expect_true(vld_double(double(0)))
  expect_true(vld_double(NA_real_))
  expect_false(vld_double(1L))
  expect_false(vld_double(TRUE))
})

test_that("chk_double", {
  expect_identical(chk_double(1), 1)
  expect_invisible(chk_double(1))
  expect_chk_error(
    chk_double(TRUE),
    "^`TRUE` must be double[.]$"
  )
})

#' Check Character
#'
#' @description
#' Checks if character using
#'
#' `is.character(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family data_type_checkers
#'
#' @seealso [is.character()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_character
#' chk_character("1")
#' try(chk_character(1))
#' @export
chk_character <- function(x, x_name = NULL) {
  if (vld_character(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be character", x = x)
}

#' @describeIn chk_character Validate Character
#'
#' @examples
#' # vld_character
#' vld_character("1")
#' vld_character(matrix("a"))
#' vld_character(character(0))
#' vld_character(NA_character_)
#' vld_character(1)
#' vld_character(TRUE)
#' vld_character(factor("text"))
#' @export
vld_character <- function(x) is.character(x)

test_that("vld_character", {
  expect_true(vld_character("1"))
  expect_true(vld_character(matrix("a")))
  expect_true(vld_character(character(0)))
  expect_true(vld_character(NA_character_))
  expect_false(vld_character(1))
  expect_false(vld_character(TRUE))
  expect_false(vld_character(factor("text")))
})

test_that("chk_character", {
  expect_identical(chk_character("1"), "1")
  expect_invisible(chk_character("1"))
  expect_chk_error(
    chk_character(TRUE),
    "^`TRUE` must be character[.]$"
  )
})

#' Check Whole Number
#'
#' @description
#'  Checks if non-missing integer scalar or double equivalent using
#'
#' `vld_number(x) && (is.integer(x) || vld_true(all.equal(x, trunc(x))))`
#'
#' **Pass**: `1`, `2L`, `1e10`, `-Inf`
#'
#' **Fail**: `"a"`, `1:3`, `NA_integer_`, `log(10)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family scalar_checker
#' @family whole_number_checkers
#'
#' @seealso [is.integer()]
#' @seealso [vld_true()]
#' @seealso [vld_number()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_whole_number
#' chk_whole_number(2)
#' try(chk_whole_number(1.1))
#' @export
chk_whole_number <- function(x, x_name = NULL) {
  if (vld_whole_number(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(
    x_name,
    " must be a whole number (non-missing integer scalar or double equivalent)",
    x = x
  )
}

#' @describeIn chk_whole_number Validate Whole Number
#'
#' @examples
#' # vld_whole_number
#' vld_whole_number(2)
#' @export
vld_whole_number <- function(x) {
  vld_number(x) &&
    (is.integer(x) || vld_true(all.equal(x, trunc(x))))
}

test_that("vld_whole_number", {
  expect_chk_error(chk_whole_number(1.1), "^`1.1` must be a whole number [(]non-missing integer scalar or double equivalent[)][.]$")
  expect_chk_error(chk_whole_number(1.1, x_name = 1L), "^1 must be a whole number [(]non-missing integer scalar or double equivalent[)][.]$")
})

test_that("chk_whole_number", {
  expect_identical(chk_whole_number(1), 1)
  expect_invisible(chk_whole_number(1))
  expect_chk_error(chk_whole_number(1.1), "^`1.1` must be a whole number [(]non-missing integer scalar or double equivalent[)][.]$")
  expect_chk_error(chk_whole_number(1.1, x_name = 1L), "^1 must be a whole number [(]non-missing integer scalar or double equivalent[)][.]$")
})

#' Check All Identical
#'
#' @description
#' Checks all elements in x identical using
#'
#' `length(x) < 2L || all(vapply(x, vld_identical, TRUE, y = x[[1]]))`
#'
#'
#' **Pass**: `c(1, 1, 1)`, `list(1, 1)`
#'
#' **Fail**: `c(1, 1.0000001)`, `list(1, NA)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family equal_checkers
#' @family all_checkers
#'
#' @seealso [length()]
#' @seealso [vld_identical()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_all_identical
#' chk_all_identical(c(1, 1))
#' try(chk_all_identical(c(1, 1.1)))
#' @export
chk_all_identical <- function(x, x_name = NULL) {
  if (vld_all_identical(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must have identical elements", x = x)
}

#' @describeIn chk_all_identical Validate All Identical
#'
#' @examples
#' # vld_all_identical
#' vld_all_identical(c(1, 1))
#' @export
vld_all_identical <- function(x) {
  length(x) < 2L || all(vapply(x, vld_identical, TRUE, y = x[[1]]))
}

#' Check Length
#'
#' @description
#' Checks length of an object.
#'
#' @inheritParams params
#' @param values A flag or a whole numeric vector of the value, value range or possible values.
#' @return An informative error if the test fails or an invisible copy of x.
#'
#' @family check
#'
#' @examples
#' check_length(1)
#' try(check_length(1, values = FALSE))
#' try(check_length(1, values = c(10, 2)))
#' @export
check_length <- function(x, values = numeric(0), x_name = NULL) {
  if (is.null(x_name)) x_name <- deparse((substitute(x)))
  check_dim(x, values = values, x_name = x_name, dim_name = "length")
}

test_that("check_length length", {
  expect_identical(check_length(1), 1)
  expect_invisible(check_length(1))

  expect_identical(check_length(integer(0)), integer(0))
  expect_chk_error(
    check_length(integer(0), values = TRUE),
    "^`length[(]integer[(]0[)][)]` must be greater than 0, not 0[.]$"
  )
  expect_chk_error(
    check_length(1, values = FALSE),
    "^`length[(]1[)]` must be equal to 0L[.]"
  )
  expect_identical(check_length(2, values = 1), check_length(2, values = 1))
  expect_chk_error(
    check_length(1, values = 2),
    "^`length[(]1[)]` must be equal to 2[.]"
  )
  expect_chk_error(
    check_length(1, values = 2:3),
    "^`length[(]1[)]` must be between 2 and 3, not 1[.]"
  )
  expect_chk_error(
    check_length(1, values = 3:2),
    "^`length[(]1[)]` must be between 2 and 3, not 1[.]"
  )
  expect_identical(check_length(1:2, values = 2:3), check_length(1:2, values = 2:3))
  expect_chk_error(
    check_length(2, values = c(2, 2)),
    "`length[(]2[)]` must be equal to 2[.]"
  )
  expect_chk_error(
    check_length(2, values = c(2, 2, 2)),
    "`length[(]2[)]` must match 2, not 1[.]"
  )
  expect_chk_error(
    check_length(2, values = c(3, 2, 2)),
    "`length[(]2[)]` must match 2 or 3, not 1[.]"
  )

  expect_chk_error(
    check_length(1, values = 1.1), "^At least one of the following conditions must be met:\n[*] `values` must be a flag [(]TRUE or FALSE[)].\n[*] `values` must be a whole numeric vector \\(integer vector or double equivalent\\)\\.$"
  )
})

try_chk <- function(quo) {
  catch_cnd(eval_tidy(quo), classes = "error")
}

try_msg <- function(x) {
  # FIXME: Tests fail when the code below is changed to conditionMessage(x)
  # because rlang implements conditionMessage.rlang_error() .
  # Does that output look satisfactory?
  conditionMessage.condition(x)
}

chkor_quos <- function(quos) {
  n <- length(quos)
  if (n == 0L) {
    return(invisible())
  }

  msg <- character(n)
  for (i in seq_len(n)) {
    try <- try_chk(quos[[i]])
    if (is.null(try)) {
      return(invisible())
    }
    msg[[i]] <- try_msg(try)
  }
  msg <- unique(msg)
  if (length(msg) > 1) {
    msg <- p0(msg, collapse = "\n* ")
    msg <- p0("At least one of the following conditions must be met:\n* ", msg)
  }
  abort_chk(msg)
}

vld__to_chk_ <- function(quo) {
  str <- deparse(quo)
  str <- sub("^vld_", "chk_", str)
  str2lang(str)
}

#' Chk OR
#'
#' @param ... Multiple `vld_` calls.
#'
#' A common mistake is to pass `chk_` calls.
#'
#' `chkor_vld()` is relatively slow.
#' If at all possible use [chk_null_or()] or first test using the individual
#' `vld_` functions and then call `chkor_vld()` to generate an informative
#' error message.
#'
#' @return An informative error if the test fails.
#'
#' @seealso [chk_null_or()]
#'
#' @export
#'
#' @examples
#' chkor_vld()
#' chkor_vld(vld_flag(TRUE))
#' try(chkor_vld(vld_flag(1)))
#' try(chkor_vld(vld_flag(1), vld_flag(2)))
#' chkor_vld(vld_flag(1), vld_flag(TRUE))
chkor_vld <- function(...) {
  quos <- enquos(...)

  n <- length(quos)
  if (n == 0L) {
    return(invisible())
  }
  for (quo in quos) {
    if (eval_tidy(quo)) {
      return(invisible())
    }
  }
  for (i in seq_len(n)) {
    quos[[i]] <- quo_set_expr(quos[[i]], vld__to_chk_(quo_get_expr(quos[[i]])))
  }
  chkor_quos(quos)
}

#' Check Double Scalar
#'
#' @description
#' Checks if double scalar using
#'
#' `is.double(x) && length(x) == 1L`
#'
#' `r lifecycle::badge("deprecated")`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family deprecated
#'
#' @examples
#' # chk_dbl
#' chk_dbl(1)
#' try(chk_dbl(1L))
#' @export
chk_dbl <- function(x, x_name = NULL) {
  if (vld_dbl(x)) {
    return(invisible(x))
  }
  deprecate_soft(
    "0.6.1",
    what = "chk::chk_dbl()",
    details = "Please use `chk::chk_scalar(x);` `chk::chk_double(x)` instead",
    id = "chk_dbl"
  )

  if (is.null(x_name)) x_name <- deparse_backtick_chk((substitute(x)))
  abort_chk(x_name, " must be double (real) scalar", x = x)
}

#' @describeIn chk_dbl Validate Double
#'
#' `r lifecycle::badge("deprecated")`
#'
#' @examples
#' # vld_dbl
#' vld_dbl(1)
#' vld_dbl(double(0))
#' vld_dbl(NA_real_)
#' vld_dbl(c(1, 1))
#' vld_dbl(1L)
#' @export
vld_dbl <- function(x) {
  deprecate_soft(
    "0.6.1",
    what = "chk::chk_dbl()",
    details = "Please use `chk::chk_scalar(x);` `chk::chk_double(x)` instead",
    id = "chk_dbl"
  )

  is.double(x) && length(x) == 1L
}

test_that("vld_dbl", {
  rlang::local_options(lifecycle_verbosity = "quiet")

  lifecycle::expect_deprecated(vld_dbl(1))

  expect_true(vld_dbl(1))
  expect_false(vld_dbl(matrix(c(1, 2))))
  expect_true(vld_dbl(dbl(0)))
  expect_true(vld_dbl(NA_real_))
  expect_false(vld_dbl(1L))
  expect_false(vld_dbl(TRUE))
})

test_that("chk_dbl", {
  rlang::local_options(lifecycle_verbosity = "quiet")

  lifecycle::expect_deprecated(chk_dbl(1))

  expect_identical(chk_dbl(1), 1)
  expect_invisible(chk_dbl(1))
  expect_chk_error(
    chk_dbl(TRUE),
    "^`TRUE` must be double [(]real[)] scalar[.]$"
  )
})

#' Check Date
#'
#' @description
#' Checks non-missing Date scalar using
#'
#' `inherits(x, "Date") && length(x) == 1L && !anyNA(x)`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family scalar_checkers
#' @family datetime_checkers
#'
#' @seealso [inherits()]
#' @seealso [length()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_date
#' chk_date(Sys.Date())
#' try(chk_date(1))
#' @export
chk_date <- function(x, x_name = NULL) {
  if (vld_date(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be a date (non-missing Date scalar)", x = x)
}

#' @describeIn chk_date Validate Date
#'
#' @examples
#' # vld_date
#' vld_date(Sys.Date())
#' vld_date(Sys.time())
#' vld_date(1)
#' @export
vld_date <- function(x) inherits(x, "Date") && length(x) == 1L && !anyNA(x)

test_that("vld_date", {
  expect_false(vld_date(NULL))
  expect_false(vld_date(as.Date("2000-01-01")[-1]))
  expect_false(vld_date(as.Date("2000-01-01")[c(1, 1)]))
  expect_true(vld_date(as.Date("2000-01-01")))
})

test_that("chk_date", {
  expect_identical(chk_date(as.Date("2000-01-01")), as.Date("2000-01-01"))
  expect_invisible(chk_date(as.Date("2000-01-01")))

  expect_chk_error(chk_date(1), "^`1` must be a date [(]non-missing Date scalar[)][.]$")
  expect_chk_error(chk_date(1, x_name = "2001-02-04"), "^2001-02-04 must be a date [(]non-missing Date scalar[)][.]$")
})

#' Check Data
#'
#' @description
#' Checks data.frame using
#'
#' `inherits(x, "data.frame")`
#'
#' Note that there is a similar function, [check_data()], which checks
#' the column names, values, number of rows, and keys of a data.frame.
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family id_checkers
#'
#' @seealso [inherits()]
#'
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_data
#' chk_data(data.frame(x = 1))
#' try(chk_data(1))
#' @export
chk_data <- function(x, x_name = NULL) {
  if (vld_data(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be a data.frame", x = x)
}

#' @describeIn chk_data Validate Data
#'
#' @examples
#' # vld_data
#' vld_data(data.frame())
#' vld_data(data.frame(x = 1))
#' vld_data(c(x = 1))
#' @export
vld_data <- function(x) inherits(x, "data.frame")

test_that("vld_data", {
  expect_false(vld_data(1))
  expect_false(vld_data(array(1)))
  expect_true(vld_data(data.frame()))
  expect_true(vld_data(data.frame(x = 1)))
})

test_that("chk_data", {
  expect_identical(chk_data(data.frame()), data.frame())
  expect_invisible(chk_data(data.frame()))
  expect_chk_error(
    chk_data(1),
    "^`1` must be a data[.]frame[.]$"
  )
})

#' Check Number
#'
#' @description
#' Checks if non-missing numeric scalar using
#'
#' `is.numeric(x) && length(x) == 1L && !anyNA(x)`
#'
#' **Pass**: `1`, `2L`, `log(10)`, `-Inf`
#'
#' **Fail**: `"a"`, `1:3`, `NA_real_`
#'
#' @inheritParams params
#' @inherit params return
#'
#' @family data_type_checkers scalar_checkers
#'
#' @seealso [is.numeric()]
#' @seealso [length()]
#' @seealso For more details about the use of this function,
#' please read the article
#' `vignette("chk-families")`.
#'
#' @examples
#' # chk_number
#' chk_number(1.1)
#' try(chk_number(TRUE))
#' @export
chk_number <- function(x, x_name = NULL) {
  if (vld_number(x)) {
    return(invisible(x))
  }
  if (is.null(x_name)) x_name <- deparse_backtick_chk(substitute(x))
  abort_chk(x_name, " must be a number (non-missing numeric scalar)", x = x)
}

#' @describeIn chk_number Validate Number
#'
#' @examples
#' # vld_number
#' vld_number(1.1)
#' @export
vld_number <- function(x) {
  is.numeric(x) && length(x) == 1L && !anyNA(x)
}

test_that("vld_number", {
  expect_false(vld_number(numeric(0)))
  expect_false(vld_number(NA_integer_))
  expect_true(vld_number(1))
  expect_true(vld_number(1L))
  expect_true(vld_number(1.1))
  expect_true(vld_number(-1.1))
  expect_true(vld_number(Inf))
  expect_false(vld_number(c(1, 2)))
  expect_false(vld_number(TRUE))
})

test_that("chk_number", {
  expect_identical(chk_number(1), 1)
  expect_invisible(chk_number(1))
  expect_chk_error(chk_number(TRUE), "^`TRUE` must be a number [(]non-missing numeric scalar[)][.]$")
  expect_chk_error(chk_number(TRUE, x_name = 1L), "^1 must be a number [(]non-missing numeric scalar[)][.]$")
})

test_that("check_dirs works", {
  expect_identical(check_dirs(character(0)), character(0))
  expect_invisible(check_dirs(character(0)))
  expect_identical(
    check_dirs(character(0), exists = TRUE),
    check_dirs(character(0), exists = TRUE)
  )
  expect_invisible(check_dirs(character(0), exists = TRUE))

  tmp <- withr::local_tempfile()
  expect_identical(check_dirs(tmp, exists = FALSE), check_dirs(tmp, exists = FALSE))
  expect_invisible(check_dirs(tmp, exists = FALSE))
  tmp <- withr::local_tempdir()
  expect_identical(check_dirs(tmp, exists = TRUE), check_dirs(tmp, exists = TRUE))
  expect_invisible(check_dirs(tmp, exists = TRUE))
})

test_that("check_dirs errors", {
  expect_chk_error(check_dirs(NA_character_))

  tmp <- withr::local_tempfile()
  writeLines(tmp, text = "some test data")

  expect_chk_error(
    check_dirs(tmp),
    "^`tmp` must specify directories [(]'.*' is a file[)][.]$"
  )
  expect_chk_error(
    check_dirs(tmp, exists = FALSE),
    "^`tmp` must specify directories [(]'.*' is a file[)][.]$"
  )

  tmp <- withr::local_tempfile()
  expect_chk_error(
    check_dirs(tmp),
    "^`tmp` must specify existing directories [(]'.*' can't be found[)][.]$"
  )

  tmp <- withr::local_tempdir()
  expect_chk_error(
    check_dirs(tmp, exists = FALSE),
    "^`tmp` must not specify existing directories [(]'.*' exists[)][.]$"
  )
})

test_that("vld_factor", {
  expect_true(vld_factor(factor("1")))
  expect_true(vld_factor(factor(0)))
  expect_true(vld_factor(factor(NA)))
  expect_false(vld_factor(NA_character_))
  expect_false(vld_factor(1))
  expect_false(vld_factor(TRUE))
})

test_that("chk_factor", {
  expect_identical(chk_factor(factor("1")), factor("1"))
  expect_invisible(chk_factor(factor("1")))
  expect_chk_error(
    chk_factor(TRUE),
    "^`TRUE` must be factor[.]$"
  )
})

test_that("vld_subset", {
  expect_true(vld_subset(integer(0), 1))
  expect_false(vld_subset(1:3, 1:2))
  expect_true(vld_subset(numeric(0), 1L))

  expect_false(vld_subset(-1, c(0, 1, NA)))
  expect_false(vld_subset(1L, 0L))
  expect_false(vld_subset(1, NA))

  expect_false(vld_subset(c(NA, 1), 1))
  expect_true(vld_subset(NA, NA))
  expect_true(vld_subset(c(NA, 1), c(1, NA)))

  expect_false(vld_subset(c(NA, 1), 1:2))
})

test_that("chk_subset", {
  expect_identical(chk_subset(integer(0), 1), integer(0), 1)
  expect_invisible(chk_subset(integer(0), 1))
  expect_chk_error(chk_subset(NA, c(0, Inf)), "^`NA` must match 0 or Inf, not NA[.]$")
  expect_chk_error(chk_subset(1:3, c(0, 1, NA)), "`1:3` must have values matching 0, 1 or NA[.]$")
  expect_chk_error(chk_subset(-1, c(0, 1, NA)), "^`-1` must match 0, 1 or NA, not -1[.]$")
  expect_chk_error(chk_subset(c(1L, 1L), 0L), c("^`c[(]1L, 1L[)]` must have values matching 0[.]$"))
  expect_chk_error(chk_subset(1, ""), "^`1` must match '', not 1[.]$")
  expect_chk_error(
    chk_subset(1, NA),
    "^`1` must match NA, not 1[.]$"
  )
  expect_chk_error(
    chk_subset(c(NA, 1), 1),
    "^`c[(]NA, 1[)]` must have values matching 1[.]$"
  )
  expect_chk_error(
    chk_subset(c(NA, 1), 1:2),
    "^`c[(]NA, 1[)]` must have values matching 1 or 2[.]$"
  )
  expect_chk_error(
    chk_subset(3L, values = c(1L, 1L, 7L)),
    "^`3L` must match 1 or 7, not 3[.]$"
  )
  expect_chk_error(
    chk_subset(3L, values = c(1L, 1L, 7L), x_name = "1"),
    "^1 must match 1 or 7, not 3[.]$"
  )
})

test_that("cc with atomic vectors", {
  expect_identical(cc(logical(0)), character(0))
  expect_match(cc(1), "^1$")
  expect_match(cc(1, brac = "'"), "^'1'$")
  expect_match(cc(c(1, 1), " or "), "^1 or 1$")
  expect_match(cc(1:2, " or "), "^1 or 2$")
  expect_match(cc(1:2), "^1, 2$")
  expect_match(cc(1:3, " or "), "^1, 2 or 3$")
  expect_match(cc(1:10, " or "), "^1, 2, 3, 4, 5, 6, 7, 8, 9 or 10$")
  expect_match(cc(1:11, " or "), "^1, 2, 3, 4, 5, 6, 7, 8, ... or 11$")
  expect_match(cc(11:1, " or "), "^11, 10, 9, 8, 7, 6, 5, 4, ... or 1$")
  expect_match(
    cc(11:1, brac = "'"),
    "^'11', '10', '9', '8', '7', '6', '5', '4', ..., '1'$"
  )
})

test_that("cc errors", {
  expect_chk_error(
    cc(1, conj = 1),
    "^`conj` must be a string [(]non-missing character scalar[)][.]$"
  )
  expect_chk_error(
    cc(1, conj = character(0)),
    "^`conj` must be a string [(]non-missing character scalar[)][.]$"
  )
  expect_chk_error(
    cc(1, conj = as.character(1:2)),
    "^`conj` must be a string [(]non-missing character scalar[)][.]$"
  )
  expect_chk_error(
    cc(1, brac = 1),
    "^`brac` must inherit from S3 class 'character'[.]$"
  )
  expect_chk_error(
    cc(1, brac = character(0)),
    "^`length[(]brac[)]` must be between 1 and 2, not 0[.]$"
  )
  expect_chk_error(
    cc(1, brac = as.character(1:3)),
    "^`length[(]brac[)]` must be between 1 and 2, not 3[.]$"
  )
  expect_chk_error(
    cc(1, sep = as.character(1:3)),
    "^`sep` must be a string [(]non-missing character scalar[)][.]$"
  )
})

test_that("cc with character", {
  expect_identical(cc(character(0)), character(0))
  expect_match(cc(as.character("")), "^''$")
  expect_match(cc(as.character("a")), "^'a'$")
})

test_that("cc with factor", {
  expect_identical(cc(factor(1)[-1]), character(0))
  expect_match(cc(as.factor("")), "^''$")
  expect_match(cc(as.factor("a")), "^'a'$")
})

test_that("cc with double brackets", {
  expect_match(cc(1:2, brac = c("{", "}")), "^\\{1\\}, \\{2\\}$")
  expect_match(cc(1:2, brac = c("", "}{")), "^1\\}\\{, 2\\}\\{$")
})

test_that("cc with random objects", {
  expect_identical(cc(list()), character(0))
  expect_match(cc(list(1)), "^1$")
  expect_match(cc(list(x = 1)), "^1$")
  expect_match(cc(matrix(1)), "^1$")
  expect_match(cc(matrix(1:9, 3), " or "), "^1, 2, 3, 4, 5, 6, 7, 8 or 9$")
})

test_that("cc with sep", {
  expect_identical(cc(1:3, sep = "|"), "1|2, 3")
})

test_that("cc with ellipsis", {
  expect_identical(cc(1:10), c("1, 2, 3, 4, 5, 6, 7, 8, 9, 10"))
  expect_identical(cc(1:10, ellipsis = 3), c("1, ..., 10"))
  expect_chk_error(
    cc(1:10, ellipsis = 2),
    "`ellipsis` must be greater than or equal to 3, not 2[.]"
  )
  expect_chk_error(
    cc(1:10, ellipsis = 2.5),
    "`ellipsis` must be a whole number [(]non-missing integer scalar or double equivalent[)][.]"
  )
})

test_that("test if incomparables implemented in anyDuplicated.data.frame", {
  expect_error(
    anyDuplicated(data.frame(), incomparables = NA),
    "^argument 'incomparables != FALSE' is not used [(]yet[)]$"
  )
})

---
title: "chk Families"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{chk Families}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(chk)
```

## Introduction
The `vld_` functions are used within the `chk_` functions.
The `chk_` functions (and their `vld_` equivalents) can be divided into the following families.

In the code in this examples, we will use `vld_*` functions

If you want to learn more about the logic behind some of the functions explained here, we recommend reading the book [Advanced R](https://adv-r.hadley.nz/) (Wickham, 2019).

For reasons of space, the `x_name = NULL` argument is not shown.
For a more simplified list of the `chk` functions, you can see the [Reference](https://poissonconsulting.github.io/chk/reference/index.html) section.

## `chk_` Functions
### Overview
  
```{r chk_, echo = FALSE, out.width= "100%", fig.align='center', fig.alt = "Classification of the chk functions by family"}
knitr::include_graphics("chk_diagram_II.png")
```

### Missing Input Checker
Check if the function input is missing or not

`chk_missing` function uses `missing()` 
to check if an argument has been left out when the function is called.

Function |  Code
:- | :---
`chk_missing()` | `missing()`
`chk_not_missing()` | `!missing()`

### `...` Checker
Check if the function input comes from `...` (`dot-dot-dot`) or not

The functions `chk_used(...)` and `chk_unused(...)` check 
if any arguments have been provided through `...` (called `dot-dot-dot` or  ellipsis), which is commonly used in R to allow a variable number of arguments.

Function | Code
:- | :---
`chk_used(...)` | `length(list(...)) != 0L`
`chk_unused(...)` | `length(list(...)) == 0L`

### External Data Source Checkers
Check if the function input is a valid external data source.

These `chk` functions check the existence of a file, the validity of its extension, and the existence of a directory.

Function |  Code
:- | :---
`chk_file(x)` | `vld_string(x) && file.exists(x) && !dir.exists(x)`
`chk_ext(x, ext)` | `vld_string(x) && vld_subset(tools::file_ext(x), ext)`
`chk_dir(x)` | `vld_string(x) && dir.exists(x)`

### NULL checker
Check if the function input is NULL or not

Function | Code
:- | :---
`chk_null(x)` | `is.null(x)`
`chk_not_null(x)` | `!is.null(x)`

### Scalar Checkers
Check if the function input is a scalar.
In R, scalars are vectors of length 1. 

Function | Code
:- | :------
`chk_scalar(x)` | `length(x) == 1L`

The following functions check if the functions inputs are vectors of length 1 of a particular data type.
Each data type has a special syntax to create an individual value or "scalar".

Function | Code
:- | :------
`chk_string(x)` | `is.character(x) && length(x) == 1L && !anyNA(x)`
`chk_number(x)` | `is.numeric(x) && length(x) == 1L && !anyNA(x)`

For logical data types, you can check flags using `chk_flag()`, which considers `TRUE` or `FALSE` as possible values, or use `chk_lgl()` to verify if a scalar is of type logical, including NA as element.

Function | Code
:- | :-
`chk_flag(x)` | `is.logical(x) && length(x) == 1L && !anyNA(x)`
`chk_lgl(x)` | `is.logical(x) && length(x) == 1L`

It is also possible to check if the user-provided argument is only `TRUE` or only `FALSE`:

Function | Code
:- | :-
`chk_true(x)` | `is.logical(x) && length(x) == 1L && !anyNA(x) && x`
`chk_false(x)` | `is.logical(x) && length(x) == 1L && !anyNA(x) && !x`

### Date or DateTime Checkers
Check if the function input is of class Date or DateTime

Date and datetime classes can be checked with `chk_date` and `chk_datetime`.

Function | Code
:- | :------
`chk_date(x)` | `inherits(x, "Date") && length(x) == 1L && !anyNA(x)`
`chk_date_time(x)` | `inherits(x, "POSIXct") && length(x) == 1L && !anyNA(x)`

### Time Zone Checker
Also you can check the time zone with `chk_tz()`.
The available time zones can be retrieved using the function `OlsonNames()`.

Function | Code
:- | :------
`chk_tz(x)` | `is.character(x) && length(x) == 1L && !anyNA(x) && x %in% OlsonNames()`

#### Data Structure Checker
Check if the function input has a specific data structure.

Vectors are a family of data types that come in two forms: atomic vectors and lists. 
When vectors consist of elements of the same data type, they can be considered atomic, matrices, or arrays.
The elements in a list, however, can be of different types.

To check if a function argument is a vector you can use `chk_vector()`.
 
Function | Code
:- | :---
`chk_vector(x)` | `is.atomic(x) && !is.matrix(x) && !is.array(x)) || is.list(x)`

Pay attention that `chk_vector()` and `vld_vector()` 
are different from `is.vector()`, that will return FALSE if the vector has any attributes except names.

```{r}
vector <- c(1, 2, 3)
is.vector(vector) # TRUE
vld_vector(vector) # TRUE

attributes(vector) <-  list("a" = 10, "b" = 20, "c" = 30)
is.vector(vector) # FALSE
vld_vector(vector) # TRUE
```
Function | Code
:- | :---
`chk_atomic(x)` | `is.atomic(x)`

Notice that `is.atomic` is true for the types logical, integer, numeric, complex, character and raw. Also, it is TRUE for NULL.

```{r}
vector <- c(1, 2, 3)
is.atomic(vector) # TRUE
vld_vector(vector) # TRUE

is.atomic(NULL) # TRUE
vld_vector(NULL) # TRUE
```

The dimension attribute converts vectors into matrices and arrays.

Function | Code
:- | :---
`chk_array(x)` | `is.array(x)`
`chk_matrix(x)` | `is.matrix(x)`

When a vector is composed by heterogeneous data types, can be a list.
Data frames are among the most important S3 vectors, constructed on top of lists.

Function | Code
:- | :---
`chk_list(x)` | `is.list()`
`chk_data(x)` | `inherits(x, "data.frame")`

Be careful not to confuse the function `chk_data` with `check_data`.
Please read the `check_` functions section below and the function documentation.

### Data Type Checkers
Check if the function input has a data type.
You can use the function `typeof()` to confirm the data type.

Function | Code
:- | :---
`chk_environment(x)` | `is.environment(x)`
`chk_logical(x)` | `is.logical(x)`
`chk_character(x)` | `is.character(x)`

For numbers there are four functions.
R differentiates between doubles (`chk_double()`) and integers (`chk_integer()`).
You can also use the generic function `chk_numeric()`, which will detect both.
The third type of number is complex (`chk_complex()`).

Function | Code
:- | :---
`chk_numeric(x)` | `is.numeric(x)`
`chk_double(x)` | `is.double(x)`
`chk_integer(x)` | `is.integer(x)`
`chk_complex(x)` | `is.complex(x)`

Consider that to explicitly create an integer in R, you need to use the suffix `L`.

```{r}
vld_numeric(33) # TRUE

vld_double(33) # TRUE
vld_integer(33) # FALSE

vld_integer(33L) # TRUE
```


### Whole Number Checkers
These functions accept whole numbers, whether they are explicitly integers or double types without fractional parts.

Function | Code
:- | :---
`chk_whole_numeric` | `is.integer(x) || (is.double(x) && vld_true(all.equal(x[!is.na(x)],  trunc(x[!is.na(x)]))))`
`chk_whole_number` |  `vld_number(x) && (is.integer(x) || vld_true(all.equal(x, trunc(x))))`
`chk_count` | `vld_whole_number(x) && x >= 0`

If you want to consider both 3.0 and 3L as integers, it is safer to use the function `chk_whole_numeric`.
Here, `x` is valid if it's an integer or a double that can be converted to an integer without changing its value.

```{r}
# Integer vector
vld_whole_numeric(c(1L, 2L, 3L)) # TRUE

# Double vector representing whole numbers
vld_whole_numeric(c(1.0, 2.0, 3.0)) # TRUE

# Double vector with fractional numbers
vld_whole_numeric(c(1.0, 2.2, 3.0)) # FALSE
```

The function `chk_whole_number` is similar to  `chk_whole_numeric`.
`chk_whole_number` checks if the number is of `length(x) == 1L`

```{r}
# Integer vector
vld_whole_numeric(c(1L, 2L, 3L)) # TRUE
vld_whole_number(c(1L, 2L, 3L)) # FALSE
vld_whole_number(c(1L)) # TRUE
```

`chk_count()` is a special case of `chk_whole_number`, differing in that it ensures values are non-negative whole numbers.

```{r}
# Positive integer
vld_count(1) #TRUE
# Zero
vld_count(0) # TRUE
# Negative number
vld_count(-1) # FALSE
# Non-whole number
vld_count(2.5) # FALSE
```

### Factor Checker
Check if the function input is a factor

Function | Code
:- | :------
`chk_factor` | `is.factor(x)`
`chk_character_or_factor` | `is.character(x) || is.factor(x)`

Factors can be specially confusing for users, because despite they are displayed as characters are built in top of integer vectors.

`chk` provides the function `chk_character_or_factor()` that allows detecting if the argument that the user is providing contains strings. 

```{r}
# Factor with specified levels

vector_fruits <- c("apple", "banana", "apple", "orange", "banana", "apple")

factor_fruits <- factor(c("apple", "banana", "apple", "orange", "banana", "apple"),
                levels = c("apple", "banana", "orange"))


is.factor(factor_fruits) # TRUE
vld_factor(factor_fruits) # TRUE

is.character(factor_fruits) # FALSE
vld_character(factor_fruits) # FALSE

vld_character_or_factor(factor_fruits) # TRUE

```


### All Elements Checkers
Check if the function input has a characteristic shared by all its elements.

If you want to apply any of the previously defined functions for `length(x) == 1L` to the elements of a vector, you can use `chk_all()`.

Function |  Code
:- | :---
`chk_all(x, chk_fun, ...)` | `all(vapply(x, chk_fun, TRUE, ...))`

```{r}
vld_all(c(TRUE, TRUE, FALSE), chk_lgl) # FALSE
```

### Function Checker
Check if the function input is another function

`formals` refers to the count of the number of formal arguments

Function | Code
:- | :------
`chk_function` | `is.function(x) && (is.null(formals) || length(formals(x)) == formals)`

```{r}
vld_function(function(x) x, formals = 1) # TRUE
vld_function(function(x, y) x + y, formals = 1) # FALSE
vld_function(function(x, y) x + y, formals = 2) # TRUE
```

### Name Checkers
Check if the function input has names and are valid `chk_named` function works with vectors, lists, data frames, and matrices that have named columns or rows.
Do not confuse with `check_names`.

`chk_valid_name` function specifically designed to check if the elements of a character vector are valid R names.
If you want to know what is considered a valid name, 
please refer to the documentation for the `make.names` function.

Function | Code
:- | :--
`chk_named(x)` |  `!is.null(names(x))`
`chk_valid_name(x)` | `identical(make.names(x[!is.na(x)]), as.character(x[!is.na(x)]))`

```{r}

vld_valid_name(c("name1", NA, "name_2", "validName"))  # TRUE
vld_valid_name(c(1, 2, 3))  # FALSE


vld_named(data.frame(a = 1:5, b = 6:10))  # TRUE
vld_named(list(a = 1, b = 2)) # TRUE
vld_named(c(a = 1, b = 2)) # TRUE 
vld_named(c(1, 2, 3)) # FALSE 

```

### Range Checkers
Check if the function input is part of a range of values.
The function input should be numeric.

Function  | Code
:- | :---
`chk_range(x, range = c(0, 1))` | `all(x[!is.na(x)] >= range[1] & x[!is.na(x)] <= range[2])`
`chk_lt(x, value = 0)` | `all(x[!is.na(x)] < value)`
`chk_lte(x, value = 0)` | `all(x[!is.na(x)] <= value)`
`chk_gt(x, value = 0)` | `all(x[!is.na(x)] > value)`
`chk_gte(x, value = 0)` | `all(x[!is.na(x)] >= value)`

### Equal Checkers
Check if the function input is equal or similar to a predefined object.

The functions `chk_identical()`, `chk_equal()`, and `chk_equivalent()` are used to compare two objects, but they differ in how strict the comparison is.

`chk_equal` and `chk_equivalent`checks if x and y are numerically equivalent within a specified tolerance, but `chk_equivalent` ignores differences in attributes.

Function | Code
:-- | :-
`chk_identical(x, y)` | `identical(x, y)`
`chk_equal(x, y, tolerance = sqrt(.Machine$double.eps))` | `vld_true(all.equal(x, y, tolerance))`
`chk_equivalent(x, y, tolerance = sqrt(.Machine$double.eps))` | `vld_true(all.equal(x, y, tolerance, check.attributes = FALSE))`

In the case you want to compare the elements of a vector, you can use the `check_all_*` functions.

Function | Code
:-- | :--
`chk_all_identical(x)` | `length(x) < 2L || all(vapply(x, vld_identical, TRUE, y = x[[1]]))`
`chk_all_equal(x, tolerance = sqrt(.Machine$double.eps))` | `length(x) < 2L || all(vapply(x, vld_equal, TRUE, y = x[[1]], tolerance = tolerance))`
`chk_all_equivalent(x, tolerance = sqrt(.Machine$double.eps))` | `length(x) < 2L || all(vapply(x, vld_equivalent, TRUE, y = x[[1]], tolerance = tolerance))`

```{r}
vld_all_identical(c(1, 2, 3)) # FALSE
vld_all_identical(c(1, 1, 1)) # TRUE
vld_identical(c(1, 2, 3), c(1, 2, 3)) # TRUE

vld_all_equal(c(0.1, 0.12, 0.13))
vld_all_equal(c(0.1, 0.12, 0.13), tolerance = 0.2)
vld_equal(c(0.1, 0.12, 0.13), c(0.1, 0.12, 0.13)) # TRUE
vld_equal(c(0.1, 0.12, 0.13), c(0.1, 0.12, 0.4), tolerance = 0.5) # TRUE

x <- c(0.1, 0.1, 0.1)
y <- c(0.1, 0.12, 0.13)
attr(y, "label") <- "Numbers"
vld_equal(x, y, tolerance = 0.5) # FALSE
vld_equivalent(x, y, tolerance = 0.5) # TRUE
```

### Order Checker
Check if the function input are numbers in increasing order
`chk_sorted` function checks if `x` is sorted in non-decreasing order, ignoring any NA values.

Function | Code
:- | :--
`chk_sorted(x)` | `!is.unsorted(x, na.rm = TRUE)`

```{r}
# Checking if sorted
vld_sorted(c(1, 2, 3, NA, 4))  # TRUE
vld_sorted(c(3, 1, 2, NA, 4))  # FALSE
```

### Set Checkers
Check if the function input is composed by certain elements 

The `setequal` function in R is used to check if two vectors contain exactly the same elements, regardless of the order or number of repetitions.

Function | Code
:- | :---
`chk_setequal(x, values)` | `setequal(x, values)`

```{r}
vld_setequal(c(1, 2, 3), c(3, 2, 1)) # TRUE
vld_setequal(c(1, 2, 3), c(3, 2, 1, 4)) # FALSE
vld_setequal(c(1, 2, 3, 4), c(3, 2, 1)) # FALSE
vld_setequal(c(1, 2), c(1, 1, 1, 1, 1, 1, 2, 1)) # TRUE
```

First, the `%in%` function is used to check whether 
the elements of a vector `x` are present in a specified set of values.
This returns a logical vector, which is then simplified by `all()`. The `all()` function checks if all values in the vector are TRUE.
If the result is TRUE, it indicates that for `vld_` and `chk_subset()`, all elements in the `x` vector are present in `values`.
Similarly, for `vld_` and `chk_superset()`, it indicates that all elements of `values` are present in `x`.

Function | Code
:-- | :--
`chk_subset(x, values)` | `all(x %in% values)`
`chk_not_subset(x, values)` | `!any(x %in% values) || !length(x)`
`chk_superset(x, values)` | `all(values %in% x)`

```{r}

# When both function inputs have the same elements,
# all functions return TRUE

vld_setequal(c(1, 2, 3), c(3, 2, 1)) # TRUE
vld_subset(c(1, 2, 3), c(3, 2, 1)) # TRUE
vld_superset(c(1, 2, 3), c(3, 2, 1)) # TRUE

vld_setequal(c(1, 2), c(1, 1, 1, 1, 1, 1, 2, 1)) # TRUE
vld_subset(c(1, 2), c(1, 1, 1, 1, 1, 1, 2, 1)) # TRUE
vld_superset(c(1, 2), c(1, 1, 1, 1, 1, 1, 2, 1)) # TRUE

# When there are elements present in one vector but not the other,
# `vld_setequal()` will return FALSE

vld_setequal(c(1, 2, 3), c(3, 2, 1, 4)) # FALSE
vld_setequal(c(1, 2, 3, 4), c(3, 2, 1)) # FALSE

# When some elements of the `x` input are not present in `values`,
# `vld_subset()` returns FALSE
vld_subset(c(1, 2, 3, 4), c(3, 2, 1)) # FALSE
vld_superset(c(1, 2, 3, 4), c(3, 2, 1)) # TRUE

# When some elements of the `values` input are not present in `x`,
# `vld_superset()` returns FALSE

vld_subset(c(1, 2, 3), c(3, 2, 1, 4)) # TRUE
vld_superset(c(1, 2, 3), c(3, 2, 1, 4)) # FALSE

# An empty set is considered a subset of any set, and any set is a superset of an empty set.
vld_subset(c(), c("apple", "banana"))  # TRUE
vld_superset(c("apple", "banana"), c())  # TRUE
```

`chk_orderset()` validate whether a given set of `values` in a vector x matches a specified set of allowed `values` (represented by `values`) while preserving the order of those values.

Function | Code
:-- | :--
`chk_orderset` | `vld_equivalent(unique(x[x %in% values]), values[values %in% x])`

```{r}
vld_orderset(c("A", "B", "C"),  c("A", "B", "C", "D")) # TRUE
vld_orderset(c("C", "B", "A"),  c("A", "B", "C", "D")) # FALSE
vld_orderset(c("A", "C"),  c("A", "B", "C", "D")) # TRUE
```


### Class Checkers
Check if the function input belongs to a class or type.

These functions check if `x` is an S3 or S4 object of the specified class.

Function | Code
:- | :---
`chk_s3_class(x, class)` | `!isS4(x) && inherits(x, class)`
`chk_s4_class(x, class)` | `isS4(x) && methods::is(x, class)`

`chk_is()` checks if x inherits from a specified class, regardless of whether it is an S3 or S4 object.

Function | Code
:- | :---
`chk_is(x, class)` | `inherits(x, class)`

### REGEX Checker
Check if the function input matches a regular expression (REGEX).

`chk_match(x, regexp = ".+")` checks if the regular expression pattern specified by `regexp` matches all the non-missing values in the vector `x`.
If `regexp` it is not specified by the user, `chk_match` checks whether all non-missing values in `x` contain at least one character (regexp = ".+")

Function | Code
:- | :--
`chk_match(x, regexp = ".+")` | `all(grepl(regexp, x[!is.na(x)]))`

### Quality Checkers (Miscellaneous)
Check if the function input meet some user defined quality criteria.

`chk_not_empty` function checks if the length of the object is not zero.
For a data frame or matrix, the length corresponds to the number of elements (not rows or columns),
while for a vector or list, it corresponds to the number of elements.

`chk_not_any_na` function checks if there are no NA values present in the entire object.

Function | Code
:- | :--
`chk_not_empty(x)` |  `length(x) != 0L`
`chk_not_any_na(x)` |  `!anyNA(x)`

```{r}
vld_not_empty(c()) # FALSE
vld_not_empty(list()) # FALSE
vld_not_empty(data.frame()) # FALSE
vld_not_empty(data.frame(a = 1:3, b = 4:6)) # TRUE


vld_not_any_na(data.frame(a = 1:3, b = 4:6)) # TRUE
vld_not_any_na(data.frame(a = c(1, NA, 3), b = c(4, 5, 6))) # FALSE
```
The `chk_unique()` function is designed to verify that there are no duplicates elements in a vector.

Function | Code
:- | :--
`chk_unique(x, incomparables = FALSE)` | `!anyDuplicated(x, incomparables = incomparables)`

```{r}
vld_unique(c(1, 2, 3, 4)) # TRUE
vld_unique(c(1, 2, 2, 4)) # FALSE
```

The function `chk_length` checks whether the length of `x` is within a specified range. 
It ensures that the length is at least equal to `length` and no more than `upper`.
It can be used with vectors, lists and data frames. 

Function | Code
:- | :--
`chk_length(x, length = 1L, upper = length)` | `length(x) >= length && length(x) <= upper`

```{r}
vld_length(c(1, 2, 3), length = 2, upper = 5)  # TRUE
vld_length(c("a", "b"), length = 3)  # FALSE

vld_length(list(a = 1, b = 2, c = 3), length = 2, upper = 4) # TRUE
vld_length(list(a = 1, b = 2, c = 3), length = 4) # FALSE

# 2 columns
vld_length(data.frame(x = 1:3, y = 4:6), length = 1, upper = 3)  # TRUE
vld_length(data.frame(x = 1:3, y = 4:6), length = 3)  # FALSE    

# length of NULL is 0
vld_length(NULL, length = 0) # TRUE
vld_length(NULL, length = 1) # FALSE
```


Another useful function is `chk_compatible_lenghts()`.
This function helps to check vectors could be 'strictly recycled'.

```{r}
a <- integer(0)
b <- numeric(0)
vld_compatible_lengths(a, b) # TRUE

a <- 1
b <- 2
vld_compatible_lengths(a, b) # TRUE

a <- 1:3
b <- 1:3
vld_compatible_lengths(a, b) # TRUE

b <- 1
vld_compatible_lengths(a, b) # TRUE

b <- 1:2
vld_compatible_lengths(a, b) # FALSE

b <- 1:6
vld_compatible_lengths(a, b) # FALSE
```

The `chk_join()` function is designed to validate whether the number of rows in the resulting data frame from merging two data frames (`x` and `y`) is equal to the number of rows in the first data frame (`x`). 
This is useful when you want to ensure that a join operation does not change the number of rows in your main data frame.

Function | Code
:- | :--
`chk_join(x, y, by)` | `identical(nrow(x), nrow(merge(x, unique(y[if (is.null(names(by))) by else names(by)]), by = by)))`

```{r}
x <- data.frame(id = c(1, 2, 3), value_x = c("A", "B", "C"))
y <- data.frame(id = c(1, 2, 3), value_y = c("D", "E", "F"))
vld_join(x, y, by = "id") # TRUE

# Perform a join that reduces the number of rows 
y <- data.frame(id = c(1, 2, 1), value_y = c("D", "E", "F"))
vld_join(x, y, by = "id") # FALSE
```


## `check_` functions

The `check_` functions combine several `chk_` functions internally.
Read the documentation for each function to learn more about its specific use.

Function | Description
:- | :--
`check_values(x, values)` | Checks values and S3 class of an atomic object. 
`check_key(x, key = character(0), na_distinct = FALSE)` | Checks if columns have unique rows.   
`check_data(x, values, exclusive, order, nrow, key)` | Checks column names, values, number of rows and key for a data.frame.
`check_dim(x, dim, values, dim_name)` | Checks dimension of an object.
`check_dirs(x, exists)` | Checks if all directories exist (or if exists = FALSE do not exist as directories or files).
`check_files(x, exists)` | Checks if all files exist (or if exists = FALSE do not exist as files or directories).
`check_names(x, names, exclusive, order)` | Checks the names of an object.

## References

Wickham, H. (2019). Advanced R, Second Edition (2nd ed.). Chapman and Hall/CRC.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expect-chk-error.R
\name{expect_chk_error}
\alias{expect_chk_error}
\title{Expect Chk Error}
\usage{
expect_chk_error(
  object,
  regexp = NULL,
  ...,
  info = NULL,
  label = NULL,
  class = NULL
)
}
\arguments{
\item{object}{Object to test.

Supports limited unquoting to make it easier to generate readable failures
within a function or for loop. See \link[testthat]{quasi_label} for more details.}

\item{regexp}{Regular expression to test against.
\itemize{
\item A character vector giving a regular expression that must match the
error message.
\item If \code{NULL}, the default, asserts that there should be an error,
but doesn't test for a specific value.
\item If \code{NA}, asserts that there should be no errors, but we now recommend
using \code{\link[testthat:expect_no_error]{expect_no_error()}} and friends instead.
}

Note that you should only use \code{message} with errors/warnings/messages
that you generate. Avoid tests that rely on the specific text generated by
another package since this can easily change. If you do need to test text
generated by another package, either protect the test with \code{skip_on_cran()}
or use \code{expect_snapshot()}.}

\item{...}{
  Arguments passed on to \code{\link[testthat:expect_match]{expect_match}}
  \describe{
    \item{\code{fixed}}{If \code{TRUE}, treats \code{regexp} as a string to be matched exactly
(not a regular expressions). Overrides \code{perl}.}
    \item{\code{perl}}{logical.  Should Perl-compatible regexps be used?}
  }}

\item{info}{Extra information to be included in the message. This argument
is soft-deprecated and should not be used in new code. Instead see
alternatives in \link[testthat]{quasi_label}.}

\item{label}{Used to customise failure messages. For expert use only.}

\item{class}{Must be NULL.}
}
\value{
If \code{regexp = NA}, the value of the first argument; otherwise
the captured condition.
}
\description{
\code{\link[=expect_chk_error]{expect_chk_error()}} checks that code throws an error
of class \code{"chk_error"} with a message that matches regexp.
See below for more details.
}
\section{Testing \code{message} vs \code{class}}{


When checking that code generates an error, it's important to check that the
error is the one you expect. There are two ways to do this. The first
way is the simplest: you just provide a \code{regexp} that match some fragment
of the error message. This is easy, but fragile, because the test will
fail if the error message changes (even if its the same error).

A more robust way is to test for the class of the error, if it has one.
You can learn more about custom conditions at
\url{https://adv-r.hadley.nz/conditions.html#custom-conditions}, but in
short, errors are S3 classes and you can generate a custom class and check
for it using \code{class} instead of \code{regexp}.

If you are using \code{expect_error()} to check that an error message is
formatted in such a way that it makes sense to a human, we recommend
using \code{\link[testthat:expect_snapshot]{expect_snapshot()}} instead.

}

\examples{
expect_chk_error(chk_true(FALSE))
try(expect_chk_error(chk_false(FALSE)))
}
\seealso{
\code{\link[testthat:expect_no_error]{expect_no_error()}}, \code{expect_no_warning()},
\code{expect_no_message()}, and \code{expect_no_condition()} to assert
that code runs without errors/warnings/messages/conditions.

Other expectations: 
\code{\link[testthat]{comparison-expectations}},
\code{\link[testthat]{equality-expectations}},
\code{\link[testthat]{expect_length}()},
\code{\link[testthat]{expect_match}()},
\code{\link[testthat]{expect_named}()},
\code{\link[testthat]{expect_null}()},
\code{\link[testthat]{expect_output}()},
\code{\link[testthat]{expect_reference}()},
\code{\link[testthat]{expect_silent}()},
\code{\link[testthat]{inheritance-expectations}},
\code{\link[testthat]{logical-expectations}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-chr.R
\name{chk_chr}
\alias{chk_chr}
\alias{vld_chr}
\title{Check Character Scalar}
\usage{
chk_chr(x, x_name = NULL)

vld_chr(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if character scalar using

\code{is.character(x) && length(x) == 1L}

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}
}
\section{Functions}{
\itemize{
\item \code{vld_chr()}: Validate Character Scalar

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

}}
\examples{
chk_chr("a")
try(chk_chr(1))
# vld_chr
vld_chr("")
vld_chr("a")
vld_chr(NA_character_)
vld_chr(c("a", "b"))
vld_chr(1)
}
\seealso{
Other deprecated: 
\code{\link{chk_dbl}()},
\code{\link{chk_deprecated}},
\code{\link{chk_wnum}()}
}
\concept{chk_character}
\concept{deprecated}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-whole-number.R
\name{chk_whole_number}
\alias{chk_whole_number}
\alias{vld_whole_number}
\title{Check Whole Number}
\usage{
chk_whole_number(x, x_name = NULL)

vld_whole_number(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if non-missing integer scalar or double equivalent using

\code{vld_number(x) && (is.integer(x) || vld_true(all.equal(x, trunc(x))))}

\strong{Pass}: \code{1}, \code{2L}, \code{1e10}, \code{-Inf}

\strong{Fail}: \code{"a"}, \code{1:3}, \code{NA_integer_}, \code{log(10)}
}
\section{Functions}{
\itemize{
\item \code{vld_whole_number()}: Validate Whole Number

}}
\examples{
# chk_whole_number
chk_whole_number(2)
try(chk_whole_number(1.1))
# vld_whole_number
vld_whole_number(2)
}
\seealso{
\code{\link[=is.integer]{is.integer()}}

\code{\link[=vld_true]{vld_true()}}

\code{\link[=vld_number]{vld_number()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other scalar_checker: 
\code{\link{chk_complex_number}()}

Other whole_number_checkers: 
\code{\link{chk_count}()},
\code{\link{chk_whole_numeric}()}
}
\concept{scalar_checker}
\concept{whole_number_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-scalar.R
\name{chk_scalar}
\alias{chk_scalar}
\alias{vld_scalar}
\title{Check Scalar}
\usage{
chk_scalar(x, x_name = NULL)

vld_scalar(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if is a vector using

\code{length(x) == 1L}
}
\section{Functions}{
\itemize{
\item \code{vld_scalar()}: Validate Scalar

}}
\examples{
# chk_scalar
chk_scalar(1)
chk_scalar(list(1))
try(chk_scalar(1:2))
# vld_scalar
vld_scalar(1)
}
\seealso{
\code{\link[=length]{length()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other scalar_checkers: 
\code{\link{chk_count}()},
\code{\link{chk_date}()},
\code{\link{chk_date_time}()},
\code{\link{chk_false}()},
\code{\link{chk_flag}()},
\code{\link{chk_lgl}()},
\code{\link{chk_string}()},
\code{\link{chk_true}()},
\code{\link{chk_tz}()}
}
\concept{scalar_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-range.R
\name{chk_range}
\alias{chk_range}
\alias{vld_range}
\title{Checks range of non-missing values}
\usage{
chk_range(x, range = c(0, 1), inclusive = TRUE, x_name = NULL)

vld_range(x, range = c(0, 1), inclusive = TRUE)
}
\arguments{
\item{x}{The object to check.}

\item{range}{A non-missing sorted vector of length 2 of the lower and
upper permitted values.}

\item{inclusive}{A flag specifying whether the range is exclusive.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks all non-missing values fall within range using

If inclusive

\code{all(x[!is.na(x)] >= range[1] & x[!is.na(x)] <= range[2])}

else

\code{all(x[!is.na(x)] > range[1] & x[!is.na(x)] < range[2])}
}
\section{Functions}{
\itemize{
\item \code{vld_range()}: Validate Range

}}
\examples{
# chk_range
chk_range(0)
try(chk_range(-0.1))
# vld_range
vld_range(numeric(0))
vld_range(0)
vld_range(-0.1)
vld_range(c(0.1, 0.2, NA))
vld_range(c(0.1, 0.2, NA), range = c(0, 1))
}
\seealso{
\code{\link[=all]{all()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other range_checkers: 
\code{\link{chk_gt}()},
\code{\link{chk_gte}()},
\code{\link{chk_lt}()},
\code{\link{chk_lte}()}
}
\concept{range_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-used.R
\name{chk_used}
\alias{chk_used}
\alias{vld_used}
\title{Check ... Used}
\usage{
chk_used(...)

vld_used(...)
}
\arguments{
\item{...}{Additional arguments.}
}
\value{
The \code{chk_} function throws an informative error if the test fails.
}
\description{
Checks if is ... used using

\code{length(list(...)) != 0L}
}
\section{Functions}{
\itemize{
\item \code{vld_used()}: Validate ... Used

}}
\examples{
# chk_used
fun <- function(x, ...) {
  chk_used(...)
  x
}
try(fun(1))
fun(1, 2)
# vld_used
fun <- function(x, ...) {
  vld_used(...)
}
fun(1)
fun(1, 2)
}
\seealso{
\code{\link[=length]{length()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other ellipsis_checkers: 
\code{\link{chk_unused}()}
}
\concept{ellipsis_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-unique.R
\name{chk_unique}
\alias{chk_unique}
\alias{vld_unique}
\title{Check Unique}
\usage{
chk_unique(x, incomparables = FALSE, x_name = NULL)

vld_unique(x, incomparables = FALSE)
}
\arguments{
\item{x}{The object to check.}

\item{incomparables}{A vector of values that cannot be compared.
FALSE means that all values can be compared.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if unique using

\code{!anyDuplicated(x, incomparables = incomparables)}
}
\section{Functions}{
\itemize{
\item \code{vld_unique()}: Validate Unique

}}
\examples{
# chk_unique
chk_unique(c(NA, 2))
try(chk_unique(c(NA, NA, 2)))
chk_unique(c(NA, NA, 2), incomparables = NA)
# vld_unique
vld_unique(NULL)
vld_unique(numeric(0))
vld_unique(c(NA, 2))
vld_unique(c(NA, NA, 2))
vld_unique(c(NA, NA, 2), incomparables = NA)
}
\seealso{
\code{\link[=anyDuplicated]{anyDuplicated()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other misc_checkers: 
\code{\link{chk_join}()},
\code{\link{chk_not_any_na}()},
\code{\link{chk_not_empty}()}
}
\concept{misc_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-match.R
\name{chk_match}
\alias{chk_match}
\alias{vld_match}
\title{Check Matches}
\usage{
chk_match(x, regexp = ".+", x_name = NULL)

vld_match(x, regexp = ".+")
}
\arguments{
\item{x}{The object to check.}

\item{regexp}{A string of a regular expression.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if all values match regular expression using

\code{all(grepl(regexp, x[!is.na(x)]))}
}
\section{Functions}{
\itemize{
\item \code{vld_match()}: Validate Matches

}}
\examples{
# chk_match
chk_match("1")
try(chk_match("1", regexp = "2"))
# vld_match
vld_match("1")
vld_match("a", regexp = "a")
vld_match("")
vld_match("1", regexp = "2")
vld_match(NA_character_, regexp = ".*")
}
\seealso{
\code{\link[=all]{all()}}

\code{\link[=grepl]{grepl()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.
}
\concept{regex_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-array.R
\name{chk_array}
\alias{chk_array}
\alias{vld_array}
\title{Check Array}
\usage{
chk_array(x, x_name = NULL)

vld_array(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if is an array using

\code{is.array(x)}
}
\section{Functions}{
\itemize{
\item \code{vld_array()}: Validate Array

}}
\examples{
# chk_array
chk_array(array(1))
try(chk_array(matrix(1)))

# vld_array
vld_array(1)
vld_array(array(1))
}
\seealso{
\code{\link[=is.array]{is.array()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other data_structure_checkers: 
\code{\link{chk_atomic}()},
\code{\link{chk_list}()},
\code{\link{chk_matrix}()},
\code{\link{chk_vector}()}
}
\concept{data_structure_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check-names.R
\name{check_names}
\alias{check_names}
\title{Check Names}
\usage{
check_names(
  x,
  names = character(0),
  exclusive = FALSE,
  order = FALSE,
  x_name = NULL
)
}
\arguments{
\item{x}{The object to check.}

\item{names}{A character vector of the required names.}

\item{exclusive}{A flag specifying whether x must only contain the required names.}

\item{order}{A flag specifying whether the order of the required names in x must match the order in names.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
An informative error if the test fails or an invisible copy of x.
}
\description{
Checks the names of an object.
}
\examples{
x <- c(x = 1, y = 2)
check_names(x, c("y", "x"))
try(check_names(x, c("y", "x"), order = TRUE))
try(check_names(x, "x", exclusive = TRUE))
}
\seealso{
Other check: 
\code{\link{check_data}()},
\code{\link{check_dim}()},
\code{\link{check_dirs}()},
\code{\link{check_files}()},
\code{\link{check_key}()},
\code{\link{check_length}()},
\code{\link{check_values}()}
}
\concept{check}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-orderset.R, R/chk-setequal.R
\name{vld_orderset}
\alias{vld_orderset}
\alias{chk_setequal}
\alias{vld_setequal}
\title{Check Set Equal}
\usage{
vld_orderset(x, values)

chk_setequal(x, values, x_name = NULL)

vld_setequal(x, values)
}
\arguments{
\item{x}{The object to check.}

\item{values}{A vector of the permitted values.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if equal set using

\code{setequal(x, values)}
}
\section{Functions}{
\itemize{
\item \code{vld_orderset()}: Validate Set Ordered

\item \code{vld_setequal()}: Validate Set Equal

}}
\examples{

# vld_orderset
vld_orderset(1, 1)
vld_orderset(1:2, 2:1)
vld_orderset(1, 2:1)
vld_orderset(1:2, 2)
# chk_setequal
chk_setequal(1:2, 2:1)
try(chk_setequal(1, 1:2))
# vld_setequal
vld_setequal(1, 1)
vld_setequal(1:2, 2:1)
vld_setequal(1, 2:1)
vld_setequal(1:2, 2)
}
\seealso{
\code{\link[=setequal]{setequal()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other set_checkers: 
\code{\link{chk_not_subset}()},
\code{\link{chk_orderset}()},
\code{\link{chk_superset}()},
\code{\link{vld_not_subset}()}
}
\concept{set_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check-values.R
\name{check_values}
\alias{check_values}
\title{Check Values and Class}
\usage{
check_values(x, values, x_name = NULL)
}
\arguments{
\item{x}{The object to check.}

\item{values}{An atomic vector specifying the S3 class and possible values.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
An informative error if the test fails or an invisible copy of x.
}
\description{
Checks values and S3 class of an atomic object.
}
\details{
To check the class simply pass a vector of the desired class.

To check that x does not include missing values pass
a single non-missing value (of the correct class).

To allow it to include missing values include a missing value.

To check that it only includes missing values only pass
a missing value (of the correct class).

To check the range of the values in x pass two non-missing values
(as well as the missing value if required).

To check that x only includes specific values
pass three or more non-missing values.

In the case of a factor ensure values has two levels to
check that the levels of x are an ordered superset of the levels of value
and three or more levels to check that they are identical.
}
\examples{
check_values(1, numeric(0))
check_values(1, 2)
try(check_values(1, 1L))
try(check_values(NA_real_, 1))
}
\seealso{
Other check: 
\code{\link{check_data}()},
\code{\link{check_dim}()},
\code{\link{check_dirs}()},
\code{\link{check_files}()},
\code{\link{check_key}()},
\code{\link{check_length}()},
\code{\link{check_names}()}
}
\concept{check}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-identical.R
\name{chk_identical}
\alias{chk_identical}
\alias{vld_identical}
\title{Check Identical}
\usage{
chk_identical(x, y, x_name = NULL)

vld_identical(x, y)
}
\arguments{
\item{x}{The object to check.}

\item{y}{An object to check against.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if is identical to y using

\code{identical(x, y)}
}
\section{Functions}{
\itemize{
\item \code{vld_identical()}: Validate Identical

}}
\examples{
# chk_identical
chk_identical(1, 1)
try(chk_identical(1, 1L))
chk_identical(c(1, 1), c(1, 1))
try(chk_identical(1, c(1, 1)))
vld_identical(1, 1)
}
\seealso{
\code{\link[=identical]{identical()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other equal_checkers: 
\code{\link{chk_all_equal}()},
\code{\link{chk_all_equivalent}()},
\code{\link{chk_all_identical}()},
\code{\link{chk_equal}()},
\code{\link{chk_equivalent}()}
}
\concept{equal_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-data.R
\name{chk_data}
\alias{chk_data}
\alias{vld_data}
\title{Check Data}
\usage{
chk_data(x, x_name = NULL)

vld_data(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks data.frame using

\code{inherits(x, "data.frame")}

Note that there is a similar function, \code{\link[=check_data]{check_data()}}, which checks
the column names, values, number of rows, and keys of a data.frame.
}
\section{Functions}{
\itemize{
\item \code{vld_data()}: Validate Data

}}
\examples{
# chk_data
chk_data(data.frame(x = 1))
try(chk_data(1))
# vld_data
vld_data(data.frame())
vld_data(data.frame(x = 1))
vld_data(c(x = 1))
}
\seealso{
\code{\link[=inherits]{inherits()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other id_checkers: 
\code{\link{chk_is}()},
\code{\link{chk_s3_class}()},
\code{\link{chk_s4_class}()}
}
\concept{id_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/err.R
\name{message_chk}
\alias{message_chk}
\title{Construct Tidyverse Style Message}
\usage{
message_chk(..., n = NULL, tidy = TRUE)
}
\arguments{
\item{...}{Multiple objects that are converted to a string using
\code{paste0(..., collapse = '')}.}

\item{n}{The value of n for converting \code{sprintf}-like types.}

\item{tidy}{A flag specifying whether capitalize the first character and add a missing period.}
}
\value{
A string of the message.
}
\description{
If \code{tidy = TRUE} constructs a tidyverse style message by
}
\details{
\itemize{
\item Capitalizing the first character if possible.
\item Adding a trailing . if missing.
}

Also if \code{n != NULL} replaces the recognized \code{sprintf}-like types.
}
\section{\code{sprintf}-like types}{


The following recognized \code{sprintf}-like types can be used in a message:

\describe{
\item{\code{n}}{The value of n.}
\item{\code{s}}{'' if n == 1 otherwise 's'}
\item{\code{r}}{'is' if n == 1 otherwise 'are'}
\item{\code{y}}{'y' if n == 1 otherwise 'ie'}
}
}

\examples{
message_chk("there \%r \%n", " problem director\%y\%s")
message_chk("there \%r \%n", " problem director\%y\%s", n = 1)
message_chk("There \%r \%n", " problem director\%y\%s.", n = 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-not-any-na.R
\name{chk_not_any_na}
\alias{chk_not_any_na}
\alias{vld_not_any_na}
\title{Check Not Any Missing Values}
\usage{
chk_not_any_na(x, x_name = NULL)

vld_not_any_na(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if not any missing values using

\code{!anyNA(x)}

\strong{Pass}: \code{1}, \code{1:2}, \code{"1"}, \code{logical(0)}.

\strong{Fail}: \code{NA}, \code{c(1, NA)}.
}
\section{Functions}{
\itemize{
\item \code{vld_not_any_na()}: Validate Not Any Missing Values

}}
\examples{
# chk_not_any_na
chk_not_any_na(1)
try(chk_not_any_na(NA))
# vld_not_any_na
vld_not_any_na(1)
vld_not_any_na(1:2)
vld_not_any_na(NA_real_)
vld_not_any_na(integer(0))
vld_not_any_na(c(NA, 1))
vld_not_any_na(TRUE)
}
\seealso{
For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other misc_checkers: 
\code{\link{chk_join}()},
\code{\link{chk_not_empty}()},
\code{\link{chk_unique}()}
}
\concept{misc_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-atomic.R
\name{chk_atomic}
\alias{chk_atomic}
\alias{vld_atomic}
\title{Check Atomic}
\usage{
chk_atomic(x, x_name = NULL)

vld_atomic(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if atomic using

\code{is.atomic(x)}
}
\section{Functions}{
\itemize{
\item \code{vld_atomic()}: Validate Atomic

}}
\examples{
# chk_atomic
chk_atomic(1)
try(chk_atomic(list(1)))
# vld_atomic
vld_atomic(1)
vld_atomic(matrix(1:3))
vld_atomic(character(0))
vld_atomic(list(1))
vld_atomic(NULL)
}
\seealso{
\code{\link[=is.atomic]{is.atomic()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other data_structure_checkers: 
\code{\link{chk_array}()},
\code{\link{chk_list}()},
\code{\link{chk_matrix}()},
\code{\link{chk_vector}()}
}
\concept{data_structure_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-ext.R
\name{chk_ext}
\alias{chk_ext}
\alias{vld_ext}
\title{Check File Extension}
\usage{
chk_ext(x, ext, x_name = NULL)

vld_ext(x, ext)
}
\arguments{
\item{x}{The object to check.}

\item{ext}{A character vector of the permitted file extensions
(without the .).}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks extension using

\code{vld_string(x) && vld_subset(tools::file_ext(x), ext)}

The user may want to use \code{\link[=toupper]{toupper()}} or \code{\link[=tolower]{tolower()}}
to ensure the case matches.
}
\section{Functions}{
\itemize{
\item \code{vld_ext()}: Validate File Extension

}}
\examples{
# chk_ext
try(chk_ext("file1.pdf", "png"))
# vld_ext
vld_ext("oeu.pdf", "pdf")
vld_ext(toupper("oeu.pdf"), "PDF")
}
\seealso{
\code{\link[=vld_string]{vld_string()}}

\code{\link[=vld_subset]{vld_subset()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other file_checkers: 
\code{\link{chk_dir}()},
\code{\link{chk_file}()}
}
\concept{file_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-compatible-lengths.R
\name{chk_compatible_lengths}
\alias{chk_compatible_lengths}
\alias{vld_compatible_lengths}
\title{Check Compatible Lengths}
\usage{
chk_compatible_lengths(..., x_name = NULL)

vld_compatible_lengths(...)
}
\arguments{
\item{...}{The objects to check for compatible lengths.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails.
}
\description{
Checks objects (including vectors) have lengths that could be 'strictly
recycled'. That is to say they must all be either zero length or the same
length with some of length 1.
}
\details{
This function helps to check vectors could be 'strictly recycled.'
For example the function will error if you had a vector of length 2 and
length 4, even though the vector of length 2 could be 'loosely recycled' to
match up to the vector of length 4 when combined.

The intent of the function is to check that only strict recycling is
occurring.
}
\section{Functions}{
\itemize{
\item \code{vld_compatible_lengths()}: Validate Compatible Lengths

}}
\examples{
# chk_compatible_lengths

a <- integer(0)
b <- numeric(0)
chk_compatible_lengths(a, b)

a <- 1
b <- 2
chk_compatible_lengths(a, b)

a <- 1:3
b <- 1:3
chk_compatible_lengths(a, b)

b <- 1
chk_compatible_lengths(a, b)

b <- 1:2
try(chk_compatible_lengths(a, b))

b <- 1:6
try(chk_compatible_lengths(a, b))
# vld_compatible_lengths

a <- integer(0)
b <- numeric(0)
vld_compatible_lengths(a, b)

a <- 1
b <- 2
vld_compatible_lengths(a, b)

a <- 1:3
b <- 1:3
vld_compatible_lengths(a, b)

b <- 1
vld_compatible_lengths(a, b)

b <- 1:2
vld_compatible_lengths(a, b)

b <- 1:6
vld_compatible_lengths(a, b)
}
\seealso{
For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other length_checkers: 
\code{\link{chk_length}()}
}
\concept{length_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-unused.R
\name{chk_unused}
\alias{chk_unused}
\alias{vld_unused}
\title{Check ... Unused}
\usage{
chk_unused(...)

vld_unused(...)
}
\arguments{
\item{...}{Additional arguments.}
}
\value{
The \code{chk_} function throws an informative error if the test fails.
}
\description{
Checks if ... is unused

\code{length(list(...)) == 0L}
}
\section{Functions}{
\itemize{
\item \code{vld_unused()}: Validate ... Unused

}}
\examples{
# chk_unused
fun <- function(x, ...) {
  chk_unused(...)
  x
}
fun(1)
try(fun(1, 2))
# vld_unused
fun <- function(x, ...) {
  vld_unused(...)
}
fun(1)
try(fun(1, 2))
}
\seealso{
\code{\link[=length]{length()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other ellipsis_checkers: 
\code{\link{chk_used}()}
}
\concept{ellipsis_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-not-empty.R
\name{chk_not_empty}
\alias{chk_not_empty}
\alias{vld_not_empty}
\title{Check Not Empty}
\usage{
chk_not_empty(x, x_name = NULL)

vld_not_empty(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if not empty using

\code{length(x) != 0L}

\strong{Pass}: \code{1}, \code{1:2}, \code{NA}, \code{matrix(1:3)}, \code{list(1)}, \code{data.frame(x = 1)}.

\strong{Fail}: \code{NULL}, \code{logical(0)}, \code{list()}, \code{data.frame()}.
}
\section{Functions}{
\itemize{
\item \code{vld_not_empty()}: Validate Not Empty

}}
\examples{
# chk_not_empty
chk_not_empty(1)
try(chk_not_empty(numeric(0)))
# vld_not_empty
vld_not_empty(1)
vld_not_empty(matrix(1:3))
vld_not_empty(character(0))
vld_not_empty(list(1))
vld_not_empty(NULL)
vld_not_empty(list())
}
\seealso{
\code{\link[=length]{length()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other misc_checkers: 
\code{\link{chk_join}()},
\code{\link{chk_not_any_na}()},
\code{\link{chk_unique}()}
}
\concept{misc_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-complex-number.R
\name{chk_complex_number}
\alias{chk_complex_number}
\alias{vld_complex_number}
\title{Check Complex Number}
\usage{
chk_complex_number(x, x_name = NULL)

vld_complex_number(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if non-missing complex scalar using

\code{is.complex(x) && length(x) == 1L && !anyNA(x)}
}
\section{Functions}{
\itemize{
\item \code{vld_complex_number()}: Validate Complex Number

}}
\examples{
# chk_complex_number
chk_complex_number(as.complex(1.1))
try(chk_complex_number(1.1))
# vld_complex_number
vld_complex_number(as.complex(2))
}
\seealso{
\code{\link[=is.integer]{is.integer()}}

\code{\link[=vld_true]{vld_true()}}

\code{\link[=vld_number]{vld_number()}}

For more details about the use of this function,
please read the article
\href{https://poissonconsulting.github.io/chk/articles/chk-families.html}{chk families}.

Other scalar_checker: 
\code{\link{chk_whole_number}()}
}
\concept{scalar_checker}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-dir.R
\name{chk_dir}
\alias{chk_dir}
\alias{vld_dir}
\title{Check Directory Exists}
\usage{
chk_dir(x, x_name = NULL)

vld_dir(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if directory exists using

\code{vld_string(x) && dir.exists(x)}
}
\section{Functions}{
\itemize{
\item \code{vld_dir()}: Validate Directory Exists

}}
\examples{
# chk_dir
chk_dir(tempdir())
try(chk_dir(tempfile()))
# vld_dir
vld_dir(1)
vld_dir(tempdir())
vld_dir(tempfile())
}
\seealso{
\code{\link[=vld_string]{vld_string()}}

\code{\link[=dir.exists]{dir.exists()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other file_checkers: 
\code{\link{chk_ext}()},
\code{\link{chk_file}()}
}
\concept{file_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-equivalent.R
\name{chk_equivalent}
\alias{chk_equivalent}
\alias{vld_equivalent}
\title{Check Equivalent}
\usage{
chk_equivalent(x, y, tolerance = sqrt(.Machine$double.eps), x_name = NULL)

vld_equivalent(x, y, tolerance = sqrt(.Machine$double.eps))
}
\arguments{
\item{x}{The object to check.}

\item{y}{An object to check against.}

\item{tolerance}{A non-negative numeric scalar.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if is equivalent (equal ignoring attributes) to y using

\code{vld_true(all.equal(x, y, tolerance, check.attributes = FALSE))}
}
\section{Functions}{
\itemize{
\item \code{vld_equivalent()}: Validate Equivalent

}}
\examples{
# chk_equivalent
chk_equivalent(1, 1.00000001)
try(chk_equivalent(1, 1.0000001))
chk_equivalent(1, 1L)
chk_equivalent(c(x = 1), c(y = 1))
vld_equivalent(c(x = 1), c(y = 1L))
}
\seealso{
\code{\link[=vld_true]{vld_true()}}

\code{\link[=all.equal]{all.equal()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other equal_checkers: 
\code{\link{chk_all_equal}()},
\code{\link{chk_all_equivalent}()},
\code{\link{chk_all_identical}()},
\code{\link{chk_equal}()},
\code{\link{chk_identical}()}
}
\concept{equal_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-string.R
\name{chk_string}
\alias{chk_string}
\alias{vld_string}
\title{Check String}
\usage{
chk_string(x, x_name = NULL)

vld_string(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if string

\code{is.character(x) && length(x) == 1L && !anyNA(x)}
}
\section{Functions}{
\itemize{
\item \code{vld_string()}: Validate String

}}
\examples{
# chk_string
chk_string("1")
try(chk_string(1))
# vld_string
vld_string("1")
vld_string("")
vld_string(1)
vld_string(NA_character_)
vld_string(c("1", "1"))
}
\seealso{
\code{\link[=length]{length()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other scalar_checkers: 
\code{\link{chk_count}()},
\code{\link{chk_date}()},
\code{\link{chk_date_time}()},
\code{\link{chk_false}()},
\code{\link{chk_flag}()},
\code{\link{chk_lgl}()},
\code{\link{chk_scalar}()},
\code{\link{chk_true}()},
\code{\link{chk_tz}()}
}
\concept{scalar_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-character-or-factor.R
\name{chk_character_or_factor}
\alias{chk_character_or_factor}
\alias{vld_character_or_factor}
\title{Check Character or Factor}
\usage{
chk_character_or_factor(x, x_name = NULL)

vld_character_or_factor(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if character or factor using

\code{is.character(x) || is.factor(x)}
}
\section{Functions}{
\itemize{
\item \code{vld_character_or_factor()}: Validate Character or Factor

}}
\examples{
# chk_character_or_factor
chk_character_or_factor("1")
chk_character_or_factor(factor("1"))
try(chk_character(1))
# vld_character_or_factor
vld_character_or_factor("1")
vld_character_or_factor(matrix("a"))
vld_character_or_factor(character(0))
vld_character_or_factor(NA_character_)
vld_character_or_factor(1)
vld_character_or_factor(TRUE)
vld_character_or_factor(factor("text"))
}
\seealso{
\code{\link[=is.character]{is.character()}}

\code{\link[=is.factor]{is.factor()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other data_type_checkers: 
\code{\link{chk_character}()},
\code{\link{chk_complex}()},
\code{\link{chk_double}()},
\code{\link{chk_environment}()},
\code{\link{chk_integer}()},
\code{\link{chk_logical}()},
\code{\link{chk_numeric}()},
\code{\link{chk_raw}()}

Other factor_checkers: 
\code{\link{chk_factor}()}
}
\concept{data_type_checkers}
\concept{factor_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-null-or.R
\name{chk_null_or}
\alias{chk_null_or}
\title{Check NULL Or}
\usage{
chk_null_or(x, chk, ..., vld, x_name = NULL)
}
\arguments{
\item{x}{The object to check.}

\item{chk}{A chk function. Soft-deprecated for vld. \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}}

\item{...}{Arguments passed to chk.}

\item{vld}{A vld function.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
An informative error if the test fails.
}
\description{
Checks if NULL or passes test.
}
\examples{
chk_null_or(NULL, chk_number)
chk_null_or(1, chk_number)
try(chk_null_or("1", chk_number))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check-dim.R
\name{check_dim}
\alias{check_dim}
\title{Check Dimension}
\usage{
check_dim(x, dim = length, values = numeric(0), x_name = NULL, dim_name = NULL)
}
\arguments{
\item{x}{The object to check.}

\item{dim}{A function returning a non-negative whole number of the dimension.}

\item{values}{A flag or a whole numeric vector of the value, value range or possible values.}

\item{x_name}{A string of the name of object x or NULL.}

\item{dim_name}{A string of the name of the dim function.}
}
\value{
An informative error if the test fails or an invisible copy of x.
}
\description{
Checks dimension of an object.
}
\examples{
check_dim(1)
try(check_dim(1, values = FALSE))
try(check_dim(1, values = c(10, 2)))
try(check_dim(data.frame(x = 1), dim = nrow, values = c(10, 10, 2)))
}
\seealso{
Other check: 
\code{\link{check_data}()},
\code{\link{check_dirs}()},
\code{\link{check_files}()},
\code{\link{check_key}()},
\code{\link{check_length}()},
\code{\link{check_names}()},
\code{\link{check_values}()}
}
\concept{check}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-named.R
\name{chk_named}
\alias{chk_named}
\alias{vld_named}
\title{Check Named}
\usage{
chk_named(x, x_name = NULL)

vld_named(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if is named using

\code{!is.null(names(x))}
}
\section{Functions}{
\itemize{
\item \code{vld_named()}: Validate Named

}}
\examples{
# chk_named
chk_named(c(x = 1))
try(chk_named(list(1)))
# vld_named
vld_named(c(x = 1))
vld_named(list(x = 1))
vld_named(c(x = 1)[-1])
vld_named(list(x = 1)[-1])
vld_named(1)
vld_named(list(1))
}
\seealso{
\code{\link[=names]{names()}}

\code{\link[=is.null]{is.null()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other name_checkers: 
\code{\link{chk_valid_name}()}
}
\concept{name_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/err.R
\name{err}
\alias{err}
\alias{wrn}
\alias{msg}
\title{Stop, Warning and Message Messages}
\usage{
err(
  ...,
  n = NULL,
  tidy = TRUE,
  .subclass = NULL,
  class = NULL,
  call = rlang::caller_call(3)
)

wrn(..., n = NULL, tidy = TRUE, .subclass = NULL, class = NULL)

msg(..., n = NULL, tidy = TRUE, .subclass = NULL, class = NULL)
}
\arguments{
\item{...}{zero or more objects which can be coerced to character
    (and which are pasted together with no separator) or a single
    condition object.}

\item{n}{The value of n for converting \code{sprintf}-like types.}

\item{tidy}{A flag specifying whether capitalize the first character and add a missing period.}

\item{.subclass}{A string of the class of the error message.}

\item{class}{Subclass of the condition.}

\item{call}{The execution environment of a currently running
function, e.g. \code{call = caller_env()}. The corresponding function
call is retrieved and mentioned in error messages as the source
of the error.

You only need to supply \code{call} when throwing a condition from a
helper function which wouldn't be relevant to mention in the
message.

Can also be \code{NULL} or a \link[rlang:topic-defuse]{defused function call} to
respectively not display any call or hard-code a code to display.

For more information about error calls, see \ifelse{html}{\link[rlang:topic-error-call]{Including function calls in error messages}}{\link[rlang:topic-error-call]{Including function calls in error messages}}.}
}
\description{
The functions call \code{\link[=message_chk]{message_chk()}} to process
the message and then
\code{\link[rlang:abort]{rlang::abort()}}, \code{\link[rlang:abort]{rlang::warn()}} and
\code{\link[rlang:abort]{rlang::inform()}}, respectively.
}
\details{
The user can set the subclass.
}
\section{Functions}{
\itemize{
\item \code{err()}: Error

\item \code{wrn()}: Warning

\item \code{msg()}: Message

}}
\examples{

# err
try(err("there \%r \%n problem value\%s", n = 2))

# wrn
wrn("there \%r \%n problem value\%s", n = 2)

# msg
msg("there \%r \%n problem value\%s", n = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-superset.R
\name{chk_superset}
\alias{chk_superset}
\alias{vld_superset}
\title{Check Superset}
\usage{
chk_superset(x, values, x_name = NULL)

vld_superset(x, values)
}
\arguments{
\item{x}{The object to check.}

\item{values}{A vector of the permitted values.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if includes all values using

\code{all(values \%in\% x)}

Pay attention to the order of the arguments \code{value} and \code{x}
in this function compared to \code{\link[=chk_subset]{chk_subset()}}
}
\section{Functions}{
\itemize{
\item \code{vld_superset()}: Validates Superset

}}
\examples{
# chk_superset
chk_superset(1:3, 1)
try(chk_superset(1:3, 4))
# vld_superset
vld_superset(1:3, 1)
vld_superset(1:3, 4)
vld_superset(integer(0), integer(0))
}
\seealso{
\code{\link[=all]{all()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other set_checkers: 
\code{\link{chk_not_subset}()},
\code{\link{chk_orderset}()},
\code{\link{vld_not_subset}()},
\code{\link{vld_orderset}()}
}
\concept{set_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/p.R
\name{p}
\alias{p}
\alias{p0}
\title{Concatenate Strings}
\usage{
p(..., sep = " ", collapse = NULL)

p0(..., collapse = NULL)
}
\arguments{
\item{...}{one or more \R objects, to be converted to character vectors.}

\item{sep}{a character string to separate the terms.  Not
    \code{\link[base]{NA_character_}}.}

\item{collapse}{an optional character string to separate the results.  Not
    \code{\link[base]{NA_character_}}.  When \code{collapse} is a string,
    the result is always a string (\code{\link[base]{character}} of length 1).}
}
\value{
A character vector.
}
\description{
A wrapper on \code{\link[base:paste]{base::paste()}}.
}
\section{Functions}{
\itemize{
\item \code{p0()}: A wrapper on \code{\link[base:paste]{base::paste0()}}

}}
\examples{
p("a", "b")
p(c("a", "b"), collapse = " ")
p0("a", "b")
p0(c("a", "b"), collapse = "")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-dbl.R
\name{chk_dbl}
\alias{chk_dbl}
\alias{vld_dbl}
\title{Check Double Scalar}
\usage{
chk_dbl(x, x_name = NULL)

vld_dbl(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if double scalar using

\code{is.double(x) && length(x) == 1L}

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}
}
\section{Functions}{
\itemize{
\item \code{vld_dbl()}: Validate Double

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

}}
\examples{
# chk_dbl
chk_dbl(1)
try(chk_dbl(1L))
# vld_dbl
vld_dbl(1)
vld_dbl(double(0))
vld_dbl(NA_real_)
vld_dbl(c(1, 1))
vld_dbl(1L)
}
\seealso{
Other deprecated: 
\code{\link{chk_chr}()},
\code{\link{chk_deprecated}},
\code{\link{chk_wnum}()}
}
\concept{deprecated}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check-files.R
\name{check_files}
\alias{check_files}
\title{Check Files Exist}
\usage{
check_files(x, exists = TRUE, x_name = NULL)
}
\arguments{
\item{x}{The object to check.}

\item{exists}{A flag specifying whether the files/directories must (or must not) exist.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
An informative error if the test fails or an invisible copy of x.
}
\description{
Checks if all files exist (or if exists = FALSE do not exist as files or directories).
}
\examples{
check_files(tempfile("unlikely-that-exists-chk"), exists = FALSE)
try(check_files(tempfile("unlikely-that-exists-chk")))
}
\seealso{
Other check: 
\code{\link{check_data}()},
\code{\link{check_dim}()},
\code{\link{check_dirs}()},
\code{\link{check_key}()},
\code{\link{check_length}()},
\code{\link{check_names}()},
\code{\link{check_values}()}
}
\concept{check}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-not-missing.R
\name{chk_not_missing}
\alias{chk_not_missing}
\alias{vld_not_missing}
\title{Check Not Missing Argument}
\usage{
chk_not_missing(x, x_name = "`x`")

vld_not_missing(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks argument not missing using

\code{!missing(x)}
}
\details{
Currently only checks if value is available
(as opposed to whether it was specified).
}
\section{Functions}{
\itemize{
\item \code{vld_not_missing()}: Validate Not Missing Argument

}}
\examples{
# chk_not_missing
fun <- function(x) {
  chk_not_missing(x)
}
fun(1)
try(fun())
# vld_not_missing
fun <- function(x) {
  vld_not_missing(x)
}
fun()
fun(1)
}
\seealso{
\code{\link[=missing]{missing()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other missing_checkers: 
\code{\link{chk_function}()},
\code{\link{chk_missing}()}
}
\concept{missing_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check-length.R
\name{check_length}
\alias{check_length}
\title{Check Length}
\usage{
check_length(x, values = numeric(0), x_name = NULL)
}
\arguments{
\item{x}{The object to check.}

\item{values}{A flag or a whole numeric vector of the value, value range or possible values.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
An informative error if the test fails or an invisible copy of x.
}
\description{
Checks length of an object.
}
\examples{
check_length(1)
try(check_length(1, values = FALSE))
try(check_length(1, values = c(10, 2)))
}
\seealso{
Other check: 
\code{\link{check_data}()},
\code{\link{check_dim}()},
\code{\link{check_dirs}()},
\code{\link{check_files}()},
\code{\link{check_key}()},
\code{\link{check_names}()},
\code{\link{check_values}()}
}
\concept{check}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-vector.R
\name{chk_vector}
\alias{chk_vector}
\alias{vld_vector}
\title{Check Vector}
\usage{
chk_vector(x, x_name = NULL)

vld_vector(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if is a vector using

\code{(is.atomic(x) && !is.matrix(x) && !is.array(x)) || is.list(x)}
}
\details{
\code{is.vector(x)} is not reliable because it returns TRUE only
if the object is a vector with no attributes apart from names.
}
\section{Functions}{
\itemize{
\item \code{vld_vector()}: Validate Vector

}}
\examples{
# chk_vector
chk_vector(1)
chk_vector(list())
try(chk_vector(matrix(1)))
# vld_vector
vld_vector(1)
}
\seealso{
\code{\link[=is.atomic]{is.atomic()}}, \code{\link[=is.matrix]{is.matrix()}}, \code{\link[=is.array]{is.array()}}, \code{\link[=is.list]{is.list()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other data_structure_checkers: 
\code{\link{chk_array}()},
\code{\link{chk_atomic}()},
\code{\link{chk_list}()},
\code{\link{chk_matrix}()}
}
\concept{data_structure_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check-dirs.R
\name{check_dirs}
\alias{check_dirs}
\title{Check Directories Exist}
\usage{
check_dirs(x, exists = TRUE, x_name = NULL)
}
\arguments{
\item{x}{The object to check.}

\item{exists}{A flag specifying whether the files/directories must (or must not) exist.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
An informative error if the test fails or an invisible copy of x.
}
\description{
Checks if all directories exist (or if exists = FALSE do not exist as directories or files).
}
\examples{
check_dirs(tempdir())
try(check_dirs(tempdir(), exists = FALSE))
}
\seealso{
Other check: 
\code{\link{check_data}()},
\code{\link{check_dim}()},
\code{\link{check_files}()},
\code{\link{check_key}()},
\code{\link{check_length}()},
\code{\link{check_names}()},
\code{\link{check_values}()}
}
\concept{check}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-integer.R
\name{chk_integer}
\alias{chk_integer}
\alias{vld_integer}
\title{Check Integer}
\usage{
chk_integer(x, x_name = NULL)

vld_integer(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if integer using

\code{is.integer(x)}
}
\section{Functions}{
\itemize{
\item \code{vld_integer()}: Validate Integer

}}
\examples{
# chk_integer
chk_integer(1L)
try(chk_integer(1))
# vld_integer
vld_integer(1L)
vld_integer(matrix(1:4, nrow = 2L))
vld_integer(integer(0))
vld_integer(NA_integer_)
vld_integer(1)
vld_integer(TRUE)
}
\seealso{
\code{\link[=is.integer]{is.integer()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other data_type_checkers: 
\code{\link{chk_character}()},
\code{\link{chk_character_or_factor}()},
\code{\link{chk_complex}()},
\code{\link{chk_double}()},
\code{\link{chk_environment}()},
\code{\link{chk_logical}()},
\code{\link{chk_numeric}()},
\code{\link{chk_raw}()}
}
\concept{data_type_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-join.R
\name{chk_join}
\alias{chk_join}
\alias{vld_join}
\title{Check Join}
\usage{
chk_join(x, y, by, x_name = NULL)

vld_join(x, y, by)
}
\arguments{
\item{x}{The object to check.}

\item{y}{A data.frame with columns in by.}

\item{by}{A character vector specifying the column names to join x and y on.
If named the names are the corresponding columns in x.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if all rows in x match at least one in y.
}
\section{Functions}{
\itemize{
\item \code{vld_join()}: Validate Join

}}
\examples{
# chk_join
chk_join(data.frame(z = 1), data.frame(z = 1:2), by = "z")
try(chk_join(data.frame(z = 1), data.frame(z = 2), by = "z"))
# vld_join
vld_join(data.frame(z = 1), data.frame(z = 1:2), by = "z")
vld_join(data.frame(z = 1), data.frame(z = 2), by = "z")
vld_join(data.frame(z = 1), data.frame(a = 1:2), by = c(z = "a"))
vld_join(data.frame(z = 1), data.frame(a = 2), by = c(z = "a"))
}
\seealso{
For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other misc_checkers: 
\code{\link{chk_not_any_na}()},
\code{\link{chk_not_empty}()},
\code{\link{chk_unique}()}
}
\concept{misc_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-file.R
\name{chk_file}
\alias{chk_file}
\alias{vld_file}
\title{Check File Exists}
\usage{
chk_file(x, x_name = NULL)

vld_file(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if file exists using

\code{vld_string(x) && file.exists(x) && !dir.exists(x)}
}
\section{Functions}{
\itemize{
\item \code{vld_file()}: Validate File Exists

}}
\examples{
# chk_file
try(chk_file(tempfile()))
# vld_file
vld_file(tempfile())
}
\seealso{
\code{\link[=vld_string]{vld_string()}}

\code{\link[=file.exists]{file.exists()}}

\code{\link[=dir.exists]{dir.exists()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other file_checkers: 
\code{\link{chk_dir}()},
\code{\link{chk_ext}()}
}
\concept{file_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cc.R
\name{cc}
\alias{cc}
\title{Concatenate with Commas}
\usage{
cc(
  x,
  conj = ", ",
  sep = ", ",
  brac = if (is.character(x) || is.factor(x)) "'" else "",
  ellipsis = 10L,
  chk = TRUE
)
}
\arguments{
\item{x}{The object to concatenate.}

\item{conj}{A string of the conjunction to separate the last value by.}

\item{sep}{A string of the separator.}

\item{brac}{A string to brac the values by.}

\item{ellipsis}{A numeric scalar of the maximum number of values to display
before using an ellipsis.}

\item{chk}{A flag specifying whether to check the other parameters.}
}
\value{
A string.
}
\description{
Concatenates object values into a string with each value separated by a comma
and the last value separated by a conjunction.
}
\details{
By default, if x has more than 10 values an ellipsis is used
to ensure only 10 values are displayed (including the ellipsis).
}
\examples{
cc(1:2)
cc(1:2, conj = " or")
cc(3:1, brac = "'")
cc(1:11)
cc(as.character(1:2))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chkor-vld.R
\name{chkor_vld}
\alias{chkor_vld}
\title{Chk OR}
\usage{
chkor_vld(...)
}
\arguments{
\item{...}{Multiple \code{vld_} calls.

A common mistake is to pass \code{chk_} calls.

\code{chkor_vld()} is relatively slow.
If at all possible use \code{\link[=chk_null_or]{chk_null_or()}} or first test using the individual
\code{vld_} functions and then call \code{chkor_vld()} to generate an informative
error message.}
}
\value{
An informative error if the test fails.
}
\description{
Chk OR
}
\examples{
chkor_vld()
chkor_vld(vld_flag(TRUE))
try(chkor_vld(vld_flag(1)))
try(chkor_vld(vld_flag(1), vld_flag(2)))
chkor_vld(vld_flag(1), vld_flag(TRUE))
}
\seealso{
\code{\link[=chk_null_or]{chk_null_or()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-complex.R
\name{chk_complex}
\alias{chk_complex}
\alias{vld_complex}
\title{Check Complex}
\usage{
chk_complex(x, x_name = NULL)

vld_complex(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if complex using

\code{is.complex(x)}
}
\section{Functions}{
\itemize{
\item \code{vld_complex()}: Validate Complex

}}
\examples{
# chk_complex
chk_complex(1i)
try(chk_complex(1))
# vld_complex
vld_complex(1i)
vld_complex(complex())
vld_complex(NA_complex_)
vld_complex(1)
vld_complex(TRUE)
}
\seealso{
\code{\link[=is.complex]{is.complex()}}

For more details about the use of this function,
please read the article
\href{https://poissonconsulting.github.io/chk/articles/chk-families.html}{chk families}.

Other data_type_checkers: 
\code{\link{chk_character}()},
\code{\link{chk_character_or_factor}()},
\code{\link{chk_double}()},
\code{\link{chk_environment}()},
\code{\link{chk_integer}()},
\code{\link{chk_logical}()},
\code{\link{chk_numeric}()},
\code{\link{chk_raw}()}
}
\concept{data_type_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-not-subset.R, R/chk-subset.R
\name{vld_not_subset}
\alias{vld_not_subset}
\alias{chk_subset}
\alias{vld_subset}
\title{Check Subset}
\usage{
vld_not_subset(x, values)

chk_subset(x, values, x_name = NULL)

vld_subset(x, values)
}
\arguments{
\item{x}{The object to check.}

\item{values}{A vector of the permitted values.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if all values in values using

\code{all(x \%in\% values)}

Pay attention to the order of the arguments \code{value} and \code{x}
in this function compared to \code{\link[=chk_superset]{chk_superset()}}
}
\section{Functions}{
\itemize{
\item \code{vld_not_subset()}: Validate Not Subset

\item \code{vld_subset()}: Validate Subset

}}
\examples{
# vld_not_subset
vld_not_subset(numeric(0), 1:10)
vld_not_subset(1, 1:10)
vld_not_subset(11, 1:10)
# chk_subset
chk_subset(1, 1:10)
try(chk_subset(11, 1:10))
# vld_subset
vld_subset(numeric(0), 1:10)
vld_subset(1, 1:10)
vld_subset(11, 1:10)
}
\seealso{
\code{\link[=all]{all()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other set_checkers: 
\code{\link{chk_not_subset}()},
\code{\link{chk_orderset}()},
\code{\link{chk_superset}()},
\code{\link{vld_orderset}()}
}
\concept{set_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-numeric.R
\name{chk_numeric}
\alias{chk_numeric}
\alias{vld_numeric}
\title{Check Numeric}
\usage{
chk_numeric(x, x_name = NULL)

vld_numeric(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if numeric using

\code{is.numeric(x)}

\strong{Pass}: \code{1}, \code{1:2}, \code{NA_real_}, \code{integer(0)}, \code{matrix(1:3)}.

\strong{Fail}: \code{TRUE}, \code{"1"}, \code{NA}, \code{NULL}.
}
\section{Functions}{
\itemize{
\item \code{vld_numeric()}: Validate Numeric

}}
\examples{
# chk_numeric
chk_numeric(1)
try(chk_numeric("1"))
# vld_numeric
vld_numeric(1)
vld_numeric(1:2)
vld_numeric(NA_real_)
vld_numeric(integer(0))
vld_numeric("1")
vld_numeric(TRUE)
}
\seealso{
\code{\link[=is.numeric]{is.numeric()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other data_type_checkers: 
\code{\link{chk_character}()},
\code{\link{chk_character_or_factor}()},
\code{\link{chk_complex}()},
\code{\link{chk_double}()},
\code{\link{chk_environment}()},
\code{\link{chk_integer}()},
\code{\link{chk_logical}()},
\code{\link{chk_raw}()}
}
\concept{data_type_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-null.R
\name{chk_null}
\alias{chk_null}
\alias{vld_null}
\title{Check NULL}
\usage{
chk_null(x, x_name = NULL)

vld_null(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if NULL using

\code{is.null(x)}
}
\section{Functions}{
\itemize{
\item \code{vld_null()}: Validate NULL

}}
\examples{
# chk_null
try(chk_null(1))
chk_null(NULL)
# vld_null
vld_null(NULL)
vld_null(1)
}
\seealso{
\code{\link[=is.null]{is.null()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other null_checkers: 
\code{\link{chk_not_null}()}
}
\concept{null_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-s3-class.R
\name{chk_s3_class}
\alias{chk_s3_class}
\alias{vld_s3_class}
\title{Check Type}
\usage{
chk_s3_class(x, class, x_name = NULL)

vld_s3_class(x, class)
}
\arguments{
\item{x}{The object to check.}

\item{class}{A string specifying the class.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks inherits from S3 class using

\code{!isS4(x) && inherits(x, class)}
}
\section{Functions}{
\itemize{
\item \code{vld_s3_class()}: Validate Inherits from S3 Class

}}
\examples{
# chk_s3_class
chk_s3_class(1, "numeric")
try(chk_s3_class(getClass("MethodDefinition"), "classRepresentation"))
# vld_s3_class
vld_s3_class(numeric(0), "numeric")
vld_s3_class(getClass("MethodDefinition"), "classRepresentation")
}
\seealso{
\code{\link[=inherits]{inherits()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other id_checkers: 
\code{\link{chk_data}()},
\code{\link{chk_is}()},
\code{\link{chk_s4_class}()}
}
\concept{id_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-all-equivalent.R
\name{chk_all_equivalent}
\alias{chk_all_equivalent}
\alias{vld_all_equivalent}
\title{Check All Equivalent}
\usage{
chk_all_equivalent(x, tolerance = sqrt(.Machine$double.eps), x_name = NULL)

vld_all_equivalent(x, tolerance = sqrt(.Machine$double.eps))
}
\arguments{
\item{x}{The object to check.}

\item{tolerance}{A non-negative numeric scalar.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks all elements in x equivalent using

\code{length(x) < 2L || all(vapply(x, vld_equivalent, TRUE, y = x[[1]], tolerance = tolerance))}
}
\section{Functions}{
\itemize{
\item \code{vld_all_equivalent()}: Validate All Equivalent

}}
\examples{
# chk_all_equivalent
chk_all_equivalent(c(1, 1.00000001))
try(chk_all_equivalent(c(1, 1.0000001)))
chk_all_equivalent(list(c(x = 1), c(x = 1)))
chk_all_equivalent(list(c(x = 1), c(y = 1)))
# vld_all_equivalent
vld_all_equivalent(c(x = 1, y = 1))
}
\seealso{
\code{\link[=length]{length()}}

\code{\link[=vld_equivalent]{vld_equivalent()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other equal_checkers: 
\code{\link{chk_all_equal}()},
\code{\link{chk_all_identical}()},
\code{\link{chk_equal}()},
\code{\link{chk_equivalent}()},
\code{\link{chk_identical}()}

Other all_checkers: 
\code{\link{chk_all}()},
\code{\link{chk_all_equal}()},
\code{\link{chk_all_identical}()}
}
\concept{all_checkers}
\concept{equal_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{deparse_backtick_chk}
\alias{deparse_backtick_chk}
\alias{backtick_chk}
\alias{unbacktick_chk}
\title{Deparse Backtick}
\usage{
deparse_backtick_chk(x)

backtick_chk(x)

unbacktick_chk(x)
}
\arguments{
\item{x}{A substituted object to deparse.}
}
\value{
A string of the backticked substituted object.
}
\description{
\code{deparse_backtick_chk} is a wrapper on \code{\link[=deparse]{deparse()}}
and \code{backtick_chk}.
}
\details{
It is exported to allow users to easily construct their own \code{chk_} functions.
}
\section{Functions}{
\itemize{
\item \code{backtick_chk()}: Backtick

\item \code{unbacktick_chk()}: Unbacktick

}}
\examples{

# deparse_backtick_chk
deparse_backtick_chk(2)
deparse_backtick_chk(2^2)
}
\seealso{
\code{\link[=deparse]{deparse()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-logical.R
\name{chk_logical}
\alias{chk_logical}
\alias{vld_logical}
\title{Check Logical}
\usage{
chk_logical(x, x_name = NULL)

vld_logical(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if logical using

\code{is.logical(x)}

If you want to check if it is a scalar,
meaning that in addition to being of logical type,
it has \code{length(x) == 1}, you should use \code{\link[=chk_lgl]{chk_lgl()}}
}
\section{Functions}{
\itemize{
\item \code{vld_logical()}: Validate Logical

}}
\examples{
# chk_logical
chk_logical(TRUE)
try(chk_logical(1))
# vld_logical
vld_logical(TRUE)
vld_logical(matrix(TRUE))
vld_logical(logical(0))
vld_logical(NA)
vld_logical(1)
vld_logical("TRUE")
}
\seealso{
\code{\link[=is.logical]{is.logical()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other logical_checkers: 
\code{\link{chk_false}()},
\code{\link{chk_flag}()},
\code{\link{chk_lgl}()},
\code{\link{chk_true}()}

Other data_type_checkers: 
\code{\link{chk_character}()},
\code{\link{chk_character_or_factor}()},
\code{\link{chk_complex}()},
\code{\link{chk_double}()},
\code{\link{chk_environment}()},
\code{\link{chk_integer}()},
\code{\link{chk_numeric}()},
\code{\link{chk_raw}()}
}
\concept{data_type_checkers}
\concept{logical_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-package.R
\docType{package}
\name{chk-package}
\alias{chk}
\alias{chk-package}
\title{chk: Check User-Supplied Function Arguments}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

For developers to check user-supplied function arguments. It is designed to be simple, fast and customizable. Error messages follow the tidyverse style guide.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/poissonconsulting/chk}
  \item \url{https://poissonconsulting.github.io/chk/}
  \item Report bugs at \url{https://github.com/poissonconsulting/chk/issues/}
}

}
\author{
\strong{Maintainer}: Joe Thorley \email{joe@poissonconsulting.ca} (\href{https://orcid.org/0000-0002-7683-4592}{ORCID})

Authors:
\itemize{
  \item Kirill Müller (\href{https://orcid.org/0000-0002-1416-3412}{ORCID})
  \item Ayla Pearson (\href{https://orcid.org/0000-0001-7388-1222}{ORCID})
}

Other contributors:
\itemize{
  \item Florencia D'Andrea [contributor]
  \item Nadine Hussein \email{nadine@poissonconsulting.ca} (\href{https://orcid.org/0000-0003-4470-8361}{ORCID}) [contributor]
  \item Evan Amies-Galonski \email{evan@poissonconsulting.ca} (\href{https://orcid.org/0000-0003-1096-2089}{ORCID}) [contributor]
  \item Poisson Consulting [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-date.R
\name{chk_date}
\alias{chk_date}
\alias{vld_date}
\title{Check Date}
\usage{
chk_date(x, x_name = NULL)

vld_date(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks non-missing Date scalar using

\code{inherits(x, "Date") && length(x) == 1L && !anyNA(x)}
}
\section{Functions}{
\itemize{
\item \code{vld_date()}: Validate Date

}}
\examples{
# chk_date
chk_date(Sys.Date())
try(chk_date(1))
# vld_date
vld_date(Sys.Date())
vld_date(Sys.time())
vld_date(1)
}
\seealso{
\code{\link[=inherits]{inherits()}}

\code{\link[=length]{length()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other scalar_checkers: 
\code{\link{chk_count}()},
\code{\link{chk_date_time}()},
\code{\link{chk_false}()},
\code{\link{chk_flag}()},
\code{\link{chk_lgl}()},
\code{\link{chk_scalar}()},
\code{\link{chk_string}()},
\code{\link{chk_true}()},
\code{\link{chk_tz}()}

Other datetime_checkers: 
\code{\link{chk_date_time}()}
}
\concept{datetime_checkers}
\concept{scalar_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{abort_chk}
\alias{abort_chk}
\title{Abort Check}
\usage{
abort_chk(..., n = NULL, tidy = TRUE, call = rlang::caller_call(2))
}
\arguments{
\item{...}{Multiple objects that are converted to a string using
\code{paste0(..., collapse = '')}.}

\item{n}{The value of n for converting \code{sprintf}-like types.}

\item{tidy}{A flag specifying whether capitalize the first character and add a missing period.}

\item{call}{The execution environment of a currently running
function, e.g. \code{call = caller_env()}. The corresponding function
call is retrieved and mentioned in error messages as the source
of the error.

You only need to supply \code{call} when throwing a condition from a
helper function which wouldn't be relevant to mention in the
message.

Can also be \code{NULL} or a \link[rlang:topic-defuse]{defused function call} to
respectively not display any call or hard-code a code to display.

For more information about error calls, see \ifelse{html}{\link[rlang:topic-error-call]{Including function calls in error messages}}{\link[rlang:topic-error-call]{Including function calls in error messages}}.}
}
\value{
Throws an error of class \code{'chk_error'}.
}
\description{
A wrapper on \code{\link[=err]{err()}} that sets the subclass to be \code{'chk_error'}.
}
\details{
It is exported to allow users to easily construct their own \code{chk_} functions.
}
\examples{
try(abort_chk("x must be NULL"))
try(abort_chk("`x` must be NULL"))
try(abort_chk("there \%r \%n problem value\%s", n = 1))
try(abort_chk("there \%r \%n problem value\%s", n = 1.5))
}
\seealso{
\code{\link[=err]{err()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/params.R
\name{params}
\alias{params}
\alias{parameters}
\alias{arguments}
\alias{args}
\title{Parameter Descriptions for chk Package}
\arguments{
\item{...}{Additional arguments.}

\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}

\item{y}{An object to check against.}

\item{chk}{A flag specifying whether to check the other parameters.}

\item{chk_fun}{A chk_ function.}

\item{tolerance}{A non-negative numeric scalar.}

\item{ext}{A character vector of the permitted file extensions
(without the .).}

\item{exists}{A flag specifying whether the files/directories must (or must not) exist.}

\item{value}{A non-missing scalar of a value.}

\item{range}{A non-missing sorted vector of length 2 of the lower and
upper permitted values.}

\item{inclusive}{A flag specifying whether the range is exclusive.}

\item{regexp}{A string of a regular expression.}

\item{values}{A vector of the permitted values.}

\item{class}{A string specifying the class.}

\item{length}{A count of the length.}

\item{upper}{A count of the max length.}

\item{formals}{A count of the number of formal arguments.}

\item{incomparables}{A vector of values that cannot be compared.
FALSE means that all values can be compared.}

\item{by}{A character vector specifying the column names to join x and y on.
If named the names are the corresponding columns in x.}

\item{exclusive}{A flag specifying whether x must only include columns named in values.}

\item{order}{A flag specifying whether the order of columns in x must match names in values.}

\item{nrow}{A flag or a whole numeric vector of the value, value range or possible values.}

\item{key}{A character vector of the columns that represent a unique key.}

\item{vld_fun}{A vld_ function.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Default parameter descriptions which may be overridden in individual
functions.
}
\details{
A flag is a non-missing logical scalar.

A string is a non-missing character scalar.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-number.R
\name{chk_number}
\alias{chk_number}
\alias{vld_number}
\title{Check Number}
\usage{
chk_number(x, x_name = NULL)

vld_number(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if non-missing numeric scalar using

\code{is.numeric(x) && length(x) == 1L && !anyNA(x)}

\strong{Pass}: \code{1}, \code{2L}, \code{log(10)}, \code{-Inf}

\strong{Fail}: \code{"a"}, \code{1:3}, \code{NA_real_}
}
\section{Functions}{
\itemize{
\item \code{vld_number()}: Validate Number

}}
\examples{
# chk_number
chk_number(1.1)
try(chk_number(TRUE))
# vld_number
vld_number(1.1)
}
\seealso{
\code{\link[=is.numeric]{is.numeric()}}

\code{\link[=length]{length()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.
}
\concept{data_type_checkers scalar_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-not-subset.R
\name{chk_not_subset}
\alias{chk_not_subset}
\title{Check Not Subset}
\usage{
chk_not_subset(x, values, x_name = NULL)
}
\arguments{
\item{x}{The object to check.}

\item{values}{A vector of the permitted values.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if not all values in values using

\code{!any(x \%in\% values) || !length(x)}
}
\examples{
# chk_not_subset
chk_not_subset(11, 1:10)
try(chk_not_subset(1, 1:10))
}
\seealso{
\code{\link[=any]{any()}}

\code{\link[=length]{length()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other set_checkers: 
\code{\link{chk_orderset}()},
\code{\link{chk_superset}()},
\code{\link{vld_not_subset}()},
\code{\link{vld_orderset}()}
}
\concept{set_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-wnum.R
\name{chk_wnum}
\alias{chk_wnum}
\alias{vld_wnum}
\title{Check Whole Numeric Scalar}
\usage{
chk_wnum(x, x_name = NULL)

vld_wnum(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if whole numeric scalar using

\code{is.numeric(x) && length(x) == 1L && (is.integer(x) || vld_true(all.equal(x, trunc(x))))}

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}
}
\section{Functions}{
\itemize{
\item \code{vld_wnum()}: Validate Whole Numeric Scalar

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

}}
\examples{
# chk_wnum
chk_wnum(1)
try(chk_wnum(1.1))
# vld_wnum
vld_wnum(1)
vld_wnum(double(0))
vld_wnum(NA_real_)
vld_wnum(c(1, 1))
vld_wnum(1L)
}
\seealso{
Other deprecated: 
\code{\link{chk_chr}()},
\code{\link{chk_dbl}()},
\code{\link{chk_deprecated}}
}
\concept{deprecated}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-valid-name.R
\name{chk_valid_name}
\alias{chk_valid_name}
\alias{vld_valid_name}
\title{Check Valid Name}
\usage{
chk_valid_name(x, x_name = NULL)

vld_valid_name(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if valid name using

\code{identical(make.names(x[!is.na(x)]), as.character(x[!is.na(x)]))}
}
\section{Functions}{
\itemize{
\item \code{vld_valid_name()}: Validate Valid Name

}}
\examples{
# chk_valid_name
chk_valid_name("text")
try(chk_valid_name(".1"))
# vld_valid_name
vld_valid_name(".1")
}
\seealso{
\code{\link[=identical]{identical()}}

\code{\link[=make.names]{make.names()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other name_checkers: 
\code{\link{chk_named}()}
}
\concept{name_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check-key.R
\name{check_key}
\alias{check_key}
\title{Check Key}
\usage{
check_key(x, key = character(0), na_distinct = FALSE, x_name = NULL)
}
\arguments{
\item{x}{The object to check.}

\item{key}{A character vector of the columns that represent a unique key.}

\item{na_distinct}{A flag specifying whether missing values should be considered distinct.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
An informative error if the test fails or an invisible copy of x.
}
\description{
Checks if columns have unique rows.
}
\examples{
x <- data.frame(x = c(1, 2), y = c(1, 1))
check_key(x)
try(check_key(x, "y"))
}
\seealso{
Other check: 
\code{\link{check_data}()},
\code{\link{check_dim}()},
\code{\link{check_dirs}()},
\code{\link{check_files}()},
\code{\link{check_length}()},
\code{\link{check_names}()},
\code{\link{check_values}()}
}
\concept{check}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-flag.R
\name{chk_flag}
\alias{chk_flag}
\alias{vld_flag}
\title{Check Flag}
\usage{
chk_flag(x, x_name = NULL)

vld_flag(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if non-missing logical scalar using

\code{is.logical(x) && length(x) == 1L && !anyNA(x)}

\strong{Pass}: \code{TRUE}, \code{FALSE}.

\strong{Fail}: \code{logical(0)}, \code{c(TRUE, TRUE)}, \code{"TRUE"}, \code{1}, \code{NA}.

Do not confuse this function with \code{\link[=chk_lgl]{chk_lgl()}},
which also checks for logical scalars of \code{length(x) == 1}
but can include \code{NA}s.
}
\section{Functions}{
\itemize{
\item \code{vld_flag()}: Validate Flag

}}
\examples{
# chk_flag
chk_flag(TRUE)
try(vld_flag(1))
# vld_flag
vld_flag(TRUE)
vld_flag(1)
}
\seealso{
\code{\link[=is.logical]{is.logical()}} \code{\link[=length]{length()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other logical_checkers: 
\code{\link{chk_false}()},
\code{\link{chk_lgl}()},
\code{\link{chk_logical}()},
\code{\link{chk_true}()}

Other scalar_checkers: 
\code{\link{chk_count}()},
\code{\link{chk_date}()},
\code{\link{chk_date_time}()},
\code{\link{chk_false}()},
\code{\link{chk_lgl}()},
\code{\link{chk_scalar}()},
\code{\link{chk_string}()},
\code{\link{chk_true}()},
\code{\link{chk_tz}()}
}
\concept{logical_checkers}
\concept{scalar_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-false.R
\name{chk_false}
\alias{chk_false}
\alias{vld_false}
\title{Check FALSE}
\usage{
chk_false(x, x_name = NULL)

vld_false(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Check if FALSE using

\code{is.logical(x) && length(x) == 1L && !anyNA(x) && !x}
}
\section{Functions}{
\itemize{
\item \code{vld_false()}: Validate FALSE

}}
\examples{
# chk_false
chk_false(FALSE)
try(chk_false(0))
# vld_false
vld_false(TRUE)
vld_false(FALSE)
vld_false(NA)
vld_false(0)
vld_false(c(FALSE, FALSE))
}
\seealso{
\code{\link[=is.logical]{is.logical()}}

\code{\link[=length]{length()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other logical_checkers: 
\code{\link{chk_flag}()},
\code{\link{chk_lgl}()},
\code{\link{chk_logical}()},
\code{\link{chk_true}()}

Other scalar_checkers: 
\code{\link{chk_count}()},
\code{\link{chk_date}()},
\code{\link{chk_date_time}()},
\code{\link{chk_flag}()},
\code{\link{chk_lgl}()},
\code{\link{chk_scalar}()},
\code{\link{chk_string}()},
\code{\link{chk_true}()},
\code{\link{chk_tz}()}
}
\concept{logical_checkers}
\concept{scalar_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-whole-numeric.R
\name{chk_whole_numeric}
\alias{chk_whole_numeric}
\alias{vld_whole_numeric}
\title{Check Whole Numeric}
\usage{
chk_whole_numeric(x, x_name = NULL)

vld_whole_numeric(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if integer vector or double equivalent using

\code{is.integer(x) || (is.double(x) && vld_true(all.equal(x, as.integer(x))))}
}
\section{Functions}{
\itemize{
\item \code{vld_whole_numeric()}: Validate Whole Numeric

}}
\examples{
# chk_whole_numeric
chk_whole_numeric(1)
try(chk_whole_numeric(1.1))
# vld_whole_numeric
vld_whole_numeric(1)
vld_whole_numeric(NA_real_)
vld_whole_numeric(1:2)
vld_whole_numeric(double(0))
vld_whole_numeric(TRUE)
vld_whole_numeric(1.5)
}
\seealso{
\code{\link[=is.integer]{is.integer()}}

\code{\link[=is.double]{is.double()}}

\code{\link[=vld_true]{vld_true()}}

\code{\link[=all.equal]{all.equal()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other whole_number_checkers: 
\code{\link{chk_count}()},
\code{\link{chk_whole_number}()}
}
\concept{whole_number_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-orderset.R
\name{chk_orderset}
\alias{chk_orderset}
\title{Check Set Ordered}
\usage{
chk_orderset(x, values, x_name = NULL)
}
\arguments{
\item{x}{The object to check.}

\item{values}{A vector of the permitted values.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if the first occurrence of each shared element
in x is equivalent to the first occurrence of each shared element in values using
\code{vld_equivalent(unique(x[x \%in\% values]), values[values \%in\% x])}.
}
\examples{

# chk_orderset
chk_orderset(1:2, 1:2)
try(chk_orderset(2:1, 1:2))
}
\seealso{
\code{\link[=vld_equivalent]{vld_equivalent()}}

\code{\link[=unique]{unique()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other set_checkers: 
\code{\link{chk_not_subset}()},
\code{\link{chk_superset}()},
\code{\link{vld_not_subset}()},
\code{\link{vld_orderset}()}
}
\concept{set_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-gt.R
\name{chk_gt}
\alias{chk_gt}
\alias{vld_gt}
\title{Check Greater Than}
\usage{
chk_gt(x, value = 0, x_name = NULL)

vld_gt(x, value = 0)
}
\arguments{
\item{x}{The object to check.}

\item{value}{A non-missing scalar of a value.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if all non-missing values are greater than value using

\code{all(x[!is.na(x)] > value)}
}
\section{Functions}{
\itemize{
\item \code{vld_gt()}: Validate Greater Than

}}
\examples{
# chk_gt
chk_gt(0.1)
try(chk_gt(c(0.1, -0.2)))
# vld_gt
vld_gt(numeric(0))
vld_gt(0)
vld_gt(0.1)
vld_gt(c(0.1, 0.2, NA))
vld_gt(c(0.1, -0.2))
vld_gt(c(-0.1, 0.2), value = -1)
vld_gt("b", value = "a")
}
\seealso{
\code{\link[=all]{all()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other range_checkers: 
\code{\link{chk_gte}()},
\code{\link{chk_lt}()},
\code{\link{chk_lte}()},
\code{\link{chk_range}()}
}
\concept{range_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-sorted.R
\name{chk_sorted}
\alias{chk_sorted}
\alias{vld_sorted}
\title{Check Sorted}
\usage{
chk_sorted(x, x_name = NULL)

vld_sorted(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if is sorted using

\code{is.unsorted(x, na.rm = TRUE)}
}
\section{Functions}{
\itemize{
\item \code{vld_sorted()}: Validate Sorted

}}
\examples{
# chk_sorted
chk_sorted(1:2)
try(chk_sorted(2:1))
# vld_sorted
vld_sorted(1:2)
vld_sorted(2:1)
}
\seealso{
\code{\link[=is.unsorted]{is.unsorted()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.
}
\concept{sorted_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-length.R
\name{chk_length}
\alias{chk_length}
\alias{vld_length}
\title{Check Length}
\usage{
chk_length(x, length = 1L, upper = length, x_name = NULL)

vld_length(x, length = 1L, upper = length)
}
\arguments{
\item{x}{The object to check.}

\item{length}{A count of the length.}

\item{upper}{A count of the max length.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks length is a particular value or range using

\code{length(x) >= length && length(x) <= upper}
}
\section{Functions}{
\itemize{
\item \code{vld_length()}: Validate Length

}}
\examples{
# chk_length
chk_length("text")
try(vld_length("text", length = 2))
# vld_length
vld_length(2:1, 2)
vld_length(2:1, 1)
}
\seealso{
\code{\link[=length]{length()}}, \code{\link[=check_length]{check_length()}}, \code{\link[=check_dim]{check_dim()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other length_checkers: 
\code{\link{chk_compatible_lengths}()}
}
\concept{length_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-count.R
\name{chk_count}
\alias{chk_count}
\alias{vld_count}
\title{Check Count}
\usage{
chk_count(x, x_name = NULL)

vld_count(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if non-negative whole number using

\code{vld_whole_number(x) && x >= 0}
}
\section{Functions}{
\itemize{
\item \code{vld_count()}: Validate Count

}}
\examples{
# chk_count
chk_count(1)
try(chk_count(1.5))
# vld_count
vld_count(1)
vld_count(0L)
vld_count(-1)
vld_count(0.5)
}
\seealso{
\code{\link[=vld_whole_number]{vld_whole_number()}}

\code{\link[=length]{length()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other scalar_checkers: 
\code{\link{chk_date}()},
\code{\link{chk_date_time}()},
\code{\link{chk_false}()},
\code{\link{chk_flag}()},
\code{\link{chk_lgl}()},
\code{\link{chk_scalar}()},
\code{\link{chk_string}()},
\code{\link{chk_true}()},
\code{\link{chk_tz}()}

Other whole_number_checkers: 
\code{\link{chk_whole_number}()},
\code{\link{chk_whole_numeric}()}
}
\concept{scalar_checkers}
\concept{whole_number_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-tz.R
\name{chk_tz}
\alias{chk_tz}
\alias{vld_tz}
\title{Check Time Zone}
\usage{
chk_tz(x, x_name = NULL)

vld_tz(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if non-missing valid scalar timezone using

\code{is.character(x) && length(x) == 1L && !anyNA(x) && x \%in\% OlsonNames()}
}
\section{Functions}{
\itemize{
\item \code{vld_tz()}: Validate Time Zone

}}
\examples{
chk_tz("UTC")
try(chk_tz("TCU"))
vld_tz("UTC")
vld_tz("TCU")
}
\seealso{
\code{\link[=length]{length()}}

\code{\link[=OlsonNames]{OlsonNames()}}

\code{\link[=is.character]{is.character()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other scalar_checkers: 
\code{\link{chk_count}()},
\code{\link{chk_date}()},
\code{\link{chk_date_time}()},
\code{\link{chk_false}()},
\code{\link{chk_flag}()},
\code{\link{chk_lgl}()},
\code{\link{chk_scalar}()},
\code{\link{chk_string}()},
\code{\link{chk_true}()}
}
\concept{date_checkers}
\concept{scalar_checkers}
\concept{tz_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-factor.R
\name{chk_factor}
\alias{chk_factor}
\alias{vld_factor}
\title{Check Factor}
\usage{
chk_factor(x, x_name = NULL)

vld_factor(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if factor using

\code{is.factor(x)}
}
\section{Functions}{
\itemize{
\item \code{vld_factor()}: Validate Factor

}}
\examples{
# chk_factor
chk_factor(factor("1"))
try(chk_factor("1"))
# vld_factor
vld_factor(factor("1"))
vld_factor(factor(0))
vld_factor("1")
vld_factor(1L)
}
\seealso{
\code{\link[=is.factor]{is.factor()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other factor_checkers: 
\code{\link{chk_character_or_factor}()}
}
\concept{data-type_checkers}
\concept{factor_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-all.R
\name{chk_all}
\alias{chk_all}
\alias{vld_all}
\title{Check All}
\usage{
chk_all(x, chk_fun, ..., x_name = NULL)

vld_all(x, vld_fun, ...)
}
\arguments{
\item{x}{The object to check.}

\item{chk_fun}{A chk_ function.}

\item{...}{Additional arguments.}

\item{x_name}{A string of the name of object x or NULL.}

\item{vld_fun}{A vld_ function.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks all elements using

\code{all(vapply(x, chk_fun, TRUE, ...))}
}
\section{Functions}{
\itemize{
\item \code{vld_all()}: Validate All

}}
\examples{
# chk_all
chk_all(TRUE, chk_lgl)
# FIXME try(chk_all(1, chk_lgl))
chk_all(c(TRUE, NA), chk_lgl)
# vld_all
vld_all(c(TRUE, NA), vld_lgl)
}
\seealso{
\code{\link[=all]{all()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other all_checkers: 
\code{\link{chk_all_equal}()},
\code{\link{chk_all_equivalent}()},
\code{\link{chk_all_identical}()}
}
\concept{all_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chkor.R
\name{chkor}
\alias{chkor}
\title{Check OR}
\usage{
chkor(...)
}
\arguments{
\item{...}{Multiple \code{chk_} functions.}
}
\value{
An informative error if the test fails.
}
\description{
The \code{chkor()} function has been deprecated for the faster \code{chkor_vld()}.
}
\details{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}
}
\examples{
chkor()
chkor(chk_flag(TRUE))
try(chkor(chk_flag(1)))
try(chkor(chk_flag(1), chk_flag(2)))
chkor(chk_flag(1), chk_flag(TRUE))
}
\seealso{
\code{\link[=chk_null_or]{chk_null_or()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-function.R
\name{chk_function}
\alias{chk_function}
\alias{vld_function}
\title{Check Function}
\usage{
chk_function(x, formals = NULL, x_name = NULL)

vld_function(x, formals = NULL)
}
\arguments{
\item{x}{The object to check.}

\item{formals}{A count of the number of formal arguments.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if is a function using

\code{is.function(x) && (is.null(formals) || length(formals(x)) == formals)}
}
\section{Functions}{
\itemize{
\item \code{vld_function()}: Validate Function

}}
\examples{
# chk_function
chk_function(mean)
try(chk_function(1))
# vld_function
vld_function(mean)
vld_function(function(x) x)
vld_function(1)
vld_function(list(1))
}
\seealso{
\code{\link[=is.function]{is.function()}} \code{\link[=formals]{formals()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other missing_checkers: 
\code{\link{chk_missing}()},
\code{\link{chk_not_missing}()}
}
\concept{ellpisis_checkers}
\concept{function_checkers}
\concept{missing_checkers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{chk_deprecated}
\alias{chk_deprecated}
\alias{chk_dirs}
\alias{chk_files}
\alias{chk_has}
\alias{chk_in}
\alias{chk_no_missing}
\alias{vld_no_missing}
\alias{chk_off}
\alias{chk_on}
\alias{is_chk_on}
\alias{chk_proportion}
\alias{deparse_backtick}
\title{Deprecated functions}
\usage{
chk_dirs(x)

chk_files(x)

chk_has(x, values, x_name = NULL)

chk_in(x, values, x_name = NULL)

chk_no_missing(x, x_name = NULL)

vld_no_missing(x)

chk_off()

chk_on()

is_chk_on()

chk_proportion(x, x_name = NULL)

deparse_backtick(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\description{
Deprecated \code{chk_()} functions.
}
\section{Functions}{
\itemize{
\item \code{chk_dirs()}: Check Directories Exist

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

Replace with \verb{[chk_all](x, [chk_dir])}

\item \code{chk_files()}: Check Files Exist

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

Replace with \verb{[chk_all](x, [chk_file])}

\item \code{chk_has()}: Check Has

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

Replace by \code{\link[=chk_superset]{chk_superset()}}

\item \code{chk_in()}: Check In

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

Replace by \code{\link[=chk_subset]{chk_subset()}}

\item \code{chk_no_missing()}: Check No Missing Values

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

Replace with \code{\link[=chk_not_any_na]{chk_not_any_na()}}

\item \code{vld_no_missing()}: Validate No Missing Values

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

Replace with \code{\link[=vld_not_any_na]{vld_not_any_na()}}

\item \code{chk_off()}: Turns checking off

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

This approach is no longer recommended

\item \code{chk_on()}: Turns checking on

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

This approach is no longer recommended

\item \code{is_chk_on()}: Tests checking on

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

This approach is no longer recommended

\item \code{chk_proportion()}: Check Proportion

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

Replace by \verb{[chk_number](x); [chk_range](x)}

\item \code{deparse_backtick()}: Deparse Backtick

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

Replace with \code{\link[=deparse_backtick_chk]{deparse_backtick_chk()}}

}}
\seealso{
Other deprecated: 
\code{\link{chk_chr}()},
\code{\link{chk_dbl}()},
\code{\link{chk_wnum}()}
}
\concept{deprecated}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa.R
\name{aaa}
\alias{aaa}
\title{Workaround: Avoid backtraces in examples}
\description{
This example will run first and set the option for the process that builds
the example.
(By default, pkgdown builds in a separate process.)
This also produces a help page that isn't linked from anywhere.
}
\examples{
options(rlang_backtrace_on_error = "none")
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check-data.R
\name{check_data}
\alias{check_data}
\title{Check Data}
\usage{
check_data(
  x,
  values = NULL,
  exclusive = FALSE,
  order = FALSE,
  nrow = numeric(0),
  key = character(0),
  x_name = NULL
)
}
\arguments{
\item{x}{The object to check.}

\item{values}{A uniquely named list of atomic vectors of the column values.}

\item{exclusive}{A flag specifying whether x must only include columns named in values.}

\item{order}{A flag specifying whether the order of columns in x must match names in values.}

\item{nrow}{A flag or a whole numeric vector of the value, value range or possible values.}

\item{key}{A character vector of the columns that represent a unique key.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
An informative error if the test fails or an invisible copy of x.
}
\description{
Checks column names, values, number of rows and key for a data.frame.
}
\examples{
check_data(data.frame())
check_data(data.frame(x = 2), list(x = 1))
try(check_data(data.frame(x = 2), list(y = 1L)))
try(check_data(data.frame(x = 2), list(y = 1)))
try(check_data(data.frame(x = 2), nrow = 2))
}
\seealso{
Other check: 
\code{\link{check_dim}()},
\code{\link{check_dirs}()},
\code{\link{check_files}()},
\code{\link{check_key}()},
\code{\link{check_length}()},
\code{\link{check_names}()},
\code{\link{check_values}()}
}
\concept{check}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chk-true.R
\name{chk_true}
\alias{chk_true}
\alias{vld_true}
\title{Check TRUE}
\usage{
chk_true(x, x_name = NULL)

vld_true(x)
}
\arguments{
\item{x}{The object to check.}

\item{x_name}{A string of the name of object x or NULL.}
}
\value{
The \code{chk_} function throws an informative error if the test fails or
returns the original object if successful so it can used in pipes.

The \code{vld_} function returns a flag indicating whether the test was met.
}
\description{
Checks if TRUE using

\code{is.logical(x) && length(x) == 1L && !anyNA(x) && x}
}
\section{Functions}{
\itemize{
\item \code{vld_true()}: Validate TRUE

}}
\examples{
# chk_true
chk_true(TRUE)
try(chk_true(1))
# vld_true
vld_true(TRUE)
vld_true(FALSE)
vld_true(NA)
vld_true(0)
vld_true(c(TRUE, TRUE))
}
\seealso{
\code{\link[=is.logical]{is.logical()}}

\code{\link[=length]{length()}}

For more details about the use of this function,
please read the article
\code{vignette("chk-families")}.

Other logical_checkers: 
\code{\link{chk_false}()},
\code{\link{chk_flag}()},
\code{\link{chk_lgl}()},
\code{\link{chk_logical}()}

Other scalar_checkers: 
\code{\link{chk_count}()},
\code{\link{chk_date}()},
\code{\link{chk_date_time}()},
\code{\link{chk_false}()},
\code{\link{chk_flag}()},
\code{\link{chk_lgl}()},
\code{\link{chk_scalar}()},
\code{\link{chk_string}()},
\code{\link{chk_tz}()}
}
\concept{logical_checkers}
\concept{scalar_checkers}
